<?php
/**
 * @package    HikaSerial for Joomla!
 * @version    4.0.2
 * @author     Obsidev S.A.R.L.
 * @copyright  (C) 2011-2023 OBSIDEV. All rights reserved.
 * @license    GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined('_JEXEC') or die('Restricted access');
?><?php
class hikaserialCartClass extends hikaserialClass {
	private static $current_cart_id = array(
		'subscription' => 0
	);

	public $tables = array('shop.cart_product', 'shop.cart');
	public $pkeys = array('cart_id', 'cart_id');

	protected $db = null;
	protected $app = null;
	protected $config = null;
	protected $user = null;
	protected $cartClass = null;
	protected $serialConfig = null;

	public function  __construct($config = array()) {
		$ret = parent::__construct($config);

		$this->db = JFactory::getDBO();
		$this->app = JFactory::getApplication();
		$this->config = hikashop_config();
		$this->serialConfig = hikaserial::config();
		$this->user = hikashop_loadUser(true);

		return $ret;
	}

	public function getCurrentCartId($type = 'cart') {
		if(hikaserial::isAdmin())
			return false;

		$type = strtolower($type);

		if(in_array($type, array('cart', 'wishlist'))) {
			if(empty($this->cartClass))
				$this->cartClass = hikaserial::get('shop.class.cart');
			return $this->cartClass->getCurrentCartId($type);
		}

		if($type != 'subscription')
			return false;

		if($type == 'subscription' && empty($this->user))
			return false;

		if(self::$current_cart_id[$type] > 0)
			return self::$current_cart_id[$type];

		$jsession = JFactory::getSession();
		$filters = array(
			'type' => 'cart.cart_type = ' . $this->db->Quote($type),
			'session' => 'cart.session_id = ' . $this->db->Quote($jsession->getId())
		);

		if((int)@$this->user->user_id > 0)
			$filters['session'] .= ' OR cart.user_id = ' . (int)$this->user->user_id;

		$query = 'SELECT cart.cart_id FROM #__hikashop_cart AS cart ' .
			' WHERE ('.implode(') AND (', $filters) . ') '.
			' ORDER BY cart.cart_current DESC, cart.cart_modified DESC';
		$this->db->setQuery($query);
		$cart_id = $this->db->loadResult();

		if(empty($cart_id) || (int)$cart_id == 0)
			return 0;
		$element = (int)$cart_id;

		self::$current_cart_id[$type] = $element;

		return self::$current_cart_id[$type];
	}

	public function get($element, $default = null) {
		if($element === 'reset_cache') {
			if(empty($this->cartClass))
				$this->cartClass = hikaserial::get('shop.class.cart');
			return $this->cartClass->get($element, $default);
		}

		if((int)$element == 0)
			$element = $this->getCurrentCartId('subscription');
		if($element === false)
			return false;
		if($element == 0)
			return null;
		if(empty($this->cartClass))
			$this->cartClass = hikaserial::get('shop.class.cart');
		return $this->cartClass->get($element, $default);
	}


	public function &getFullCart($cart_id = 0) {
		if((int)$element == 0)
			$element = $this->getCurrentCartId('subscription');
		if($element === false)
			return false;
		if($element == 0)
			return null;
		if(empty($this->cartClass))
			$this->cartClass = hikaserial::get('shop.class.cart');
		return $this->cartClass->getFullCart($cart_id);
	}

	private function createNewSubscriptionCart() {
		$element = new stdClass();
		$element->cart_type = 'subscription';
		$element->cart_modified = time();
		$element->cart_currency_id = hikashop_getCurrency();

		if(!hikaserial::isAdmin()) {
			$jsession = JFactory::getSession();

			$element->user_id = !empty($this->user) ? (int)$this->user->user_id : 0;
			$element->session_id = $jsession->getId();
			$element->cart_current = 1;
		}
		$status = parent::save($element);
		if(!$status)
			return $status;

		$element->cart_id = $status;
		return $element;
	}

	protected function getOrCreateSubscriptionCart() {
		$cart_id = $this->getCurrentCartId('subscription');
		if($cart_id == 0) {
			$cart = $this->createNewSubscriptionCart();
			if(empty($cart))
				return false;
			return (int)$cart->cart_id;
		}

		$cart = $this->get($cart_id);
		if(empty($cart) || $cart->cart_type != 'subscription')
			return false;

		$query = 'DELETE FROM ' . hikashop_table('cart_product') . ' WHERE (cart_id = ' . (int)$cart_id . ')';
		$this->db->setQuery($query);
		$this->db->execute();

		if(isset($cart->cart_params->subscription)) {
			unset($cart->cart_params->subscription);
			$query = 'UPDATE '.hikashop_table('cart').' SET cart_params = '.$this->db->Quote(json_encode($cart->cart_params)).' WHERE cart_id = '.(int)$cart_id;
			$this->db->setQuery($query);
			$this->db->execute();
		}

		$this->get('reset_cache', $cart_id);
		return $cart_id;
	}

	protected function initCartSubscriptionParams(&$cart) {
		if(empty($cart->cart_params))
			$cart->cart_params = new stdClass();
		if(empty($cart->cart_params->subscription))
			$cart->cart_params->subscription = array();

		if(is_object($cart->cart_params->subscription)) {
			$sub = array();
			foreach($cart->cart_params->subscription as $k => $v) {
				$sub[$k] = $v;
			}
			$cart->cart_params->subscription = $sub;
		}
	}

	public function addRenewSubscription($subscription_id, $product_id) {
		$cart_id = $this->getOrCreateSubscriptionCart();
		if(empty($cart_id))
			return false;

		if(empty($this->cartClass))
			$this->cartClass = hikaserial::get('shop.class.cart');
		$status = $this->cartClass->addProduct($cart_id, array(
			array('id' => $product_id, 'qty' => 1)
		));

		if($status === false)
			return false;

		$cart = $this->get($status);
		$this->initCartSubscriptionParams($cart);

		foreach($cart->cart_products as $cart_product) {
			if((int)$cart_product->product_id != $product_id)
				continue;

			if(empty($cart->cart_params->subscription[$subscription_id]))
				$cart->cart_params->subscription[$subscription_id] = array();
			$cart->cart_params->subscription[$subscription_id][] = (int)$cart_product->cart_product_id;
		}

		return $this->cartClass->save($cart);
	}

	public function addUpgradeSubscription($subscription_id, $product_id) {
		$cart_id = $this->getOrCreateSubscriptionCart();
		if(empty($cart_id))
			return false;

		if(empty($this->cartClass))
			$this->cartClass = hikaserial::get('shop.class.cart');
		$status = $this->cartClass->addProduct($cart_id, array(
			array('id' => $product_id, 'qty' => 1)
		));

		if($status === false)
			return false;

		$cart = $this->get($status);
		$this->initCartSubscriptionParams($cart);

		foreach($cart->cart_products as $cart_product) {
			if((int)$cart_product->product_id != $product_id)
				continue;

			if(empty($cart->cart_params->subscription[$subscription_id]))
				$cart->cart_params->subscription[$subscription_id] = array();
			$cart->cart_params->subscription[$subscription_id][] = (int)$cart_product->cart_product_id;
		}

		return $this->cartClass->save($cart);
	}

	public function checkPaymentOptions(&$paymentOptions, &$order) {
		if(empty($order->products) && empty($order->cart->full_products))
			return;

		$config = hikaserial::config();
		if($config->get('no_recurring', 0))
			return;

		$order_products = isset($order->products) ? $order->products : $order->cart->full_products;

		$product_ids = array();
		foreach($order_products as $product) {
			if(empty($product->cart_product_quantity) && empty($product->order_product_quantity))
				continue;

			$product_ids[ (int)$product->product_id ] = (int)$product->product_id;
			if(!empty($product->product_parent_id) && $product->product_type == 'variant')
				$product_ids[ (int)$product->product_parent_id ] = (int)$product->product_parent_id;
		}
		if(empty($product_ids)) {
			unset($order_products);
			return;
		}

		$query = 'SELECT pp.*, pack.* '.
			' FROM ' . hikaserial::table('product_pack') . ' as pp '.
			' INNER JOIN ' . hikaserial::table('pack') . ' as pack ON pp.pack_id = pack.pack_id '.
			' WHERE pack.pack_published = 1 AND pack.pack_type = '.$this->db->Quote('subscription').
				' AND pp.relation_type IN ('.$this->db->Quote('create_renew').','.$this->db->Quote('renew_create').') AND pp.product_id IN (' . implode(',', $product_ids) . ')'.
			' ORDER BY pp.pack_id ASC';
		$this->db->setQuery($query);
		$plans = $this->db->loadObjectList();

		if(empty($plans)) {
			unset($order_products);
			return;
		}

		$products_plans = array();
		$used_plans = array();
		$fullcart = true;
		foreach($order_products as $k => $product) {
			if(!is_numeric($k) || (empty($product->cart_product_quantity) && empty($product->order_product_quantity)))
				continue;

			$qty = !empty($product->cart_product_quantity) ? (int)$product->cart_product_quantity : (int)$product->order_product_quantity;

			foreach($plans as $plan) {
				if((int)$product->product_id != (int)$plan->product_id && (empty($product->product_parent_id) || (int)$product->product_parent_id != (int)$plan->product_id))
					continue;

				if((int)$product->product_id == (int)$plan->product_id) {
					if(empty($products_plans[$k]))
						$products_plans[$k] = array();
					if(empty($products_plans[$k]['main']))
						$products_plans[$k]['main'] = array();

					$p = clone($plan);
					$p->order_product_quantity = (int)$qty;
					$products_plans[$k]['main'][] = $p;
				}
				if((int)$product->product_parent_id == (int)$plan->product_id) {
					if(empty($products_plans[$k]))
						$products_plans[$k] = array();
					if(empty($products_plans[$k]['parent']))
						$products_plans[$k]['parent'] = array();

					$p = clone($plan);
					$p->order_product_quantity = (int)$qty;
					$products_plans[$k]['parent'][] = $p;
				}
			}

			if(empty($products_plans[$k]))
				$fullcart = false;
		}

		if(!empty($order->order_payment_price) || !empty($order->order_discount_price) || !empty($order->order_shipping_price))
			$fullcart = false;
		if( isset($order->cart) && (!empty($order->cart->order_payment_price) || !empty($order->cart->order_discount_price) || !empty($order->cart->order_shipping_price)))
			$fullcart = false;

		foreach($products_plans as $k => &$product_plans) {
			if(!empty($product_plans['main'])) {
				$product_plans = $product_plans['main'];
			} else {
				$product_plans = $product_plans['parent'];
			}
			$used_plans = array_merge($used_plans, $product_plans);
		}
		unset($product_plans);

		$duration = '';
		foreach($used_plans as $plan) {
			if(!in_array($plan->relation_type, array('create_renew', 'renew_create')))
				continue;

			$plan_duration = $plan->relation_duration;

			if($plan->relation_type == 'renew_create' && $plan->order_product_quantity > 1) {
				$unit = ltrim($plan_duration, '0123456789');
				$time = (int)$plan_duration;
				$plan_duration = $time * $plan->order_product_quantity . $unit;
			}

			if(empty($duration)) {
				$duration = $plan_duration;
				continue;
			}
			if($duration != $plan_duration) {
				$duration = false;
				break;
			}
		}

		if($duration === false) {
			if($config->get('subscription_display_duration_error', 0))
				$app->enqueueMessage(JText::_('SUBSCRIPTION_DIFFERENT_DURATIONS'), 'error');
			return;
		}

		if(empty($duration)) {
			unset($order_products);
			return;
		}

		$paymentOptions['recurring'] = array(
			'duration' => $duration
		);

		if($config->get('recurring_optional', 1))
			$paymentOptions['recurring']['optional'] = true;

		if($fullcart) {
			unset($order_products);
			return;
		}

		$total = 0.0;
		$processed_product_ids = array_keys($products_plans);

		unset($products_plans);
		unset($used_plans);
		unset($plans);

		foreach($processed_product_ids as $k) {
			$p =& $order_products[$k];

			$qty = isset($p->cart_product_quantity) ? (int)$p->cart_product_quantity : (int)$p->order_product_quantity;

			$price = reset($p->prices);
			$total += $price->price_value_with_tax;

			unset($p);
		}

		unset($order_products);

		$paymentOptions['recurring']['value'] = $total;
		$paymentOptions['recurring']['products'] = $processed_product_ids;
	}

	public function onAfterCartProductsLoad(&$cart) {
		if(empty($cart->cart_params) || empty($cart->cart_params->subscription))
			return;
		$subscriptions = (array)$cart->cart_params->subscription;

		if(isset($subscriptions[0]))
			unset($subscriptions[0]);

		$ids = array_keys($subscriptions);
		hikaserial::toInteger($ids);
		if(empty($ids))
			return;
		$query = 'SELECT subscription_id, subscription_data FROM ' . hikaserial::table('subscription') .
			' WHERE subscription_id IN ('.implode(',', $ids).')';
		$this->db->setQuery($query);
		$subs = $this->db->loadObjectList('subscription_id');
		if(empty($subs))
			return;

		foreach($cart->products as $pid => &$cart_product) {
			foreach($subscriptions as $sub_id => $sub_products) {
				if(!in_array($pid, $sub_products))
					continue;
				if(empty($cart_product->extraData))
					$cart_product->extraData = new stdClass();
				$txt = JText::sprintf('PRODUCT_CART_SUBCRIPTION', $subs[$sub_id]->subscription_data);

				if(empty($cart_product->extraData->checkout))
					$cart_product->extraData->checkout = array();
				$cart_product->extraData->checkout['subscription'] = $txt;
				if(empty($cart_product->extraData->cart))
					$cart_product->extraData->cart = array();
				$cart_product->extraData->cart['subscription'] = $txt;
			}
		}
	}

	public function onAfterCartSaveWithSub(&$cart) {
		$extras = array();
		$times = array();
		foreach($cart->cart_products as $k => $cp) {
			if(empty($cp->extra) || empty($cp->extra['sub']))
				continue;
			$extras[$k] = clone $cp;
			$times[(int)$cp->cart_product_modified] = (int)$cp->cart_product_modified;
		}
		if(empty($extras))
			return;

		$query = 'SELECT * FROM '.hikaserial::table('shop.cart_product').
			' WHERE cart_id = ' . (int)$cart->cart_id . ' AND cart_product_modified IN ('.implode(',', $times).')';
		$this->db->setQuery($query);
		$db_cart_products = $this->db->loadObjectList('cart_product_id');

		if(empty($db_cart_products))
			return;

		$cart_params = isset($cart->cart_params) ? $cart->cart_params : $cart->old->cart_params;
		if(is_string($cart_params))
			$cart_params = json_decode($cart_params);

		if(empty($cart_params->subscription))
			$cart_params->subscription = array();
		else if(is_object($cart_params->subscription))
			$cart_params->subscription = (array)$cart_params->subscription;

		foreach($extras as $k => &$cp) {
			$f = false;
			foreach($db_cart_products as $v => $dbcp) {
				if(!empty($dbcp->_marked))
					continue;

				if((int)$dbcp->product_id != (int)$cp->product_id)
					continue;
				if((int)$dbcp->cart_product_modified != (int)$cp->cart_product_modified)
					continue;
				if((int)$dbcp->cart_product_quantity != (int)$cp->cart_product_quantity)
					continue;

				$cp->cart_product_id = (int)$dbcp->cart_product_id;
				$db_cart_products[$v]->_marked = true;
				$f = true;
				break;
			}

			if(!$f)
				continue;

			foreach($cp->extra['sub'] as $plan_id => $subscription_id) {
				if(empty($cart_params->subscription[(int)$subscription_id]))
					$cart_params->subscription[(int)$subscription_id] = array();
				$cart_params->subscription[(int)$subscription_id][] = $cp->cart_product_id;
			}
		}
		unset($cp);

		$query = 'UPDATE ' . hikaserial::table('shop.cart') .
			' SET cart_params = ' . $this->db->Quote(json_encode($cart_params)) .
			' WHERE cart_id = ' . (int)$cart->cart_id;
		$this->db->setQuery($query);
		$this->db->execute();

		$cart->cart_params = $cart_params;
	}

	public function onCompareCartProducts($product, $cart_product, &$do) {
		if(!empty($product['extra']['sub']))
			$do = false;
	}

	public function onProductCheckQuantities(&$products, &$cart, $options) {
		if(!empty($cart->cart_type) && $cart->cart_type == 'subscription')
			return;

		$pids = array();
		$subs = array();
		foreach($products as $p) {
			$pid = !empty($p['id']) ? (int)$p['id'] : (int)@$p['data']->product_id;
			if(empty($pid))
				continue;
			$pids[ $pid ] = (int)$pid;
			if(!empty($p['data']) && !empty($p['data']->product_parent_id)) {
				$i = (int)$p['data']->product_parent_id;
				$pids[$i] = $i;
			}
			if(isset($p['extra']['sub'])) {
				foreach($p['extra']['sub'] as $plan_id => $sub_id) {
					if($sub_id > 0)
						$subs[$sub_id] = $sub_id;
				}
			}
		}
		unset($pids[0]);
		if(count($pids) == 1)
			$pids = reset($pids);

		$plans = $this->getRelatedPlans($pids);

		if(empty($plans) && empty($subs))
			return;

		if(!empty($subs)) {
			$renew = array();
			$upgrade = array();
			foreach($plans as $plan) {
				if(in_array($plan->relation_type, array('create', 'create_renew', 'renew_create')))
					$renew[(int)$plan->pack_id] = (int)$plan->pack_id;
				if($plan->relation_type == 'upgrade' && !empty($plan->relation_upgrade))
					$upgrade[(int)$plan->relation_upgrade] = (int)$plan->relation_upgrade;
			}

			$subscriptionClass = hikaserial::get('class.subscription');
			$renewals = array();
			if(!empty($renew))
				$renewals = $subscriptionClass->listRenewableSubscriptions( $renew );
			$upgrades = array();
			if(!empty($upgrade))
				$upgrades = $subscriptionClass->listUpgradableSubscriptions( $upgrade );
		}

		$plan_ids = array();
		foreach($plans as $plan) {
			if($plan->relation_type == 'upgrade')
				$plan_ids[(int)$plan->relation_upgrade] = (int)$plan->relation_upgrade;
			else
				$plan_ids[(int)$plan->pack_id] = (int)$plan->pack_id;
		}

		$cartClass = hikaserial::get('shop.class.cart');
		foreach($products as &$p) {
			$pid = !empty($p['id']) ? (int)$p['id'] : (int)@$p['data']->product_id;

			$packs = array();
			foreach($plans as $i => $plan) {
				if($plan->product_id != $pid)
					continue;
				if(in_array($plan->relation_type, array('create', 'create_renew', 'renew_create', 'upgrade')))
					continue;
				$packs[] = $plan->pack_id;
			}

			$err = false;
			if(isset($p['extra']['sub'])) {
				foreach($p['extra']['sub'] as $plan_id => $sub_id) {
					$valid_sub = empty($sub_id) || (isset($renew[$plan_id]) && isset($renewals[$sub_id])) || (isset($upgrade[$plan_id]) && isset($upgrades[$sub_id]));
					if(isset($plan_ids[$plan_id]) && $valid_sub)
						continue;

					unset($p['extra']['sub'][$plan_id]);
					$err = true;
				}
			}

			if(empty($packs) && !$err)
				continue;


			$p['qty'] = 0;
			$cartClass->addMessage($cart, array(
				'msg' => JText::sprintf('NO_SUBSCRIPTION_DATA_FOR_PRODUCT', $p['data']->product_name),
				'product_id' => $pid,
				'type' => 'notice'
			));
		}
	}

	public function onCheckCartQuantities(&$cart, $parent_products, &$ret) {
		if($cart->cart_type == 'subscription') {
			$this->fallbackCheckCart($cart, $parent_products, $ret);
			return;
		}

		if(empty($cart->products)) {
			if(!empty($cart->cart_params->subscription))
				unset($cart->cart_params->subscription);
			return;
		}

		$product_ids = array();
		foreach($cart->products as $cart_product_id => &$product) {
			if(empty($product->product_id))
				continue;
			if(!is_numeric($cart_product_id) && substr($cart_product_id, 0, 1) == 'p')
				continue;
			$product_ids[ (int)$product->product_id ] = (int)$product->product_id;
			if(!empty($product->product_parent_id))
				$product_ids[ (int)$product->product_parent_id ] = (int)$product->product_parent_id;
		}

		$plans = $this->getRelatedPlans($product_ids);

		if(empty($plans)) {
			if(!empty($cart->cart_params->subscription))
				unset($cart->cart_params->subscription);
			return;
		}
		$plan_products = array();
		if(!empty($cart->cart_params->subscription)) {
			foreach($cart->cart_params->subscription as $subscription_id => $products) {
				$plan_products = array_merge($plan_products, $products);
			}
		}

		foreach($cart->cart_products as $pid => $cart_product) {
			if(in_array($pid, $plan_products))
				continue;
			$f = false;
			foreach($plans as $plan) {
				if((int)$plan->product_id != $cart_product->product_id)
					continue;
				if(in_array($plan->relation_type, array('create', 'create_renew', 'renew_create')))
					continue;

				$f = true;
				break;
			}
			if(!$f)
				continue;

			$cart->cart_products[$pid]->cart_product_quantity = 0;
			$cart->products[$pid]->cart_product_quantity = 0;
			if(empty($cartClass))
				$cartClass = hikaserial::get('shop.class.cart');
			$cartClass->addMessage($cart, array(
				'msg' => JText::sprintf('NO_SUBSCRIPTION_DATA_FOR_PRODUCT', $cart->products[$pid]->product_name),
				'product_id' => $cart_product->product_id,
				'type' => 'error'
			));
		}
	}

	protected function getRelatedPlans($product_id) {
		if(is_array($product_id))
			hikaserial::toInteger($product_id);

		$relations = array(
			$this->db->Quote('upgrade'),
			$this->db->Quote('renew'),
			$this->db->Quote('renew_create'),
		);

		$query = 'SELECT pp.*, pack.* '.
			' FROM ' . hikaserial::table('product_pack') . ' as pp '.
			' INNER JOIN ' . hikaserial::table('pack') . ' as pack ON pp.pack_id = pack.pack_id '.
			' WHERE pack.pack_published = 1 AND pack.pack_type = '.$this->db->Quote('subscription').
				' AND pp.relation_type IN ('.implode(',', $relations).') '.
				' AND pp.product_id ' . (is_array($product_id) ? ('IN (' . implode(',', $product_id) . ')') : (' = '.$product_id)).
			' ORDER BY pp.pack_id ASC';
		$this->db->setQuery($query);
		return $this->db->loadObjectList();
	}

	protected function fallbackCheckCart(&$cart, $parent_products, &$ret) {
		$userClass = null;
		$notUsable = array();
		$errorMessagesProductNames = array();
		foreach($cart->products as $cart_product_id => &$product) {
			if(empty($product->product_id))
				continue;
			if(!is_numeric($cart_product_id) && substr($cart_product_id, 0, 1) == 'p')
				continue;

			$parent_product = new stdClass();

			if(!isset($product->old))
				$product->old = new stdClass();

			if((int)$product->cart_product_quantity <= 0)
				$product->cart_product_quantity = 0;
			$product->old->quantity = (int)$product->cart_product_quantity;
			if(empty($product->cart_product_quantity) && empty($product->old->quantity))
				continue;

			if((int)$product->product_parent_id > 0 && $product->product_type == 'variant' && isset($parent_products[ (int)$product->product_parent_id ]))
				$errorMessagesProductNames[$cart_product_id] = trim($parent_products[ (int)$product->product_parent_id ]->product_name . ' ' . $product->product_name);
			else
				$errorMessagesProductNames[$cart_product_id] = $product->product_name;

			if(empty($product->product_published)) {
				$notUsable[$cart_product_id] = array('id' => $cart_product_id, 'qty' => 0);
				$cart->messages[] = array('msg' => JText::sprintf('PRODUCT_NOT_AVAILABLE', $errorMessagesProductNames[$cart_product_id]), 'product_id' => $product->product_id, 'type' => 'notice');
				continue;
			}

			if((int)$product->product_quantity != -1 && (int)$product->product_quantity < (int)$product->cart_product_quantity) {
				$notUsable[$cart_product_id] = array('id' => $cart_product_id, 'qty' => 0);
				$cart->messages[] = array('msg' => JText::sprintf('NOT_ENOUGH_STOCK_FOR_PRODUCT', $errorMessagesProductNames[$cart_product_id]), 'product_id' => $product->product_id, 'type' => 'notice');
				continue;
			}

			if(isset($product->bundle_quantity) && (int)$product->bundle_quantity < (int)$product->cart_product_quantity) {
				$notUsable[$cart_product_id] = array('id' => $cart_product_id, 'qty' => 0);
				$cart->messages[] = array('msg' => JText::sprintf('NOT_ENOUGH_STOCK_FOR_PRODUCT', $errorMessagesProductNames[$cart_product_id]), 'product_id' => $product->product_id, 'type' => 'notice');
				continue;
			}

			if(!!empty($product->product_sale_start) && (int)$product->product_sale_start > time()) {
				$notUsable[$cart_product_id] = array('id' => $cart_product_id, 'qty' => 0);
				$cart->messages[] = array('msg' => JText::sprintf('PRODUCT_NOT_YET_ON_SALE', $errorMessagesProductNames[$cart_product_id]), 'product_id' => $product->product_id, 'type' => 'notice');
				continue;
			}

			if(!empty($product->product_sale_end) && (int)$product->product_sale_end < time()) {
				$notUsable[$cart_product_id] = array('id' => $cart_product_id, 'qty' => 0);
				$cart->messages[] = array('msg' => JText::sprintf('PRODUCT_NOT_SOLD_ANYMORE', $errorMessagesProductNames[$cart_product_id]), 'product_id' => $product->product_id, 'type' => 'notice');
				continue;
			}

			if((int)$product->product_parent_id > 0 && $product->product_type == 'variant') {

				if(!isset($parent_products[ (int)$product->product_parent_id ])) {
					$notUsable[$cart_product_id] = array('id' => $cart_product_id, 'qty' => 0);
					$cart->messages[] = array('msg' => JText::sprintf('PRODUCT_NOT_AVAILABLE', $errorMessagesProductNames[$cart_product_id]), 'product_id' => $product->product_id, 'type' => 'notice');
					continue;
				}

				$parent_product = $parent_products[ (int)$product->product_parent_id ];

				if(empty($parent_product->product_published)) {
					$notUsable[$cart_product_id] = array('id' => $cart_product_id, 'qty' => 0);
					$cart->messages[] = array('msg' => JText::sprintf('PRODUCT_NOT_AVAILABLE', $errorMessagesProductNames[$cart_product_id]), 'product_id' => $product->product_id, 'type' => 'notice');
					continue;
				}
				if((int)$product->product_quantity == -1 && (int)$parent_product->product_quantity == 0) {
					$notUsable[$cart_product_id] = array('id' => $cart_product_id, 'qty' => 0);
					$cart->messages[] = array('msg' => JText::sprintf('NOT_ENOUGH_STOCK_FOR_PRODUCT', $errorMessagesProductNames[$cart_product_id]), 'product_id' => $product->product_id, 'type' => 'notice');
					continue;
				}

				if(empty($product->product_sale_start) && (int)$parent_product->product_sale_start > 0 && (int)$parent_product->product_sale_start > time()) {
					$notUsable[$cart_product_id] = array('id' => $cart_product_id, 'qty' => 0);
					$cart->messages[] = array('msg' => JText::sprintf('PRODUCT_NOT_YET_ON_SALE', $errorMessagesProductNames[$cart_product_id]), 'product_id' => $product->product_id, 'type' => 'notice');
					continue;
				}
				if(empty($product->product_sale_end) && (int)$parent_product->product_sale_end > 0 && (int)$parent_product->product_sale_end < time()) {
					$notUsable[$cart_product_id] = array('id' => $cart_product_id, 'qty' => 0);
					$cart->messages[] = array('msg' => JText::sprintf('PRODUCT_NOT_SOLD_ANYMORE', $errorMessagesProductNames[$cart_product_id]), 'product_id' => $product->product_id, 'type' => 'notice');
					continue;
				}

				if(!in_array($parent_product->product_access, array('', 'all')) && hikashop_level(2)) {
					if(empty($userClass))
						$userClass = hikashop_get('class.user');
					$user_groups = $userClass->getGroups( (int)$cart->user_id );
					$parent_product_groups = explode(',', $parent_product->product_access);
					hikaserial::toInteger($parent_product_groups);
					$intersect = array_intersect($user_groups, $parent_product_groups);
					if(empty($intersect)){
						$notUsable[$cart_product_id] = array('id' => $cart_product_id, 'qty' => 0);
						$cart->messages[] = array('msg' => JText::sprintf('PRODUCT_NOT_AVAILABLE', $errorMessagesProductNames[$cart_product_id]), 'product_id' => $product->product_id, 'type' => 'notice');
						continue;
					}
				}

				if(empty($limits['max']) && (int)$parent_product->product_quantity > 0 && (int)$product->product_quantity == -1)
					$limits['max'] = true;
			}

			if(empty($limits['min']) && (!empty($product->product_min_per_order) || !empty($parent_product->product_min_per_order)))
				$limits['min'] = true;
			if(empty($limits['max']) && (!empty($product->product_max_per_order) || !empty($parent_product->product_max_per_order)))
				$limits['max'] = true;

			if(empty($limits['max']) && (int)$product->product_quantity > 0 && (int)$product->product_parent_id == 0)
				$limits['max'] = true;
		}
		unset($product);

		if(!empty($notUsable)) {
			$cartClass = hikashop_get('class.cart');
			$cartClass->updateProduct($cart->cart_id, $notUsable);

			foreach($notUsable as $k => $v) {
				unset($cart->cart_products[$k]);
				unset($cart->products[$k]);
				if(isset($cart->products['p'.$k])) {
					unset($cart->cart_products['p'.$k]);
					unset($cart->products['p'.$k]);
				}
			}
			$ret = false;
		}

		if(hikashop_level(1)) {
			if(!$this->limitCartProducts($cart))
				$ret = false;

			$limitClass = hikashop_get('class.limit');
			$limits = $limitClass->getCart($cart);
			if(!$limitClass->checkLimits($limits, $cart))
				$ret = false;
		}

		$notUsable = array();
		foreach($cart->cart_products as $k => $p) {
			if(!is_numeric($k) && substr($k, 0, 1) == 'p')
				continue;
			if(empty($p->cart_product_quantity) || (int)$p->cart_product_quantity < 0)
				$notUsable[$k] = array('id' => $k, 'qty' => 0);
		}
		if(empty($notUsable))
			return;

		$cartClass = hikashop_get('class.cart');
		$cartClass->updateProduct($cart->cart_id, $notUsable);
		foreach($notUsable as $k => $v) {
			unset($cart->cart_products[$k]);
			unset($cart->products[$k]);
			if(isset($cart->products['p'.$k])) {
				unset($cart->cart_products['p'.$k]);
				unset($cart->products['p'.$k]);
			}
		}
		$ret = false;
	}

	protected function limitCartProducts(&$cart) {
		$item_limit = (int)$this->config->get('cart_item_limit', 0);
		if($item_limit <= 0)
			return true;

		$current_items = 0;
		foreach($cart->cart_products as $product) {
			$current_items += (int)$product->cart_product_quantity;
		}
		if($current_items <= $item_limit)
			return true;

		$remove = $current_items - $item_limit;

		$product_cart_ids = array_reverse(array_keys($cart->cart_products));
		foreach($product_cart_ids as $k) {
			if(!is_numeric($k) && substr($k, 0, 1) == 'p')
				continue;

			$q = (int)$cart->cart_products[$k]->cart_product_quantity;
			if($q <= $remove) {
				$remove -= $q;
				$cart->cart_products[$k]->cart_product_quantity = 0;
				$cart->products[$k]->cart_product_quantity = 0;
			} else {
				$q -= $remove;
				$cart->cart_products[$k]->cart_product_quantity = $q;
				$cart->products[$k]->cart_product_quantity = $q;
				$remove = 0;
			}
			if(empty($remove))
				break;
		}

		return false;
	}

	public function getAjaxCartData($data, $cart, $status) {
		$ret = array(
			'ret' => (int)$status
		);
		if(!empty($cart->messages))
			$ret['messages'] = $cart->messages;

		if(empty($cart->cart_products))
			$ret['empty'] = true;

		if(!empty($data['type']) && $data['type'] == 'product' && !empty($data['form']) && count($data['form'])) {
			$added = array();
			foreach($data['form'] as $prod => $qty) {
				if($qty)
					$added[] = array('product' => $prod, 'quantity' => $qty);
			}

			if(count($added) == 1) {
				$data = array_merge($data,$added[0]);
			}
		}

		if(!isset($data['product']))
			return $ret;

		$imageHelper = hikashop_get('helper.image');
		if(!empty($cart->products)) {
			$config = hikashop_config();
			$imageSize = (int)$config->get('addtocart_popup_image_size', 50);
			foreach($cart->products as $product) {
				if($product->product_id != $data['product'])
					continue;

				$ret['product_name'] = $product->product_name;
				$ret['quantity'] = (int)$product->cart_product_quantity;

				if($imageSize > 0) {
					$image_path = (isset($product->images[0]->file_path) ? $product->images[0]->file_path : '');
					$img = $imageHelper->getThumbnail($image_path, array(50,50), array('default' => true), true);
					if($img->success)
						$ret['image'] = $img->url;
				}
				break;
			}
		}
		if(!isset($ret['product_name'])) {
			$productClass = hikashop_get('class.product');
			$product = $productClass->getProduct($data['product']);
			$ret['product_name'] = $product->product_name;
			$ret['quantity'] = 0;

			$image_path = ((isset($product->parent) && isset($product->parent->images[0]->file_path)) ? $product->parent->images[0]->file_path : '');
			$image_path = (isset($product->images[0]->file_path) ? $product->images[0]->file_path : $image_path);
			$img = $imageHelper->getThumbnail($image_path, array(50,50), array('default' => true), true);
			if($img->success)
				$ret['image'] = $img->url;
		}

		if(!empty($cart->messages)) {
			foreach($cart->messages as $msg) {
				if(empty($msg['product_id']) || (int)$msg['product_id'] != $data['product'])
					continue;
				$ret['message'] = $msg['msg'];
				break;
			}
		}

		return $ret;
	}
}
