<?php
/**
 * @package    HikaSerial for Joomla!
 * @version    4.0.2
 * @author     Obsidev S.A.R.L.
 * @copyright  (C) 2011-2023 OBSIDEV. All rights reserved.
 * @license    GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined('_JEXEC') or die('Restricted access');
?><?php
class hikaserialOrderClass extends hikaserialClass {

	protected $order_id = 0;
	protected $order_packs = array();
	protected $order_plans = array();
	protected $order_products = array();
	protected $products = array();

	private $cache = array();

	protected function loadPacks(&$order) {
		if(empty($order->order_id) || (int)$order->order_id == $this->order_id)
			return;

		$this->order_id = (int)$order->order_id;
		$this->sqlPacks = array();
		$this->order_products = array();
		$this->products = array();

		$p = null;
		if(isset($order->cart)) {
			$p = reset($order->cart->products);
		}
		if(!isset($p->order_product_id) || (isset($p->cart_product_id) && $p->cart_product_id == $p->order_product_id)) {
			$orderClass = hikaserial::get('shop.class.order');
			$order->cart = $orderClass->loadFullOrder($order->order_id, false, false);
		}

		if(!empty($order->order_serial_params) && empty($order->order_serials)) {
			$this->loadOrderSerialParams($order);
		}

		if(!empty($order->cart->products)) {
			foreach($order->cart->products as $p) {
				if(!isset($p->order_product_id) || empty($p->product_id))
					continue;

				$this->order_products[(int)$p->order_product_id] = array(
					'id' => (int)$p->product_id,
					'quantity' => (int)$p->order_product_quantity
				);

				if((int)$p->order_product_quantity == 0 && !empty($p->order_product_options)) {
					$p_options = is_string($p->order_product_options) ? hikaserial::unserialize($p->order_product_options) : $p->order_product_options;
					if(is_array($p_options) && isset($p_options['type']) && $p_options['type'] == 'bundle') {
						$parent_qty = $this->getParentProductQuantity($p, $order->cart->products);
						$this->order_products[(int)$p->order_product_id] = $parent_qty * (int)$p_options['related_quantity'];
					}
				}

				if(empty($this->products[(int)$p->product_id]))
					$this->products[(int)$p->product_id] = array();
				$this->products[(int)$p->product_id][] = (int)$p->order_product_id;

				if(!empty($p->bundle)) {
					foreach($p->bundle as $b) {
						$this->order_products[(int)$b->order_product_id] = array(
							'id' => (int)$b->product_id,
							'quantity' => (int)$p->order_product_quantity * (int)$b->order_product_options['related_quantity']
						);
						$this->products[(int)$b->product_id][] = (int)$b->order_product_id;
					}
				}
			}
		}
		if(empty($this->products))
			return;

		$this->order_packs = array();
		$this->order_plans = array();
		$query = 'SELECT product.product_id as `main_product_id`, product.product_parent_id, pp.*, pack.* '.
			' FROM ' . hikaserial::table('shop.product') . ' as product '.
			' INNER JOIN ' . hikaserial::table('product_pack') . ' as pp ON ( pp.product_id = product.product_id OR (pp.product_id = product.product_parent_id AND pp.product_id > 0)) '.
			' INNER JOIN ' . hikaserial::table('pack') . ' as pack ON pp.pack_id = pack.pack_id '.
			' WHERE product.product_id IN (' . implode(',', array_keys($this->products)) . ') AND pack.pack_published = 1'.
			' ORDER BY pp.pack_id ASC';
		$this->db->setQuery($query);
		$order_packs = $this->db->loadObjectList();
		foreach($order_packs as $order_pack) {
			if(empty($order_pack->pack_type) || $order_pack->pack_type == 'serial') {
				$this->order_packs[] = $order_pack;
			} else if($order_pack->pack_type == 'subscription') {
				$this->order_plans[] = $order_pack;
			}
		}
	}

	private function getParentProductQuantity($child, $products) {
		foreach($products as $p) {
			if($child->order_product_id == $p->order_product_id)
				continue;
			if(isset($child->cart_product_option_parent_id) && (int)$child->cart_product_option_parent_id == (int)$p->cart_product_id)
				return (int)$p->order_product_quantity;
			if(isset($child->order_product_option_parent_id) && (int)$child->order_product_option_parent_id == (int)$p->order_product_id)
				return (int)$p->order_product_quantity;
		}
		return 0;
	}

	private function loadOrderSerialParams(&$order) {
		$order_serial_params = json_decode($order->order_serial_params, true);

		$serials = array();
		$packs = array();

		if(!empty($order_serial_params['order'])) {
			$qty = 0;
			foreach($order->cart->products as $p) {
				$qty += (int)$p->order_product_quantity;
			}
			foreach($order_serial_params['order'] as $k => $v) {
				$a = 0; $b = 0;
				if(is_array($v)) list($a, $b) = $v;
				else $a = (int)$v;

				if(!isset($packs[(int)$k]))
					$packs[(int)$k] = 0;
				$packs[(int)$k] += ((int)$a * (int)$qty) + (int)$b;
			}
		}

		if(!empty($order_serial_params['product']) && !empty($order->cart->products)) {
			foreach($order_serial_params['product'] as $pid => $data) {
				$qty = 0;
				foreach($order->cart->products as $p) {
					if((int)$p->product_id == (int)$k)
						$qty += (int)$p->order_product_quantity;
				}
				if(empty($qty))
					continue;

				foreach($data as $k => $v) {
					$a = 0; $b = 0;
					if(is_array($v)) list($a, $b) = $v;
					else $a = (int)$v;

					if(!isset($packs[(int)$k]))
						$packs[(int)$k] = 0;
					$packs[(int)$k] += ((int)$a * (int)$qty) + (int)$b;
				}
			}
		}

		if(!empty($order_serial_params['order_product']) && !empty($order->cart->products)) {
			foreach($order_serial_params['order_product'] as $pid => $data) {
				$qty = 0;
				foreach($order->cart->products as $p) {
					if((int)$p->order_product_id == (int)$k) {
						$qty = (int)$p->order_product_quantity;
						break;
					}
				}
				if(empty($qty))
					continue;
				foreach($data as $k => $v) {
					$a = 0; $b = 0;
					if(is_array($v)) list($a, $b) = $v;
					else $a = (int)$v;

					if(!isset($packs[(int)$k]))
						$packs[(int)$k] = 0;
					$packs[(int)$k] += ((int)$a * (int)$qty) + (int)$b;
				}
			}
		}

		if(!empty($order_serial_params['serial'])) {
			foreach($order_serial_params['serial'] as $v) {
				$serials[(int)$v] = (int)$v;
			}
		}

		$order->order_serials = new stdClass();
		$order->order_serials->packs = $packs;
		$order->order_serials->serials = $serials;
	}

	public function loadSerialData($order_id, $viewName = null) {
		if(empty($order_id))
			return false;

		$config = hikaserial::config();

		$display_serial_statuses = $config->get('display_serial_statuses','');
		if(empty($display_serial_statuses)) {
			$display_serial_statuses = array($config->get('used_serial_status','used'));
		} else {
			$display_serial_statuses = explode(',', $display_serial_statuses);
		}
		foreach($display_serial_statuses as &$s) {
			$s = $this->db->Quote($s);
		}
		unset($s);

		$query = 'SELECT s.*, p.*, op.* FROM '.
			hikaserial::table('serial') . ' as s '.
			'INNER JOIN '. hikaserial::table('pack') . ' as p ON s.serial_pack_id = p.pack_id '.
			'LEFT JOIN ' . hikaserial::table('shop.order_product') . ' as op ON s.serial_order_product_id = op.order_product_id '.
			'WHERE s.serial_status IN ('.implode(',',$display_serial_statuses).') AND s.serial_order_id = ' . $order_id.' '.
			'ORDER BY op.order_product_id ASC, p.pack_id ASC, s.serial_id ASC';
		$this->db->setQuery($query);
		$data = $this->db->loadObjectList();

		if(!empty($viewName)) {
			JPluginHelper::importPlugin('hikaserial');
			$dispatcher = hikaserial::getDispatcher();
			$dispatcher->trigger('onDisplaySerials', array(&$data, $viewName));
		}

		if(!empty($data)) {
			foreach($data as &$serial) {
				if(!isset($serial->serial_text_data)) {
					$serial->serial_text_data = $serial->serial_data;
					$serial->serial_data = str_replace(array("\r\n","\r","\n"), '<br/>', $serial->serial_data);
				}
				if(!empty($serial->serial_extradata) && is_string($serial->serial_extradata))
					$serial->serial_extradata = hikaserial::unserialize($serial->serial_extradata);
				unset($serial);
			}
		}

		return $data;
	}

	public function preUpdate(&$order) {
		if(isset($order->order_type) && $order->order_type != 'sale')
			return;
		if(isset($order->old->order_type) && $order->old->order_type != 'sale')
			return;

		$creation = !isset($order->old->order_type) && !empty($order->cart);

		$order->serial_data = new stdClass();
		$order->serial_data->old_order_status = '';
		if(!empty($order->old) && !empty($order->old->order_status)) {
			$order->serial_data->old_order_status = $order->old->order_status;
		} else if(isset($order->order_id) && (int)$order->order_id > 0) {
			$this->db->setQuery('SELECT order_status FROM ' . hikaserial::table('shop.order') . ' WHERE order_id = ' . (int)$order->order_id);
			$ret = $this->db->loadObject();
			$order->serial_data->old_order_status = $ret->order_status;
		}

		$order_serial_params = array(
			'order' => array(),
			'product' => array(),
			'order_product' => array(),
			'serial' => array()
		);
		if($creation && !empty($order->cart->cart_params->subscription)) {
			$order_serial_params['cart_subscriptions'] = array();
			foreach($order->cart->cart_params->subscription as $subscription_id => $cart_products) {
				foreach($cart_products as $cart_product_id) {
					if(!isset($order->cart->cart_products[$cart_product_id]))
						continue;
					if(empty($order_serial_params['cart_subscriptions'][$subscription_id]))
						$order_serial_params['cart_subscriptions'][$subscription_id] = array();
					$order_serial_params['cart_subscriptions'][$subscription_id][] = $cart_product_id;
				}
			}
		}
		if(empty($order->order_serial_params) && !empty($order->old->order_serial_params))
			$order->order_serial_params = $order->old->order_serial_params;
		if(!empty($order->order_serial_params)) {
			if(is_string($order->order_serial_params))
				$order_serial_params = array_merge($order_serial_params, json_decode($order->order_serial_params, true));
			elseif(is_array($order->order_serial_params))
				$order_serial_params = array_merge($order_serial_params, $order->order_serial_params);
		}

		$new = empty($order->order_id);

		JPluginHelper::importPlugin('hikaserial');
		$dispatcher = hikaserial::getDispatcher();
		$dispatcher->trigger('onSerialOrderPreUpdate', array($new, &$order, &$order_serial_params));

		$order->order_serial_params = json_encode($order_serial_params);
	}

	public function postUpdate(&$order) {
		if(!isset($order->order_status))
			return;
		if(!isset($order->serial_data->old_order_status))
			return;

		if(isset($order->order_type) && $order->order_type != 'sale')
			return;
		if(isset($order->old->order_type) && $order->old->order_type != 'sale')
			return;
		if(empty($order->old->order_id) && empty($order->serial_data->old_order_status)) {
			$this->assignSubscriptions($order);
		} elseif(!empty($order->old->order_id) && $order->serial_data->old_order_status != $order->order_status) {
			$this->updateSubscriptions($order);
		}
		$config = hikaserial::config();
		$statuses = explode(',', $config->get('assignable_order_statuses', 'confirmed,shipped'));
		if(empty($statuses) || (count($statuses) == 1 && empty($statuses[0]))) {
			$shopConfig = hikaserial::config(false);
			$statuses = array($shopConfig->get('order_confirmed_status'));
		}

		$current_status_valid = in_array($order->order_status, $statuses);
		$old_status_valid = in_array($order->serial_data->old_order_status, $statuses);

		if($current_status_valid && !$old_status_valid) {
			$this->assignSerials($order);
		} else if(!$current_status_valid && $old_status_valid) {
			$this->unassignSerials($order);
		}
	}

	public function assignSerials(&$order) {
		if(isset($order->order_type) && $order->order_type != 'sale')
			return;
		if(isset($order->old->order_type) && $order->old->order_type != 'sale')
			return;

		$config = hikaserial::config();
		$shopConfig = hikaserial::config(false);

		$useable_serial_statuses = explode(',', str_replace('\'', '\\\'', $config->get('useable_serial_statuses', 'free')));
		$this->useable_serial_statuses = $useable_serial_statuses;
		$assigned_serial_status = $config->get('assigned_serial_status', 'assigned');
		$used_serial_status = $config->get('used_serial_status', 'used');
		$unassignedStatus = $config->get('unassigned_serial_status', '');
		$statuses = explode(',', $config->get('assignable_order_statuses', 'confirmed,shipped'));
		if(empty($statuses) || (count($statuses) == 1 && empty($statuses[0]))) {
			$statuses = array($shopConfig->get('order_confirmed_status'));
		}

		if(!in_array($order->order_status, $statuses))
			return;

		$serial_statuses = array(
			$this->db->Quote($used_serial_status),
			$this->db->Quote($assigned_serial_status)
		);
		$query = 'SELECT serial.serial_pack_id, serial.serial_id, serial_order_product_id FROM ' . hikaserial::table('serial') . ' as serial '.
			' WHERE serial_status IN ('.implode(',',$serial_statuses).') AND serial_order_id = '.(int)$order->order_id.
			' ORDER BY serial_id';
		$this->db->setQuery($query);
		$current_serials = $this->db->loadObjectList('serial_id');
		$current = array();
		$used = array();
		if(!empty($current_serials)) {
			foreach($current_serials as &$serial) {
				$serial->serial_pack_id = (int)$serial->serial_pack_id;
				$serial->serial_id = (int)$serial->serial_id;
				$serial->serial_order_product_id = (int)$serial->serial_order_product_id;

				if(empty($current[$serial->serial_pack_id]))
					$current[$serial->serial_pack_id] = array();
				$current[$serial->serial_pack_id][] = $serial->serial_id;
			}
			unset($serial);
		}

		$this->loadPacks($order);
		if(empty($this->order_packs) && empty($order->order_serials->serials) && empty($order->order_serials->packs)) {
			if(!empty($current)) {
				$this->unassignSerials($order);
			}
			return;
		}

		$packClass = hikaserial::get('class.pack');
		$historyClass = hikaserial::get('class.history');

		$packs = array();
		$unassigned = array();
		foreach($this->order_packs as &$pack) {
			$pack->quantity = (int)$pack->quantity;
			$pack->main_product_id = (int)$pack->main_product_id;
			$pack->product_parent_id = (int)$pack->product_parent_id;
			$pack->product_id = (int)$pack->product_id;
			$pack->pack_id = (int)$pack->pack_id;

			if( $pack->quantity <= 0 )
				continue;

			$pid = (int)$pack->product_id;
			if($pack->main_product_id != $pack->product_id) {
				$found = false;
				foreach($this->order_packs as $search) {
					if($search->product_id == $pack->main_product_id) {
						$found = true;
						break;
					}
				}
				if($found)
					continue;

				$pid = (int)$pack->main_product_id;
			}

			foreach($this->products[$pid] as $order_product_id) {
				$order_product = $this->order_products[$order_product_id];

				$pack_quantity = $pack->quantity * $order_product['quantity'];
				$final_quantity = $pack_quantity;

				$tmp_serials = array();
				foreach($current_serials as $k => &$v) {
					if($v->serial_pack_id != $pack->pack_id || (isset($v->used) && $v->used == true))
						continue;

					if($v->serial_order_product_id != $order_product_id) {
						if(!isset($v->used))
							$v->used = false;
						continue;
					}

					$pack_quantity--;

					$tmp_serials[] = $k;

					$v->used = true;
				}
				unset($v);

				if($pack_quantity == 0 )
					continue;

				if($pack_quantity < 0 ) {
					$unassigned = array_merge($unassigned, array_slice($tmp_serials, $final_quantity));
					continue;
				}

				if(is_string($pack->pack_params))
					$pack->pack_params = hikaserial::unserialize($pack->pack_params);
				$packs[$pack->pack_id] =& $pack;

				$this->assignPack($pack, $order, $order_product_id, $pack_quantity);
			}
		}
		unset($pack);

		if(!empty($order->order_serials->packs)) {
			foreach($order->order_serials->packs as $pack_id => $quantity) {
				foreach($current_serials as $k => &$v) {
					if($v->serial_pack_id != $pack_id || !empty($v->used))
						continue;

					if(!isset($v->used) || $v->used == false) {
						$v->used = true;
						$quantity--;
					}
					if($quantity == 0)
						break;
				}
				unset($v);

				if($quantity > 0) {
					$pack = $packClass->get($pack_id);
					$pack->quantity = $quantity;
					$this->assignPack($pack, $order, 0);
					$packs[] =& $pack;
				}
			}
		}

		if(!empty($order->order_serials->serials)) {
			$update_time = time();
			$query = 'UPDATE ' . hikaserial::table('serial') .
				' SET serial_order_id = ' . (int)$order->order_id .
				', serial_assign_date = ' . (int)$update_time .
				', serial_status = ' . $this->db->quote($used_serial_status) .
				', serial_user_id = ' . (int)@$order->cart->order_user_id .
				', serial_order_product_id = 0' .
				' WHERE serial_id IN (' . implode(',',$order->order_serials->serials) . ')'.
					' AND serial_status IN (\'\', \''.implode('\',\'', $this->useable_serial_statuses).'\')'.
					' AND serial_user_id IN (NULL, 0, '.(int)@$order->cart->order_user_id.')';
			$this->db->setQuery($query);
			$this->db->execute();

			if($config->get('save_history', 1)) {
				$histories = array();
				foreach($order->order_serials->serials as $id) {
					$histories[] = $historyClass->create($id, $used_serial_status, 'assignation');
				}
				if(!empty($histories))
					$historyClass->save($histories);
			}
		}

		foreach($current_serials as $k => $v) {
			if(!isset($v->used) || $v->used == false)
				$unassigned[] = $k;
		}

		if(!empty($unassigned)) {
			$querySet = array();
			if(!empty($unassignedStatus)) {
				$querySet[] = 'serial_status = ' . $this->db->Quote($unassignedStatus);
			}
			if((int)$config->get('unassigned_remove_data', 1) == 1) {
				$querySet[] = 'serial_assign_date = NULL';
				$querySet[] = 'serial_order_id = NULL';
				$querySet[] = 'serial_user_id = NULL';
				$querySet[] = 'serial_order_product_id = NULL';
			}
			if(!empty($querySet)) {
				$query = 'SELECT a.*, b.* FROM ' . hikaserial::table('serial') . ' AS a INNER JOIN ' . hikaserial::table('pack') . ' AS b ON a.serial_pack_id = b.pack_id WHERE serial_id IN (' . implode(',',$unassigned).')';
				$this->db->setQuery($query);
				$serials = $this->db->loadObjectList();

				$query = 'UPDATE ' . hikaserial::table('serial') . ' SET ' . implode(', ', $querySet) . ' WHERE serial_id IN (' . implode(',',$unassigned).')';
				$this->db->setQuery($query);
				$this->db->execute();

				foreach($serials as &$serial) {
					$serial->unassignedStatus = $unassignedStatus;
					if(!empty($serial->pack_params)) {
						$serial->pack_params = hikaserial::unserialize($serial->pack_params);
						if(!empty($serial->pack_params->status_for_refund) && $serial->pack_params->status_for_refund != $unassignedStatus) {
							$serial->unassignedStatus = $serial->pack_params->status_for_refund;

							$query = 'UPDATE ' . hikaserial::table('serial') . ' SET serial_status = '.$this->db->Quote($serial->pack_params->status_for_refund).' WHERE serial_id = ' . (int)$serial->serial_id;
							$this->db->setQuery($query);
							$this->db->execute();
							$updated = true;
						}
					}
				}

				if($config->get('save_history', 1)) {
					$histories = array();
					foreach($serials as &$serial) {
						$histories[] = $historyClass->create($serial->serial_id, $serial->unassignedStatus, 'unassignation');
					}
					if(!empty($histories))
						$historyClass->save($histories);
				}
			}
		}

		if(!empty($packs)) {
			foreach($packs as &$pack) {
				$packClass->checkQuantity($pack);
			}
		}

		if(!empty($unassigned) || !empty($packs)){
			$productClass = hikaserial::get('class.product');
			$productClass->refreshQuantities();
		}
	}

	protected function assignSubscriptions(&$order) {
		if(isset($order->order_type) && $order->order_type != 'sale')
			return;
		if(isset($order->old->order_type) && $order->old->order_type != 'sale')
			return;

		$order_serial_params = $this->getOrderSerialParams($order);

		if(isset($order->cart) && !empty($order_serial_params['cart_subscriptions']) && isset($order->cart_order_products)) {
			$order_serial_params['subscriptions'] = array();
			foreach($order_serial_params['cart_subscriptions'] as $subscription_id => $cart_products) {
				foreach($cart_products as $cart_product_id) {
					if(!isset($order->cart_order_products[$cart_product_id]))
						continue;
					if(empty($order_serial_params['subscriptions'][$subscription_id]))
						$order_serial_params['subscriptions'][$subscription_id] = array();
					$order_serial_params['subscriptions'][$subscription_id][] = (int)$order->cart_order_products[$cart_product_id];
				}
			}
			unset($order_serial_params['cart_subscriptions']);

			$order->order_serial_params = json_encode($order_serial_params);

			$query = 'UPDATE '.hikaserial::table('shop.order').' AS hk_order '.
				' SET hk_order.order_serial_params = '.$this->db->Quote($order->order_serial_params).
				' WHERE order_id = '.(int)$order->order_id;
			$this->db->setQuery($query);
			$this->db->execute();
		}

		$this->loadPacks($order);
		if(empty($this->order_plans))
			return;

		$product_assignations = array();
		$subscriptions = array();
		if(!empty($order_serial_params['subscriptions'])) {
			$subscription_ids = array_keys($order_serial_params['subscriptions']);
			hikaserial::toInteger($subscription_ids);

			$query = 'SELECT * FROM '.hikaserial::table('subscription').' WHERE subscription_id IN ('.implode(',', $subscription_ids).') AND subscription_user_id = '.(int)$order->order_user_id;
			$this->db->setQuery($query);
			$subscriptions = $this->db->loadObjectList('subscription_id');

			foreach($order_serial_params['subscriptions'] as $subscription_id => $order_product_ids) {
				foreach($order_product_ids as $order_product_id) {
					if(empty($product_assignations[$order_product_id]))
						$product_assignations[$order_product_id] = array();
					$product_assignations[$order_product_id][] = $subscription_id;
				}
			}
		}

		foreach($this->order_plans as $order_plan) {

			$pid = (int)$order_plan->product_id;
			if($order_plan->main_product_id != $order_plan->product_id) {
				$found = false;
				foreach($this->order_plans as $search) {
					if($search->product_id == $order_plan->main_product_id) {
						$found = true;
						break;
					}
				}
				if($found)
					continue;
				$pid = (int)$order_plan->main_product_id;
			}

			foreach($this->products[$pid] as $order_product_id) {
				$order_product = $this->order_products[$order_product_id];
				$qty = (int)$order_product['quantity'];

				$order_plan->order_product_id = $order_product_id;
				$order_plan->order_product_qty = $qty;

				switch($order_plan->relation_type) {
					case 'renew':
						$this->renewOrderSubscription($order, $order_plan, $product_assignations, $subscriptions);
						break;
					case 'upgrade':
						$this->upgradeOrderSubscription($order, $order_plan, $product_assignations, $subscriptions);
						break;
					case 'renew_create':
					case 'create_renew':
						$this->createOrRenewOrderSubscription($order, $order_plan, $product_assignations, $subscriptions);
						break;
					case 'create':
					default:
						for($i = 1; $i <= $qty; $i++) {
							$this->createOrderSubscription($order, $order_plan);
						}
						break;
				}
			}
		}
	}

	private function createOrRenewOrderSubscription($order, $order_plan, $product_assignations, $subscriptions) {
		$order_serial_params = $this->getOrderSerialParams($order);

		$subscription_pack_id = (int)$order_plan->pack_id;
		$order_product_id = (int)$order_plan->order_product_id;

		$qty = (int)$order_plan->order_product_qty;

		if(!empty($product_assignations[$order_product_id])) {
			foreach($product_assignations[$order_product_id] as $subscription_id) {
				if(empty($subscriptions[$subscription_id]))
					continue;
				if((int)$subscriptions[$subscription_id]->subscription_pack_id == $subscription_pack_id) {
					$order_plan->related_subcription_id = $subscription_id;
					return $this->renewOrderSubscription($order, $order_plan, $product_assignations, $subscriptions);
				}
			}
		}

		if($order_plan->relation_type == 'renew_create')
			return $this->createOrderSubscription($order, $order_plan);

		$ret = array();
		for($i = 1; $i <= $qty; $i++) {
			$ret[] = $this->createOrderSubscription($order, $order_plan);
		}
		return $ret;
	}

	private function createOrderSubscription($order, $order_plan) {
		$config = hikaserial::config();
		$shopConfig = hikaserial::config(false);

		$statuses = explode(',', $config->get('assignable_order_statuses', 'confirmed,shipped'));
		if(empty($statuses) || (count($statuses) == 1 && empty($statuses[0])))
			$statuses = array($shopConfig->get('order_confirmed_status'));
		$valid_order = in_array($order->order_status, $statuses);
		$now = time();

		$qty = ($order_plan->relation_type == 'renew_create') ? (int)$order_plan->order_product_qty : 1;

		$subscription = new stdClass();
		$subscription->subscription_created = $now;
		$subscription->subscription_user_id = (int)$order->order_user_id;
		$subscription->subscription_pack_id = (int)$order_plan->pack_id;
		$subscription->subscription_status = $valid_order ? 'active' : 'closed';
		if($valid_order) {
			$subscription->subscription_start_date = $now;
			$subscription->subscription_end_date = hikaserial::addDurationTime($now, $order_plan->relation_duration, $qty);
		}

		$subscription->orders = array( $order );

		$subscriptionClass = hikaserial::get('class.subscription');
		$subscription_id = $subscriptionClass->save($subscription);

		if(!$subscription_id)
			return false;

		$order_product_id = (int)$order_plan->order_product_id;

		$query = 'INSERT INTO '.hikaserial::table('order_subscription').
			' (`subscription_id`, `order_id`, `order_product_id`, `subscription_validated`, `subscription_duration`, `order_product_quantity`, `plan_id`, `upgrade_id`)'.
			' VALUES ('.(int)$subscription_id.','.(int)$order->order_id.','.$order_product_id.','.($valid_order ? $now : 0).','.$this->db->Quote($order_plan->relation_duration).','.$qty.','.(int)$order_plan->pack_id.',NULL)';
		$this->db->setQuery($query);
		$this->db->execute();
		$order_subscription_id = $this->db->insertid();

		if(!$order_subscription_id)
			return false;

		return array(
			'subscription_id' => $subscription_id,
			'order_subscription_id' => $order_subscription_id,
		);
	}

	private function renewOrderSubscription($order, $order_plan, $product_assignations, $subscriptions) {
		$config = hikaserial::config();
		$shopConfig = hikaserial::config(false);

		$order_serial_params = $this->getOrderSerialParams($order);
		$subscription_pack_id = (int)$order_plan->pack_id;
		$order_product_id = (int)$order_plan->order_product_id;

		if(!empty($order_plan->related_subcription_id)) {
			$subscription_id = $order_plan->related_subcription_id;
		} else {
			$subscription_id = 0;
			foreach($product_assignations[$order_product_id] as $sub_id) {
				if(empty($subscriptions[$sub_id]))
					continue;
				if($subscriptions[$sub_id]->subscription_pack_id == $subscription_pack_id) {
					$subscription_id = $sub_id;
					break;
				}
			}
			if(empty($subscription_id))
				return false;
		}


		$statuses = explode(',', $config->get('assignable_order_statuses', 'confirmed,shipped'));
		if(empty($statuses) || (count($statuses) == 1 && empty($statuses[0])))
			$statuses = array($shopConfig->get('order_confirmed_status'));
		$valid_order = in_array($order->order_status, $statuses);
		$now = time();

		$qty = max((int)$order_plan->order_product_qty, 1);

		$query = 'INSERT INTO '.hikaserial::table('order_subscription').
			' (`subscription_id`, `order_id`, `order_product_id`, `subscription_validated`, `subscription_duration`, `order_product_quantity`, `plan_id`, `upgrade_id`)'.
			' VALUES ('.(int)$subscription_id.','.(int)$order->order_id.','.$order_product_id.','.($valid_order ? $now : 0).','.$this->db->Quote($order_plan->relation_duration).','.$qty.','.$subscription_pack_id.',NULL)';
		$this->db->setQuery($query);
		$this->db->execute();
		$order_subscription_id = $this->db->insertid();

		if(!$order_subscription_id)
			return false;

		if(!$valid_order) {
			return array(
				'subscription_id' => $subscription_id,
				'order_subscription_id' => $order_subscription_id,
			);
		}

		$subscriptionClass = hikaserial::get('class.subscription');
		$subscription = $subscriptionClass->get($subscription_id);
		$subscription->order_subscription_id = (int)$order_subscription_id;
		$subscription->order_id = (int)$order->order_id;
		$subscription->order_product_id = (int)$order_product_id;
		$subscription->order_product_quantity = $qty;
		$subscription->subscription_validated = $now;
		$subscription->subscription_duration = $order_plan->relation_duration;
		$subscription->plan_id = (int)$subscription_pack_id;
		$subscription->upgrade_id = null;
		$order_subscriptions = array(
			$subscription
		);
		$this->activeOrderSubscriptions($order, $order_subscriptions, array('history_type' => 'renewal'));

		return array(
			'subscription_id' => $subscription_id,
			'order_subscription_id' => $order_subscription_id,
		);
	}

	private function upgradeOrderSubscription($order, $order_plan, $product_assignations, $subscriptions) {
		$config = hikaserial::config();
		$shopConfig = hikaserial::config(false);

		$order_serial_params = $this->getOrderSerialParams($order);
		$subscription_pack_id = (int)$order_plan->pack_id;
		$relation_upgrade_id = (int)$order_plan->relation_upgrade;
		$order_product_id = (int)$order_plan->order_product_id;

		$subscription_id = 0;
		foreach($product_assignations[$order_product_id] as $sub_id) {
			if(empty($subscriptions[$sub_id]))
				continue;
			if($subscriptions[$sub_id]->subscription_pack_id == $relation_upgrade_id) {
				$subscription_id = $sub_id;
				break;
			}
		}
		if(empty($subscription_id))
			return false;

		$statuses = explode(',', $config->get('assignable_order_statuses', 'confirmed,shipped'));
		if(empty($statuses) || (count($statuses) == 1 && empty($statuses[0])))
			$statuses = array($shopConfig->get('order_confirmed_status'));
		$valid_order = in_array($order->order_status, $statuses);
		$now = time();

		$qty = max((int)$order_plan->order_product_qty, 1);

		$query = 'INSERT INTO '.hikaserial::table('order_subscription').
			' (`subscription_id`, `order_id`, `order_product_id`, `subscription_validated`, `subscription_duration`, `order_product_quantity`, `plan_id`, `upgrade_id`)'.
			' VALUES ('.(int)$subscription_id.','.(int)$order->order_id.','.$order_product_id.','.($valid_order ? $now : 0).','.$this->db->Quote($order_plan->relation_duration).','.$qty.','.$relation_upgrade_id.','.$subscription_pack_id.')';
		$this->db->setQuery($query);
		$this->db->execute();
		$order_subscription_id = $this->db->insertid();

		if(!$order_subscription_id)
			return false;

		if(!$valid_order) {
			return array(
				'subscription_id' => $subscription_id,
				'order_subscription_id' => $order_subscription_id,
			);
		}

		$subscriptionClass = hikaserial::get('class.subscription');
		$subscription = $subscriptionClass->get($subscription_id);
		$subscription->order_subscription_id = (int)$order_subscription_id;
		$subscription->order_id = (int)$order->order_id;
		$subscription->order_product_id = (int)$order_product_id;
		$subscription->order_product_quantity = $qty;
		$subscription->subscription_validated = $now;
		$subscription->subscription_duration = $order_plan->relation_duration;
		$subscription->plan_id = (int)$relation_upgrade_id;
		$subscription->upgrade_id = (int)$subscription_pack_id;
		$order_subscriptions = array(
			$subscription
		);
		$this->activeOrderSubscriptions($order, $order_subscriptions, array('history_type' => 'upgrade'));

		return array(
			'subscription_id' => $subscription_id,
			'order_subscription_id' => $order_subscription_id,
		);
	}

	protected function updateSubscriptions(&$order) {
		$config = hikaserial::config();
		$shopConfig = hikaserial::config(false);

		$statuses = explode(',', $config->get('assignable_order_statuses', 'confirmed,shipped'));
		if(empty($statuses) || (count($statuses) == 1 && empty($statuses[0])))
			$statuses = array($shopConfig->get('order_confirmed_status'));

		$current_status_valid = in_array($order->order_status, $statuses);
		$old_status_valid = in_array($order->serial_data->old_order_status, $statuses);

		$order_subscriptions = array();
		if(($current_status_valid && !$old_status_valid) || (!$current_status_valid && $old_status_valid)) {
			$query = 'SELECT hk_os.*, hk_s.* FROM '.hikaserial::table('order_subscription').' as hk_os '.
				' INNER JOIN '.hikaserial::table('subscription').' AS hk_s ON hk_os.subscription_id = hk_s.subscription_id '.
				' WHERE order_id = '.(int)$order->order_id;
			$this->db->setQuery($query);
			$order_subscriptions = $this->db->loadObjectList();
		}
		if(empty($order_subscriptions))
			return;

		if($current_status_valid && !$old_status_valid) {
			$this->activeOrderSubscriptions($order, $order_subscriptions);
		} else if(!$current_status_valid && $old_status_valid) {
			$this->closeOrderSubscriptions($order, $order_subscriptions);
		}
	}

	private function activeOrderSubscriptions(&$order, &$order_subscriptions, $options = array()) {
		if(empty($order_subscriptions))
			return false;

		$config = hikaserial::config();
		$upgrade_type = $config->get('subscription_upgrade_mode', 'diff');
		if(!in_array($upgrade_type, array('diff','extend')))
			$upgrade_type = 'diff';

		$now = time();
		$time_base = time();
		$subscriptions = array();

		if(!empty($order->order_invoice_created))
			$time_base = (int)$order->order_invoice_created;

		$errors = array();

		foreach($order_subscriptions as $order_subscription) {
			$id = (int)$order_subscription->subscription_id;
			$upgrade = !empty($order_subscription->upgrade_id);

			if(!empty($order_subscription->plan_id) && $order_subscription->plan_id != $order_subscription->subscription_pack_id) {
				$errors[] = $order_subscription;
				continue;
			}
			if($upgrade && $order_subscription->subscription_status != 'active') {
				$errors[] = $order_subscription;
				continue;
			}

			if(!isset($subscriptions[$id])) {
				$s = new stdClass();
				$s->subscription_id = $id;
				$s->subscription_status = $order_subscription->subscription_status;
				$s->subscription_start_date = (int)$order_subscription->subscription_start_date;
				$s->subscription_end_date = (int)$order_subscription->subscription_end_date;

				$s->history = new stdClass();
				$s->history->history_type = isset($options['history_type']) ? $options['history_type'] : 'order update';
				$s->history->history_data = array('order_id' => (int)$order->order_id);

				$subscriptions[$id] =& $s;
				unset($s);
			}

			$sub = &$subscriptions[$id];

			$qty = isset($order_subscription->order_product_quantity) ? $order_subscription->order_product_quantity : 1;

			if($sub->subscription_status != 'active') {
				$sub->subscription_status = 'active';
				$sub->subscription_end_date = $time_base;

				if(empty($sub->subscription_start_date))
					$sub->subscription_start_date = $time_base;
			}

			$duration = $order_subscription->subscription_duration;
			if($upgrade) {
				if($upgrade_type == 'diff') {
					$upgrade_end = hikaserial::addDurationTime($time_base, $duration, $qty);
					$duration = max(0, $upgrade_end - $sub->subscription_end_date);
				}
				if(empty($sub->upgrade_order_sub))
					$sub->upgrade_order_sub = array();
				$sub->upgrade_order_sub[(int)$order_subscription->order_subscription_id] = $duration;
				$sub->subscription_pack_id = (int)$order_subscription->upgrade_id;
				$qty = 1;
				if($upgrade_type == 'diff') {
					$duration .= 's';
				}
			}

			$sub->subscription_end_date = hikaserial::addDurationTime($sub->subscription_end_date, $duration, $qty);
			$sub->history->history_data['end_date'] = $sub->subscription_end_date;

			unset($sub);
		}

		$ret = 0;
		$subscriptionClass = hikaserial::get('class.subscription');
		foreach($subscriptions as $subscription) {
			if(!$subscriptionClass->save($subscription)) {
				continue;
			}
			$ret++;

			if(empty($subscription->upgrade_order_sub))
				continue;
			foreach($subscription->upgrade_order_sub as $k => $v) {
				$query = 'UPDATE '.hikaserial::table('order_subscription').' SET upgrade_duration = '.(int)$v.
					' WHERE order_subscription_id = '.$k.' AND subscription_id = '.(int)$subscription->subscription_id;
				$this->db->setQuery($query);
				$this->db->execute();
			}
		}

		if(!empty($errors)) {
		}

		return $ret;
	}

	private function closeOrderSubscriptions(&$order, &$order_subscriptions) {
		if(empty($order_subscriptions))
			return false;

		$now = time();
		$subscriptions = array();
		foreach($order_subscriptions as $order_subscription) {
			$id = (int)$order_subscription->subscription_id;

			$upgrade = !empty($order_subscription->upgrade_id);

			if(!$upgrade && !empty($order_subscription->plan_id) && $order_subscription->plan_id != $order_subscription->subscription_pack_id) {
				$errors[] = $order_subscription;
				continue;
			}
			if($upgrade && $order_subscription->upgrade_id != $order_subscription->subscription_pack_id) {
				$errors[] = $order_subscription;
				continue;
			}

			if(!isset($subscriptions[$id])) {
				$s = new stdClass();
				$s->subscription_id = $id;
				$s->subscription_status = $order_subscription->subscription_status;
				$s->subscription_end_date = (int)$order_subscription->subscription_end_date;

				$subscriptions[$id] =& $s;
				unset($s);
			}

			$sub = &$subscriptions[$id];

			$duration = $order_subscription->subscription_duration;

			if($upgrade) {
				$now = time();
				$duration = (int)$order_subscription->upgrade_duration;
				if(empty($sub->upgrade_order_sub))
					$sub->upgrade_order_sub = array();
				$sub->upgrade_order_sub[] = (int)$order_subscription->order_subscription_id;
				$sub->subscription_pack_id = (int)$order_subscription->plan_id;
				$qty = 1;
			}

			if($sub->subscription_end_date > 0 && !empty($duration))
				$sub->subscription_end_date = hikaserial::addDurationTime($sub->subscription_end_date, '-'.$duration);

			if($sub->subscription_end_date < 0)
				$sub->subscription_end_date = 0;

			if($sub->subscription_end_date <= $now)
				$sub->subscription_status = 'expired';

			unset($sub);
		}

		$ret = 0;
		$subscriptionClass = hikaserial::get('class.subscription');
		foreach($subscriptions as $subscription) {
			if(!$subscriptionClass->save($subscription)) {
				continue;
			}
			$ret++;
			if(empty($subscription->upgrade_order_sub))
				continue;
			$query = 'UPDATE '.hikaserial::table('order_subscription').' SET upgrade_duration = NULL'.
				' WHERE order_subscription_id IN ('.implode(',', $subscription->upgrade_order_sub).') AND subscription_id = '.(int)$subscription->subscription_id;
			$this->db->setQuery($query);
			$this->db->execute();
		}

		if(!empty($errors)) {
print_r($errors);
		}

		return $ret;
	}

	protected function assignPack($pack, $order, $order_product_id, $request_quantity = 0) {
		$config = hikaserial::config();
		if($config->get('save_history', 1))
			$historyClass = hikaserial::get('class.history');

		if(empty($pack->pack_params->consumer) || !$pack->pack_params->consumer)
			$pack_serial_status = $config->get('used_serial_status', 'used');
		else
			$pack_serial_status = $config->get('assigned_serial_status', 'assigned');

		if(empty($request_quantity) || (int)$request_quantity == 0)
			$request_quantity = (int)$pack->quantity;

		$ids = null;
		if($pack->pack_data == 'sql') {
			$select = array('serial_id');
			if(!empty($pack->pack_params->random_pickup))
				$select[] = 'RAND() as serial_random';

			$query = 'SELECT '.implode(',', $select).' FROM ' . hikaserial::table('serial') . ' WHERE serial_pack_id = '.$pack->pack_id.' AND '.
					'(serial_status = \'\' OR serial_status IN (\''.implode('\',\'', $this->useable_serial_statuses).'\')) AND '.
					'(serial_user_id = '.(int)@$order->cart->order_user_id.' OR serial_user_id = 0 OR serial_user_id IS NULL) AND '.
					'(serial_order_id = '.(int)$order->order_id.' OR serial_order_id = 0 OR serial_order_id IS NULL) ';
			if(!empty($pack->pack_params->random_pickup))
				$query .= ' ORDER BY serial_user_id DESC, serial_random ASC, serial_id ASC';
			else
				$query .= ' ORDER BY serial_user_id DESC, serial_id ASC';

			$this->db->setQuery($query, 0, (int)$request_quantity);
			$ids = $this->db->loadColumn();
		}

		if(!empty($ids)) {
			$query = 'UPDATE ' . hikaserial::table('serial') .
				' SET serial_order_id = ' . (int)$order->order_id .
				', serial_assign_date = ' . (int)time() .
				', serial_status = ' . $this->db->quote($pack_serial_status) .
				', serial_user_id = ' . (int)@$order->cart->order_user_id .
				', serial_order_product_id = ' . (int)$order_product_id .
				' WHERE serial_id IN (' . implode(',',$ids) . ') AND (serial_status = \'\' OR serial_status IN (\''.implode('\',\'', $this->useable_serial_statuses).'\'))';
			$this->db->setQuery($query);
			$this->db->execute();

			if($config->get('save_history', 1)) {
				$histories = array();
				foreach($ids as $id) {
					$histories[] = $historyClass->create($id, $pack_serial_status, 'assignation');
				}
				if(!empty($histories))
					$historyClass->save($histories);
			}
		}

		if(!empty($ids) && count($ids) >= $request_quantity)
			return;

		$app = JFactory::getApplication();
		if(substr($pack->pack_generator, 0, 4) !== 'plg.') {
			if(hikaserial::isAdmin()) {
				$app->enqueueMessage(JText::sprintf('PACK_IS_EMPTY_AND_NO_GENERATOR', $pack->pack_name), 'error');
			} else {
			}
			return false;
		}

		$quantity = $request_quantity;
		if(!empty($ids))
			$quantity -= count($ids);

		$generateTime = time();
		$data = array(
			'serial_pack_id' => (int)$pack->pack_id,
			'serial_data' => '',
			'serial_extradata' =>  $this->db->Quote(''),
			'serial_status' => $this->db->quote($pack_serial_status),
			'serial_assign_date' => (int)$generateTime,
			'serial_order_id' => (int)$order->order_id,
			'serial_user_id' => (int)@$order->cart->order_user_id,
			'serial_order_product_id' => (int)$order_product_id
		);

		if(!empty($pack->pack_params->no_user_assign))
			$data['serial_user_id'] = 0;

		$query = 'INSERT IGNORE INTO ' . hikaserial::table('serial') . ' (' . implode(',', array_keys($data)) . ') VALUES ';

		$pluginId = 0;
		$pluginName = substr($pack->pack_generator, 4);
		if(strpos($pluginName,'-') !== false){
			list($pluginName,$pluginId) = explode('-', $pluginName, 2);
			$pack->$pluginName = $pluginId;
		}
		$serials = array();

		$plugin = hikaserial::import('hikaserial', $pluginName);
		if(method_exists($plugin, 'generate')) {
			$pack->order_product_id = (int)$order_product_id;
			ob_start();
			$ret = $plugin->generate($pack, $order, $quantity, $serials);
			ob_get_clean();
		}

		$queryData = array();
		foreach($serials as $serial) {
			$data['serial_extradata'] = $this->db->Quote('');
			if(is_array($serial)) {
				$data['serial_data'] = $this->db->Quote($serial['data']);
				if(!empty($serial['extradata']))
					$data['serial_extradata'] = $this->db->Quote(serialize($serial['extradata']));
			} else if(is_object($serial)) {
				$data['serial_data'] = $this->db->Quote($serial->data);
				if(!empty($serial->extradata))
					$data['serial_extradata'] = $this->db->Quote(serialize($serial->extradata));
			} else {
				$data['serial_data'] = $this->db->Quote($serial);
			}
			$queryData[] = '(' . implode(',', $data) . ')';
		}
		if(!empty($queryData)) {
			$query .= implode(',', $queryData);

			$this->db->setQuery($query);
			$this->db->execute();

			if($config->get('save_history', 1)) {
				$query = 'SELECT a.* FROM '.hikaserial::table('serial').' AS a WHERE a.serial_assign_date = '.(int)$generateTime.' AND serial_order_id = '.(int)$order->order_id;
				$this->db->setQuery($query);
				$generateSerials = $this->db->loadObjectList();

				$histories = array();
				foreach($generateSerials as $serial) {
					$histories[] = $historyClass->generate(null, $serial, true);
				}
				if(!empty($histories))
					$historyClass->save($histories);
			}
		}

		if(empty($serials) || count($serials) < $quantity) {
			if(isset($ret) && $ret === false || (int)$ret == count($serials)) {
				if(hikaserial::isAdmin()) {
					$app->enqueueMessage(JText::sprintf('COULD_NOT_GENERATE_SERIALS_FOR_PACK', $pack->pack_name), 'error');
				} else {
				}
			}
		}
	}

	public function unassignSerials(&$order) {
		$config = hikaserial::config();

		$updated = false;
		$querySet = array();
		$unassignedStatus = $config->get('unassigned_serial_status', '');

		$query = 'SELECT a.*, b.* FROM ' . hikaserial::table('serial') . ' AS a '.
				' INNER JOIN ' . hikaserial::table('pack') . ' AS b ON a.serial_pack_id = b.pack_id '.
				' WHERE serial_order_id = ' . (int)$order->order_id;
		$this->db->setQuery($query);
		$serials = $this->db->loadObjectList();

		if(!empty($unassignedStatus)) {
			$querySet['serial_status'] = 'serial_status = ' . $this->db->Quote($unassignedStatus);
		}
		if($config->get('unassigned_remove_data', '1') == '1') {
			$querySet['serial_assign_date'] = 'serial_assign_date = NULL';
			$querySet['serial_order_id'] = 'serial_order_id = NULL';
			$querySet['serial_user_id'] = 'serial_user_id = NULL';
			$querySet['serial_order_product_id'] = 'serial_order_product_id = NULL';
		}
		if(!empty($querySet)) {
			$query = 'UPDATE ' . hikaserial::table('serial') . ' SET ' . implode(', ', $querySet) . ' WHERE serial_order_id = ' . (int)$order->order_id;
			$this->db->setQuery($query);
			$this->db->execute();
			$updated = true;
		}

		foreach($serials as &$serial) {
			$serial->unassignedStatus = $unassignedStatus;
			if(!empty($serial->pack_params)) {
				$serial->pack_params = hikaserial::unserialize($serial->pack_params);
				if(!empty($serial->pack_params->status_for_refund) && $serial->pack_params->status_for_refund != $unassignedStatus) {
					$serial->unassignedStatus = $serial->pack_params->status_for_refund;

					$query = 'UPDATE ' . hikaserial::table('serial') . ' SET serial_status = '.$this->db->Quote($serial->pack_params->status_for_refund).' WHERE serial_id = ' . (int)$serial->serial_id;
					$this->db->setQuery($query);
					$this->db->execute();
					$updated = true;
				}
			}
		}

		JPluginHelper::importPlugin('hikaserial');
		$dispatcher = hikaserial::getDispatcher();
		$dispatcher->trigger('onAfterSerialUnassigned', array(&$serials));

		if($config->get('save_history', 1)) {
			$historyClass = hikaserial::get('class.history');
			$histories = array();
			foreach($serials as &$serial) {
				$histories[] = $historyClass->create($serial->serial_id, $serial->unassignedStatus, 'unassignation');
			}
			if(!empty($histories))
				$historyClass->save($histories);
		}

		if($updated) {
			$productClass = hikaserial::get('class.product');
			$productClass->refreshQuantities();
		}
	}

	public function refresh($order_id) {
		$config = hikaserial::config();
		$shopConfig = hikaserial::config(false);
		$orderClass = hikaserial::get('shop.class.order');

		if(empty($order_id))
			return;
		$order = $orderClass->loadFullOrder($order_id);
		if(empty($order))
			return;

		if($order->order_type != 'sale') {
			$this->unassignSerials($order);
			return;
		}

		$statuses = explode(',', $config->get('assignable_order_statuses', 'confirmed,shipped'));
		if(empty($statuses)) {
			$shopConfig = hikaserial::config(false);
			$statuses = array($shopConfig->get('order_confirmed_status'));
		}
		if(in_array($order->order_status, $statuses)) {
			$this->assignSerials($order);
		} else {
			$this->unassignSerials($order);
		}
	}

	private function getOrderSerialParams($order) {
		$order_serial_params = null;
		if(isset($order->order_serial_params))
			$order_serial_params = $order->order_serial_params;
		if(!empty($order_serial_params) && is_string($order_serial_params))
			$order_serial_params = json_decode($order_serial_params, true);
		return $order_serial_params;
	}

	public function beforeOrderExport(&$rows, &$view) {
		$orders = array();
		foreach($rows as $order) {
			$orders[] = (int)$order->order_id;
		}

		if(!empty($orders)) {
			$query = 'SELECT serial.serial_order_id, serial.serial_data FROM ' . hikaserial::table('serial') . ' AS serial WHERE serial.serial_order_id IN (' . implode(',', $orders) . ') ORDER BY serial.serial_order_id, serial.serial_id';
			$this->db->setQuery($query);
			$serials = $this->db->loadObjectList();
		}
		unset($orders);

		foreach($rows as $k => $order) {
			$order_serials = array();
			foreach($serials as $serial) {
				if($serial->serial_order_id == $order->order_id)
					$order_serials[] = $serial->serial_data;
			}
			$rows[$k]->serials = implode(';', $order_serials);
		}
		unset($serials);
	}

	public function modifyPack($order_id, $pack_id, $qty, $addition = true) {

		$orderClass = hikaserial::get('shop.class.order');
		$order = $orderClass->get($order_id);
		if(empty($order))
			return false;

		if($order->order_type != 'sale')
			return false;

		$order_serial_params = array();
		if(!empty($order->order_serial_params))
			$order_serial_params = json_decode($order->order_serial_params, true);

		if(empty($order_serial_params['order']))
			$order_serial_params['order'] = array();
		if(empty($order_serial_params['order'][$pack_id]))
			$order_serial_params['order'][$pack_id] = array(0,0);

		if($addition)
			$order_serial_params['order'][$pack_id][1] += $qty;
		else
			$order_serial_params['order'][$pack_id][1] = $qty;

		if($order_serial_params['order'][$pack_id][0] == 0 && $order_serial_params['order'][$pack_id][1] == 0)
			unset($order_serial_params['order'][$pack_id]);

		$updateOrder = new stdClass();
		$updateOrder->order_id = (int)$order_id;
		$updateOrder->order_serial_params = json_encode($order_serial_params);
		$orderClass->save($updateOrder);

		return true;
	}

	public function modifySerial($order_id, $serial_id, $action = true) {
		$orderClass = hikaserial::get('shop.class.order');
		$order = $orderClass->get($order_id);
		if(empty($order))
			return false;

		if($order->order_type != 'sale')
			return false;

		$serial_id = (int)$serial_id;
		$serialClass = hikamarket::get('class.serial');
		$serial = $serialClass->get($serial_id);

		if(empty($serial) || (!empty($serial->serial_order_id) && (int)$serial->serial_order_id != $order_id))
			return false;

		$order_serial_params = array();
		if(!empty($order->order_serial_params))
			$order_serial_params = json_decode($order->order_serial_params, true);

		if(empty($order_serial_params['serial']))
			$order_serial_params['serial'] = array();

		hikaserial::toInteger($order_serial_params['serial']);
		if($action === 1 && in_array($serial_id, $order_serial_params['serial']))
			return false;
		if($action === 0 && !in_array($serial_id, $order_serial_params['serial']))
			return false;

		if($action === true)
			$action = !in_array($serial_id, $order_serial_params['serial']);

		if($action) {
			$order_serial_params['serial'][] = $serial_id;
		} else {
			$order_serial_params['serial'] = array_diff($order_serial_params['serial'], array($serial_id));
		}

		$updateOrder = new stdClass();
		$updateOrder->order_id = (int)$order_id;
		$updateOrder->order_serial_params = json_encode($order_serial_params);
		$orderClass->save($updateOrder);

		return true;
	}

	public function beforeCreateRecurringSuborder($order_id, &$order) {
		$reprocess = false;

		if(!empty($order->order_shipping_id)) {
			$order->order_shipping_id = 0;
			$order->order_shipping_price = 0.0;
			$order->order_shipping_tax = 0.0;
			$order->order_shipping_method = '';

			if(!empty($order->shippings))
				$order->shippings = array();
			if(!empty($order->order_shipping_params->prices))
				$order->order_shipping_params->prices = array();

			foreach($order->order_tax_info as $k => &$v) {
				if(empty($v->tax_amount_for_shipping))
					continue;
				$v->tax_amount -= $v->tax_amount_for_shipping;
				unset($v->tax_amount_for_shipping);
			}
			unset($v);

			$reprocess = true;
		}

		if(!empty($order->order_payment_price)) {
			$order->order_payment_price = 0.0;
			$order->order_payment_tax = 0.0;

			$reprocess = true;
		}

		if(!empty($order->order_discount_code) || !empty($order->order_discount_price)) {
			$order->order_discount_code = '';
			$order->order_discount_price = 0.0;
			$order->order_discount_tax = 0.0;

			$reprocess = true;
		}

		if( $reprocess ) {
			$orderClass = hikaserial::get('shop.class.order');
			$orderClass->recalculateFullPrice($order, $order->products);
		}

		$query = 'SELECT * FROM '.hikaserial::table('order_subscription').' WHERE order_id = '.(int)$order_id;
		$this->db->setQuery($query);
		$subscriptions = $this->db->loadObjectList('order_subscription_id');
		$order->order_serial_params = array(
			'cart_subscriptions' => array()
		);
		foreach($subscriptions as $subscription) {
			if(empty($order->order_serial_params['cart_subscriptions'][ (int)$subscription->subscription_id ]))
				$order->order_serial_params['cart_subscriptions'][ (int)$subscription->subscription_id ] = array();
			$order->order_serial_params['cart_subscriptions'][ (int)$subscription->subscription_id ][] = (int)$subscription->order_product_id;
		}

		$order->order_serial_params = json_encode($order->order_serial_params);
	}

	public function processView(&$view) {
		$viewName = $view->getName();
		$layout = $view->getLayout();

		if($viewName != 'order' || $layout != 'listing')
			return;

		$config = hikaserial::config();
		$user_id = hikaserial::loadUser(false);
		$order_ids = array_keys($view->rows);

		if(empty($order_ids))
			return;

		$display_serial_statuses = $config->get('display_serial_statuses','');
		if(empty($display_serial_statuses)) {
			$display_serial_statuses = array($config->get('used_serial_status','used'));
		} else {
			$display_serial_statuses = explode(',', $display_serial_statuses);
		}
		foreach($display_serial_statuses as &$s) {
			$s = $this->db->Quote($s);
		}
		unset($s);

		$order_statuses = explode(',', $config->get('assignable_order_statuses', 'confirmed,shipped'));
		if(empty($order_statuses) || (count($order_statuses) == 1 && empty($order_statuses[0]))) {
			$shopConfig = hikaserial::config(false);
			$order_statuses = array($shopConfig->get('order_confirmed_status'));
		}
		foreach($order_statuses as &$o) {
			$o = $this->db->Quote($o);
		}
		unset($o);

		$filters = array(
			'serial.serial_status IN ('.implode(',', $display_serial_statuses).')',
			'hk_order.order_id IN (' . implode(',', $order_ids) . ')',
			'hk_order.order_user_id = '.(int)$user_id,
			'hk_order.order_status IN ('.implode(',', $order_statuses).')',
		);
		$query = 'SELECT hk_order.order_id, COUNT(serial.serial_id) as serials '.
			' FROM ' . hikaserial::table('serial') . ' AS serial ' .
			' INNER JOIN '.hikaserial::table('shop.order').' AS hk_order ON serial.serial_order_id = hk_order.order_id '.
			' WHERE ('.implode(') AND (', $filters).') '.
			' GROUP BY hk_order.order_id';
		$this->db->setQuery($query);
		$serial_orders = $this->db->loadObjectList('order_id');

		if(empty($serial_orders))
			return;

		global $Itemid;
		$url_itemid = (!empty($Itemid) ? '&Itemid=' . $Itemid : '');

		foreach($serial_orders as $k => $serial_order) {
			if(!isset($view->rows[$k]))
				continue;

			$view->action_column = true;
			if(empty($view->rows[$k]->actions))
				$view->rows[$k]->actions = array();

			$view->rows[$k]->actions[] = array(
				'name' => JText::_('DOWNLOAD_SERIALS'),
				'link' => hikashop_completeLink('serial&task=download&order_id='.$k.$url_itemid)
			);
		}
	}

	private function getAllOrderProducts(&$order) {
		$order_products = $order->products;
		foreach($order->products as $p) {
			if(!isset($p->bundle))
				continue;
			$order_products = array_merge($order_products, $p->bundle);
		}
		return $order_products;
	}

	public function retrieveConvertedData($table, $field, &$object, $order) {
		if(!isset($object->serial_id) && isset($object->subscription_id)) {
			$object->serial_id = $object->subscription_id;
			$object->serial_data = $object->subscription_data;
			$object->serial_extradata = $object->subscription_extradata;
			$object->serial_status = $object->subscription_status;
			$object->serial_user_id = $object->subscription_user_id;
			$object->serial_assign_date = $object->subscription_start_date;
			$object->serial_order_id = $object->order_id;
			$object->serial_order_product_id = $object->order_product_id;
		}
		$field = trim($field);
		$table = trim($table);
		switch($table) {
			case 'serial':
				if(substr($field, 0, 10) == 'extradata.') {
					$field = substr($field, 10);
					if(is_string($object->serial_extradata))
						$object->serial_extradata = hikaserial::unserialize($object->serial_extradata);
					if(!empty($object->serial_extradata->$field))
						return $object->serial_extradata->$field;
					if(!empty($object->serial_extradata[$field]))
						return $object->serial_extradata[$field];
				} elseif(strpos($field, '.') !== false) {
					list($f1,$f2) = explode('.', $field);
					if(!empty($object->$f1->$f2))
						return $object->$f1->$f2;
				} elseif(!empty($object->$field))
					return $object->$field;
				break;
			case '_':
				return JText::_($field);
				break;
			case 'rawtext':
				return $field;
				break;
			case 'number':
				return $field;
				break;
			case 'order':
				if(!empty($order->$field))
					return $order->$field;
				break;
			case 'customer':
				if(!empty($order->customer->$field))
					return $order->customer->$field;
				break;
			case 'order_product':
				if(empty($object)) return null;
				$opid = 0; $pid = (int)$object->product_id;
				if(!empty($object->serial_order_product_id)) {
					$opid = (int)$object->serial_order_product_id;
				}
				$order_products = $this->getAllOrderProducts($order);
				foreach($order_products as $product) {
					if(($opid > 0 && $product->order_product_id == $opid) || ($opid > 0 && $product->order_product_parent_id == $opid) || ($opid == 0 && $product->product_id == $pid)) {
						if(!empty($product->$field)) {
							if($field == 'order_product_name')
								return strip_tags($product->$field);
							return $product->$field;
						}
						return null;
					}
				}
				break;
			case 'product_price':
			case 'full_product_price':
				if(empty($object)) return null;
				$opid = 0; $pid = (int)$object->product_id;
				if(!empty($object->serial_order_product_id)) {
					$opid = (int)$object->serial_order_product_id;
				}
				$price = null;
				$order_products = $this->getAllOrderProducts($order);
				foreach($order_products as $product) {
					if(($opid > 0 && $product->order_product_id == $opid) || ($opid == 0 && $product->product_id == $pid)) {
						if($price === null) $price = 0.0;
						$price += $product->order_product_price;
						if($field == 'incvat') {
							$price += $product->order_product_tax;
						}
						if($table == 'product_price')
							return $price;
					}
					if($table == 'full_product_price' && $opid > 0 && $product->order_product_option_parent_id == $opid) {
						if($price === null) $price = 0.0;
						$price += $product->order_product_price;
						if($field == 'incvat') {
							$price += $product->order_product_tax;
						}
					}
				}
				if($price != null)
					return $price;
				break;
			case 'product':
				if($field == 'image' || substr($field, 0, 6) == 'image.') {
					if(!isset($this->cache['product_image']))
						$this->cache['product_image'] = array();
					if(!isset($this->cache['product_image'][$object->product_id])) {
						$query = 'SELECT * FROM '.hikaserial::table('shop.file').' as f '.
							' LEFT JOIN '.hikaserial::table('shop.product').' as p ON (f.file_ref_id = '.(int)$object->product_id.' OR f.file_ref_id = p.product_parent_id) '.
							' WHERE file_type = \'product\' AND p.product_id = '.(int)$object->product_id.
							' ORDER BY product_parent_id DESC, file_ordering ASC, file_id ASC';
						$this->db->setQuery($query);
						$this->cache['product_image'][$object->product_id] = $this->db->loadObjectList();
					}
					$images = $this->cache['product_image'][$object->product_id];
					if($field == 'image' || $field == 'image.0') {
						$main_image = reset($images);
						if(!empty($main_image))
							return $main_image->file_path;
					} else {
						list($field, $pos) = explode('.', $field, 2);
						$pos = (int)$pos;
						foreach($images as $k => $v) {
							if($k == $pos)
								return $v->file_path;
						}
					}
				} else {
					if(empty($this->productClass))
						$this->productClass = hikaserial::get('shop.class.product');
					if(!isset($this->cache['product']))
						$this->cache['product'] = array();
					if(!isset($this->cache['product'][$object->product_id]))
						$this->cache['product'][$object->product_id] = $this->productClass->get($object->product_id);
					$product = $this->cache['product'][$object->product_id];
					if(!empty($product) && !empty($product->$field))
						return $product->$field;
				}
				break;
			case 'shipping':
				if(!empty($order->shipping_address->$field))
					return $order->shipping_address->$field;
				break;
			case 'billing':
				if(!empty($order->billing_address->$field))
					return $order->billing_address->$field;
				break;
			case 'option_product':
				if(empty($object)) return null;
				if(strpos($field, '.') === false) return null;
				list($pos, $field) = explode('.', $field, 2);
				$pos = (int)$pos + 1;

				if(empty($this->productClass))
					$this->productClass = hikaserial::get('shop.class.product');
				if(!isset($this->cache['product']))
					$this->cache['product'] = array();
				if(!isset($this->cache['product_image']))
					$this->cache['product_image'] = array();

				$opid = 0; $pid = (int)$object->product_id;
				if(!empty($object->serial_order_product_id)) {
					$opid = (int)$object->serial_order_product_id;
				}
				$order_products = $this->getAllOrderProducts($order);
				if(empty($opid)) {
					foreach($order_products as $product) {
						if($product->product_id == $pid) {
							$opid = (int)$product->order_product_id;
							break;
						}
					}
				}
				if($opid <= 0)
					return null;
				$i = 0;
				foreach($order_products as $product) {
					if($product->order_product_option_parent_id != $opid)
						continue;
					$i++;
					if($i != $pos)
						continue;

					if($field == 'image' || substr($field, 0, 6) == 'image.') {
						if(!isset($this->cache['product_image'][$product->product_id])) {
							$this->db->setQuery('SELECT * FROM '.hikaserial::table('shop.file').' as f LEFT JOIN '.hikaserial::table('shop.product').' as p ON (f.file_ref_id = '.(int)$product->product_id.' OR f.file_ref_id = p.product_parent_id) WHERE file_type = \'product\' AND p.product_id = '.(int)$product->product_id.' ORDER BY product_parent_id DESC, file_ordering ASC, file_id ASC');
							$this->cache['product_image'][$product->product_id] = $this->db->loadObjectList();
						}
						$images = $this->cache['product_image'][$product->product_id];
						if($field == 'image' || $field == 'image.0') {
							$main_image = reset($images);
							if(!empty($main_image))
								return $main_image->file_path;
							return null;
						}
						list($field, $pos) = explode('.', $field, 2);
						$pos = (int)$pos;
						foreach($images as $k => $v) {
							if($k == $pos)
								return $v->file_path;
						}
						return null;
					}

					if(!isset($this->cache['product'][$product->product_id]))
						$this->cache['product'][$product->product_id] = $this->productClass->get($product->product_id);
					$full_product = $this->cache['product'][$product->product_id];
					if(!empty($full_product) && !empty($full_product->$field))
						return $full_product->$field;
					return null;
				}
				break;
			case 'vendor_product':
				if(empty($object)) return null;
				if(!isset($this->cache['product_vendor']))
					$this->cache['product_vendor'] = array();

				if(!defined('HIKAMARKET_COMPONENT')) {
					$marketHelper = rtrim(JPATH_ADMINISTRATOR,DS).DS.'components'.DS.'com_hikamarket'.DS.'helpers'.DS.'helper.php';
					if(!file_exists($marketHelper))
						return null;
					include_once($marketHelper);
					if(!defined('HIKAMARKET_COMPONENT'))
						return null;
				}

				if(!isset($this->cache['product_vendor'][$object->product_id])) {
					$query = 'SELECT * FROM '.hikamarket::table('vendor').' AS v INNER JOIN '.hikamarket::table('shop.product').' AS p ON p.product_vendor_id = v.vendor_id WHERE p.product_id = '.(int)$object->product_id;
					$this->db->setQuery($query);
					$this->cache['product_vendor'][$object->product_id] = $this->db->loadObject();
				}
				$vendor = $this->cache['product_vendor'][$object->product_id];
				if(!empty($vendor) && !empty($vendor->$field))
					return $vendor->$field;

				break;
			case 'option_order':
				if(empty($object)) return null;
				if(strpos($field, '.') === false) return null;
				list($pos, $field) = explode('.', $field, 2);
				$pos = (int)$pos + 1;
				$opid = 0; $pid = (int)$object->product_id;
				if(!empty($object->serial_order_product_id)) {
					$opid = (int)$object->serial_order_product_id;
				}
				$order_products = $this->getAllOrderProducts($order);
				if(empty($opid)) {
					foreach($order_products as $product) {
						if($product->product_id != $pid)
							continue;

						$opid = $product->order_product_id;
						break;
					}
				}
				if($opid <= 0)
					return null;

				$i = 0;
				foreach($order_products as $product) {
					if($opid > 0 && $product->order_product_option_parent_id == $opid)
						$i++;

					if($i != $pos)
						continue;

					if(empty($product->$field))
						return null;
					if($field == 'order_product_name')
						return strip_tags($product->$field);
					return $product->$field;
				}
				break;
			case 'category':
			case 'manufacturer':
				if(empty($object))
					return null;

				$pos = 0;
				if(strpos($field, '.') !== false)
					list($pos, $field) = explode('.', $field, 2);
				$pos = (int)$pos + 1;

				if(!isset($this->cache['category']))
					$this->cache['category'] = array();
				if(!isset($this->cache['category_image']))
					$this->cache['category_image'] = array();

				$cache_key = (($table == 'category') ? 'c_' : 'm_') . (int)$pos . '_' . (int)$object->product_id;

				if(!isset($this->cache['category'][$cache_key])) {
					if($table == 'category') {
						$query = 'SELECT category.* FROM ' . hikaserial::table('shop.category') . ' AS category ' .
							' INNER JOIN ' . hikaserial::table('shop.product_category') . ' AS product_category ON product_category.category_id = category.category_id ' .
							' WHERE product_category.product_id = ' . (int)$object->product_id . ' AND category.category_published = 1 AND category.category_type = ' . $this->db->Quote('product') .
							' ORDER BY ordering ASC, product_category_id ASC';
					} else {
						$query = 'SELECT category.* FROM ' . hikaserial::table('shop.category') . ' AS category ' .
							' INNER JOIN ' . hikaserial::table('shop.product') . ' AS product ON category.category_id = product.product_manufacturer_id ' .
							' WHERE product.product_id = ' . (int)$object->product_id . ' AND category.category_published = 1 AND category.category_type = ' . $this->db->Quote('manufacturer');
					}
					$this->db->setQuery($query, 0, 1);
					$this->cache['category'][$cache_key] = $this->db->loadObject();
				}

				if($field != 'image' && substr($field, 0, 6) != 'image.') {
					$category = $this->cache['category'][$cache_key];
					if(!empty($category) && !empty($category->$field))
						return $category->$field;
					return null;
				}

				if(!isset($this->cache['category_image'][$cache_key])) {
					$category = $this->cache['category'][$cache_key];
					if(empty($category))
						return null;

					$query = 'SELECT * FROM '.hikaserial::table('shop.file').' as f '.
						' WHERE f.file_type = \'category\' AND f.file_ref_id = '.(int)$category->category_id.
						' ORDER BY file_ordering ASC, file_id ASC';
					$this->db->setQuery($query, 0, 1);
					$this->cache['category_image'][$cache_key] = $this->db->loadObject();
				}

				$image = $this->cache['category_image'][$cache_key];
				if(empty($image))
					return null;
				return $image->file_path;
				break;
			case 'entry':
				if(empty($order->entries)) {
					$query='SELECT * FROM '.hikaserial::table('shop.entry').' WHERE order_id='.$order->order_id;
					$this->db->setQuery($query);
					$order->entries = $this->db->loadObjectList();
				}
				if(empty($order->entries))
					return null;

				$opid = 0;
				$pid = (int)$object->product_id;
				if(!empty($object->serial_order_product_id)) {
					$opid = (int)$object->serial_order_product_id;
				}
				$cpt_product = 0;
				$target_product = 0;
				$order_products = $this->getAllOrderProducts($order);
				foreach($order_products as $product) {
					if($product->product_id == $pid)
						$cpt_product += $product->order_product_quantity;
					if($opid > 0 && $opid == $product->order_product_id)
						$target_product = $cpt_product;
				}
				if($target_product <= 0)
					return null;

				$fieldClass = hikaserial::get('shop.class.field');
				$data = new stdClass();
				$entryFields = $fieldClass->getFields('', $data, 'entry');
				$accept_values = array();
				foreach($entryFields as $entryField) {
					if($entryField->field_options['product_id'] == $pid) {
						$accept_values[$entryField->field_namekey] = $entryField->field_options['product_value'];
					}
				}
				$cpt = 0;
				if(empty($object->position))
					$object->position = 1;
				foreach($order->entries as $entry) {
					foreach($accept_values as $key => $value) {
						if($entry->$key != $value && (!is_array($value) || !in_array($entry->$key, $value)))
							continue;

						$cpt++;
						if(($cpt + $object->position - 1) == $target_product) {
							return $entry->$field;
						}
					}
				}
				break;
			default:
				$ret = null;
				JPluginHelper::importPlugin('hikaserial');
				$dispatcher = hikaserial::getDispatcher();
				$dispatcher->trigger('onSerialRetrieveConvertedData', array($table, $field, $object, $order, &$ret));
				return $ret;
		}
		return null;
	}
}
