<?php
/**
 * @package    HikaSerial for Joomla!
 * @version    4.0.2
 * @author     Obsidev S.A.R.L.
 * @copyright  (C) 2011-2023 OBSIDEV. All rights reserved.
 * @license    GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined('_JEXEC') or die('Restricted access');
?><?php
class hikaserialPackClass extends hikaserialClass {

	protected $tables = array('pack');
	protected $pkeys = array('pack_id');
	protected $toggle = array('pack_published' => 'pack_id');

	public function get($element, $default = null) {
		$ret = parent::get($element, $default);
		if(!empty($ret->pack_params))
			$ret->pack_params = hikaserial::unserialize($ret->pack_params);

		return $ret;
	}

	public function save(&$element) {
		$newPack = empty($element->pack_id);

		if(!$newPack)
			$element->old = $this->get($element->pack_id);

		if(!isset($element->pack_params)) {
			$element->pack_params = null;
		} else if(is_string($element->pack_params)) {
			$element->pack_params = hikaserial::unserialize($element->pack_params);
		}

		$do = true;
		$dispatcher = hikaserial::getDispatcher();
		JPluginHelper::importPlugin('hikaserial');
		JPluginHelper::importPlugin('hikashop');
		if($newPack)
			$dispatcher->trigger('onBeforePackCreate', array(&$element, &$do));
		else
			$dispatcher->trigger('onBeforePackUpdate', array(&$element, &$do));

		if(!$do)
			return false;

		$unserializedParams = $element->pack_params;
		if($element->pack_params !== null)
			$element->pack_params = serialize($element->pack_params);

		$ret = parent::save($element);

		$element->pack_params = $unserializedParams;

		if(!$ret)
			return $ret;

		$element->pack_id = $ret;
		if(!empty($element->pack_type) && $element->pack_type == 'subscription') {
			if($element->pack_email_inherit == 1) {
				$this->deletePlanEmails($ret);
			} else {
				$this->savePlanEmails($element);
			}
		}

		if($newPack)
			$dispatcher->trigger('onAfterPackCreate', array(&$element, $ret));
		else
			$dispatcher->trigger('onAfterPackUpdate', array(&$element, $ret));
		return $ret;
	}

	public function getList($group_id, $full = true) {
		if(empty($group_id) || (int)$group_id <= 0)
			return false;
		if(!$full) {
			$query = 'SELECT pack_id FROM '.hikaserial::table('pack').' WHERE pack_type = '.$this->db->Quote('subscription').' AND pack_group_id = '.(int)$group_id;
			$this->db->setQuery($query);
			return $this->db->loadColum();
		}
		$query = 'SELECT * FROM '.hikaserial::table('pack').' WHERE pack_type = '.$this->db->Quote('subscription').' AND pack_group_id = '.(int)$group_id;
		$this->db->setQuery($query);
		return $this->db->loadObjectList('pack_id');
	}

	protected function deletePlanEmails($id) {
		$query = 'DELETE FROM '.hikaserial::table('pack_email').' WHERE pack_id = '.(int)$id;
		$this->db->setQuery($query);
		$this->db->execute();
	}

	public function savePlanEmails(&$element) {
		if(!isset($element->pack_id))
			return false;

		$query = 'SELECT * FROM '.hikaserial::table('pack_email').' WHERE pack_id = '.(int)$element->pack_id;
		$this->db->setQuery($query);
		$pack_emails = $this->db->loadObjectList('pack_email_id');

		$insert = array();
		$update = array();

		$k = 0;
		$emails = array();
		foreach($element->emails as $email_type => $data) {
			if(!in_array($email_type, array('renewal')))
				continue;

			foreach($data as $key => $vals) {
				foreach($vals as $i => $val) {
					$emails[ $k + $i ][$key] = $val;
				}
			}
		}

		foreach($emails as $email) {
			if(empty($email['days']))
				continue;

			if(empty($email['id'])) {
				$insert[] = implode(',', array(
					$this->db->Quote($email_type),
					(int)$email['days'],
					$this->db->Quote($email['template'])
				));
				continue;
			}

			$m_id = (int)$email['id'];
			if(!isset($pack_emails[$m_id]))
				continue;

			$update[ $m_id ] = array(
				$email_type,
				(int)$email['days'],
				$email['template']
			);
		}

		if(!empty($insert)) {
			$query = 'INSERT INTO '.hikaserial::table('pack_email').' (pack_id, pack_email_type, pack_email_days, pack_email_template) '.
				' VALUES ('.(int)$element->pack_id.','.implode('), ('.(int)$element->pack_id.',', $insert).')';
			$this->db->setQuery($query);
			$this->db->execute();
		}

		foreach($update as $k => $up) {
			$query = 'UPDATE '.hikaserial::table('pack_email').' SET '.
				'pack_email_type = '.$this->db->Quote($up[0]).', '.
				'pack_email_days = '.(int)$up[1].', '.
				'pack_email_template = '.$this->db->Quote($up[2]).' '.
				'WHERE pack_id = '.(int)$element->pack_id.' AND pack_email_id = '.(int)$k;
			$this->db->setQuery($query);
			$this->db->execute();
		}

		$delete = array_diff( array_keys($pack_emails), array_keys($update) );
		if(!empty($delete)) {
			$query = 'DELETE FROM '.hikaserial::table('pack_email').
				' WHERE pack_id = '.(int)$element->pack_id.' AND pack_email_id IN ('.implode(',', $delete).')';
			$this->db->setQuery($query);
			$this->db->execute();
		}

		return true;
	}

	public function saveForm() {
		$pack = new stdClass();
		$pack->pack_id = hikaserial::getCID('pack_id');
		$formData = hikaInput::get()->get('data', array(), 'array');
		foreach($formData['pack'] as $col => $val) {
			hikaserial::secureField($col);
			$pack->$col = strip_tags($val);
		}
		$pack->pack_params = null;
		if(!empty($formData['pack_params'])) {
			$pack->pack_params = new stdClass();
			foreach($formData['pack_params'] as $k => $v) {
				hikaserial::secureField($k);
				$pack->pack_params->$k = $v;
			}
		}
		$pack->pack_description = hikaInput::get()->getRaw('pack_description', '');
		$pack->pack_email_inherit = isset($pack->pack_email_inherit) ? (int)$pack->pack_email_inherit : 1;
		if($pack->pack_type == 'subscription') {
			if($pack->pack_email_inherit == 0) {
				$pack->emails = array();
				if(!empty($formData['emails']) && is_array($formData['emails']))
					$pack->emails = $formData['emails'];
			}
			if(!empty($formData['pack_files'])) {
				$pack->files = array();
				if(!empty($formData['pack_file']))
					$pack->files = $formData['pack_file'];
				hikaserial::toInteger($pack->files);
			}
		}
		$status = $this->save($pack);

		if($status) {
			if($pack->pack_type == 'subscription') {
				$this->handlePlanFiles($pack);
			}
		}
		return $status;
	}

	public function frontSaveForm() {
		if(hikaserial::isAdmin() || !hikaserial::initMarket() || !hikamarket::acl('plugins/hikaserial/pack/edit'))
			return false;

		$marketConfig = hikamarket::config();
		$vendor_id = hikamarket::loadVendor(false);

		$pack = new stdClass();
		$pack->pack_id = hikaserial::getCID('pack_id');

		$new = empty($pack->pack_id);
		if($new && !hikamarket::acl('plugins/hikaserial/pack/add'))
			return false;



		$formData = hikaInput::get()->get('data', array(), 'array');
		foreach($formData['pack'] as $col => $val) {
			hikaserial::secureField($col);
			$pack->$col = strip_tags($val);
		}
		unset($pack->pack_params);

		unset($pack->pack_description);
		if(hikamarket::acl('plugins/hikaserial/pack/edit/description')) {
			$pack->pack_description = hikaInput::get()->getRaw('pack_description', '');
			if((int)$marketConfig->get('vendor_safe_product_description', 1)) {
				$safeHtmlFilter = JFilterInput::getInstance(array(), array(), 1, 1);
				$pack->pack_description = $safeHtmlFilter->clean($pack->pack_description, 'string');
			}
		}

		if(hikamarket::acl('plugins/hikaserial/pack/edit/generator')) {
			$packgeneratorType = hikaserial::get('type.pack_generator');
			$gen = $packgeneratorType->get($pack->pack_generator);
			if($gen === null)
				unset($pack->pack_generator);
		} else
			unset($pack->pack_generator);

		if(!hikamarket::acl('plugins/hikaserial/pack/edit/name')) unset($pack->pack_name);
		if(!hikamarket::acl('plugins/hikaserial/pack/edit/data')) unset($pack->pack_data);
		if(!hikamarket::acl('plugins/hikaserial/pack/edit/published')) unset($pack->pack_published);

		unset($pack->pack_manage_access);

		if($vendor_id > 1 || !hikamarket::acl('plugins/hikaserial/pack/edit/vendor'))
			unset($pack->pack_vendor_id);

		if($new && $vendor_id > 1)
			$pack->pack_vendor_id = (int)$vendor_id;

		$status = $this->save($pack);

		if($status) {

		}
		return $status;
	}

	protected function handlePlanFiles(&$plan) {
		$query = 'SELECT * FROM '.hikaserial::table('shop.file').
			' WHERE file_type = '.$this->db->Quote('plan').' AND file_ref_id = '.(int)$plan->pack_id;
		$this->db->setQuery($query);
		$oldFiles = $this->db->loadObjectList('file_id');
		if(empty($oldFiles))
			return;

		$removeFiles = array_diff(array_keys($oldFiles), $plan->files);
		if(empty($removeFiles))
			return;

		$query = 'DELETE FROM '.hikaserial::table('shop.file').
			' WHERE file_type = '.$this->db->Quote('plan').
				' AND file_ref_id = '.(int)$plan->pack_id.
				' AND file_id IN ('.implode(',', $removeFiles).')';
		$this->db->setQuery($query);
		$this->db->execute();
	}
	public function delete($elements) {
		if(!is_array($elements)) {
			$elements = array($elements);
		}
		hikaserial::toInteger($elements);

		$query = 'SELECT serial_pack_id, count(*) as `cpt` FROM '.hikaserial::table('serial').' WHERE serial_pack_id IN ( '.implode(',',$elements).');';
		$this->db->setQuery($query);
		$serialPacks = $this->db->loadObjectList();
		$exclude = array();
		foreach($serialPacks as $serialPack) {
			if($serialPack->cpt > 0) {
				$exclude[] = $serialPack->serial_pack_id;
			}
		}
		if(!empty($exclude)) {
			$elements = array_diff($elements, $exclude);
		}
		return parent::delete($elements);
	}

	public function checkQuantity(&$pack) {
		if(empty($pack->pack_params->stock_level_notify) || $pack->pack_params->stock_level_notify <= 0)
			return true;

		$status = 'free';
		$query = 'SELECT count(*) as qty FROM ' . hikaserial::table('serial') . ' AS a WHERE a.serial_status = '.$this->db->Quote($status).' AND a.serial_pack_id='.$pack->pack_id;
		$this->db->setQuery($query);
		$current_quantity = (int)$this->db->loadResult();

		if((int)$current_quantity > (int)$pack->pack_params->stock_level_notify)
			return true;

		$pack->current_quantity = $current_quantity;

		$send_email = true;

		$dispatcher = hikaserial::getDispatcher();
		JPluginHelper::importPlugin('hikashop');
		JPluginHelper::importPlugin('hikaserial');
		$dispatcher->trigger('onHikaserialPackQuantityLow', array(&$pack, &$send_email));

		if(!$send_email) {
			unset($pack->current_quantity);
			return true;
		}

		$mailClass = hikaserial::get('class.mail');
		$mail = $mailClass->load('pack_quantity_low', $pack);

		if(empty($mail)) {
			unset($pack->current_quantity);
			return true;
		}

		$mail->subject = JText::sprintf($mail->subject, HIKASERIAL_LIVE);
		$shopConfig =& hikaserial::config(false);
		if(!empty($pack->email))
			$mail->dst_email = $pack->email;
		else
			$mail->dst_email = $shopConfig->get('from_email');

		if(!empty($pack->name))
			$mail->dst_name = $pack->name;
		else
			$mail->dst_name = $shopConfig->get('from_name');

		$mailClass->sendMail($mail);

		unset($pack->current_quantity);
		return true;
	}

	public function generate($pack_id, $options = array()) {
		$db = JFactory::getDBO();
		$config = hikaserial::config();

		$pack = $this->get($pack_id);
		if(empty($pack))
			return false;

		$quantity = (int)@$options['number_serials'];
		if($quantity <= 0)
			$quantity = 1;

		$serial_status = @$options['serial_status'];
		if(empty($serial_status)){
			$serial_status = $config->get('unassigned_serial_status', 'free');
		}

		$pluginId = 0;
		$pluginName = substr($pack->pack_generator, 4);
		if(strpos($pluginName,'-') !== false){
			list($pluginName,$pluginId) = explode('-', $pluginName, 2);
			$pack->$pluginName = $pluginId;
		}

		if(empty($pluginName))
			return false;

		$formData = hikaInput::get()->get('data', array(), 'array');

		$serials = array();

		$order = new stdClass();
		$order->hikaserial = new stdClass();
		$order->hikaserial->type = 'generate';
		$order->hikaserial->formData = $formData;

		$plugin = hikaserial::import('hikaserial', $pluginName);
		if(empty($plugin))
			return false;

		if(method_exists($plugin, 'generate')) {
			ob_start();
			$plugin->generate($pack, $order, $quantity, $serials);
			ob_get_clean();
		}

		return $serials;
	}

	public function isVendorPack($pack_id, $vendor_id = -1) {
		static $vendorPackCache = array();

		if(!hikaserial::initMarket())
			return false;

		if($vendor_id === null || $vendor_id == -1) {
			$vendor_id = hikamarket::loadVendor(false, false);
			if($vendor_id == null)
				return false;
		}

		if(!empty($vendor_id) && (int)$vendor_id == 0)
			return false;
		if(!empty($pack_id) && (int)$pack_id <= 0)
			return false;

		if($vendor_id == 1)
			return true;

		if((int)$vendor_id > 1 && (int)$pack_id > 0) {
			if(empty($vendorPackCache[$vendor_id]))
				$vendorPackCache[$vendor_id] = array();

			if(isset($vendorPackCache[$vendor_id][$pack_id]))
				return $vendorPackCache[$vendor_id][$pack_id];

			$db = JFactory::getDBO();
			$query = 'SELECT count(pack.pack_id) '.
				' FROM ' . hikaserial::table('pack') . ' AS pack '.
				' LEFT JOIN '.hikaserial::table('product_pack').' AS pp ON pack.pack_id = pp.pack_id '.
				' LEFT JOIN '.hikaserial::table('shop.product').' AS product ON pp.product_id = product.product_id '.
				' WHERE pack.pack_id = '. (int)$pack_id . ' AND (product.product_vendor_id = ' . (int)$vendor_id . ' OR (product.product_vendor_id = 0 AND pack.pack_vendor_id = ' . (int)$vendor_id . '))';
			$db->setQuery($query);
			$vendorPackCache[$vendor_id][$pack_id] = ((int)$db->loadResult() >= 1);

			return $vendorPackCache[$vendor_id][$pack_id];
		}
		return true;
	}

	public function &getNameboxData($typeConfig, &$fullLoad, $mode, $value, $search, $options) {

		$ret = array(
			0 => array(),
			1 => array()
		);

		$db = JFactory::getDBO();
		$app = JFactory::getApplication();

		$start = 0;
		if(!empty($options['start']))
			$start = $options['start'];

		$limit = (int)@$typeConfig['limit'];
		if(!empty($options['limit']))
			$limit = (int)$options['limit'];
		if(empty($limit))
			$limit = 30;

		$filters = array(
			'published' => 'pack_published = 1'
		);

		if(!empty($typeConfig['params']['pack_type'])) {
			if($typeConfig['params']['pack_type'] == 'serial') {
				$filters['pack_type'] = 'pack_type IN ('.$db->Quote('serial').','.$db->Quote('').')';
			} else {
				$filters['pack_type'] = 'pack_type = '.$db->Quote($typeConfig['params']['pack_type']);
			}
		}

		if(!empty($search)) {
			$searchStr = "'%" . ((HIKASHOP_J30) ? $db->escape($search, true) : $db->getEscaped($search, true) ) . "%'";
			$filters['search'] = 'pack_name LIKE ' . $searchStr;
		}

		if(!hikaserial::isAdmin() && hikaserial::initMarket()) {
			$vendor = hikamarket::loadVendor(true);
			if($vendor->vendor_id > 1) {
				$acl = array();
				$accesses = explode(',', $vendor->vendor_access);
				foreach($accesses as $ax) {
					if(substr($ax,0,1) != '@')
						continue;
					$ax_id = (int)substr($ax,1);
					if($ax_id > 0)
						$acl[] = $ax_id;
				}

				if(empty($acl)) {
					$filters['vendor'] = '(pack_vendor_id = '.(int)$vendor->vendor_id.') OR (pack_vendor_id = 0 AND pack_manage_access = \'all\')';
				} else {
					$filters['vendor'] = '(pack_vendor_id = '.(int)$vendor->vendor_id.') OR (pack_vendor_id = 0 AND (pack_manage_access = \'all\' OR pack_manage_access LIKE \'%,'.implode(',%\' OR pack_manage_access LIKE \'%,', $acl).',%\'))';
				}
			}
		}

		$query = 'SELECT pack_id, pack_name, pack_data, pack_generator, pack_type '.
			' FROM ' . hikaserial::table('pack') .
			' WHERE ('.implode(') AND (', $filters).')'.
			' ORDER BY pack_name';

		$db->setQuery($query, $start, $limit);
		$packs = $db->loadObjectList('pack_id');

		foreach($packs as $pack) {
			$ret[0][$pack->pack_id] = $pack;
		}

		if(count($packs) == $limit)
			$fullLoad = false;

		if(!empty($value)) {
			if(!is_array($value))
				$value = array($value);

			if($fullLoad) {
				foreach($value as $v) {
					if(isset($ret[0][(int)$v]))
						$ret[1][(int)$v] = $ret[0][(int)$v];
				}
			} else {
				$values = array_merge($value);
				hikaserial::toInteger($values);

				$query = 'SELECT pack_id, pack_name, pack_data, pack_generator '.
					' FROM ' . hikaserial::table('pack') .
					' WHERE pack_id IN ('.implode(',', $values).') '.
					' ORDER BY pack_name';
				$db->setQuery($query);
				$ret[1] = $db->loadObjectList('pack_id');
			}
		}

		return $ret;
	}
}
