<?php
/**
 * @package    HikaSerial for Joomla!
 * @version    4.0.2
 * @author     Obsidev S.A.R.L.
 * @copyright  (C) 2011-2023 OBSIDEV. All rights reserved.
 * @license    GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined('_JEXEC') or die('Restricted access');
?><?php
class hikaserialProductClass extends hikaserialClass {

	public function saveForm(&$product) {
		if(!isset($product->product_id))
			return;
		$product_id = (int)$product->product_id;

		if(!hikaserial::isAdmin()) {
			if(!hikaserial::initMarket())
				return;

			$process_serials = hikamarket::acl('product/edit/plugin/hikaserial');
			$process_subscriptions = hikaserial::level(1) && hikamarket::acl('product/edit/plugin/hikasubscription');
		} else {
			$process_serials = true;
			$process_subscriptions = hikaserial::level(1);
		}

		$formData = hikaInput::get()->get('data', array(), 'array');
		if(empty($formData))
			return;

		if(!empty($formData['hikaserial']['form']) && $process_serials)
			$this->saveFormSerials($product_id, $formData);
		if(!empty($formData['hikasubscription']['form']) && $process_subscriptions)
			$this->saveFormSubscriptions($product_id, $formData);
	}

	private function saveFormSerials($product_id, $formData) {
		$serialData = $formData['hikaserial'];

		if(isset($serialData[$product_id]))
			$serialData = $serialData[$product_id];
		else if(isset($serialData[0]))
			$serialData = $serialData[0];
		else
			$serialData = array();

		$packs = array();
		if(isset($serialData['pack_qty']) && isset($serialData['pack_id'])) {
			$packs = array_combine($serialData['pack_id'], $serialData['pack_qty']);
		}

		$query = 'DELETE FROM ' . hikaserial::table('product_pack') . ' WHERE product_id = ' . $product_id;
		if(hikaserial::level(1)) {
			$query = 'DELETE pp.* FROM '. hikaserial::table('product_pack') . ' as pp ' .
				' LEFT JOIN ' . hikaserial::table('pack') .' as pack ON pp.pack_id = pack.pack_id ' .
				' WHERE pp.product_id = ' . $product_id.' AND (pack.pack_id IS NULL OR pack.pack_type IN (\'\',\'serial\'))';
		}
		$this->db->setQuery($query);
		$this->db->execute();

		if(empty($packs))
			return;

		$query = 'INSERT IGNORE INTO ' . hikaserial::table('product_pack') . ' (`product_id`, `pack_id`, `quantity`) VALUES ';
		$data = array();
		foreach($packs as $id => $qty) {
			if((int)$qty > 0) {
				$data[] = '(' . $product_id . ', ' . (int)$id . ', ' .(int)$qty . ')';
			}
			if(count($data) >= 50) {
				$this->db->setQuery($query . implode(',', $data));
				$this->db->execute();
				$data = array();
			}
		}
		if(count($data) > 0) {
			$this->db->setQuery($query . implode(',', $data));
			$this->db->execute();
			$data = array();
		}

		$this->refreshQuantity( (int)$product_id );
	}

	private function saveFormSubscriptions($product_id, $formData) {
		$subscriptionData = $formData['hikasubscription'];

		if(isset($subscriptionData[$product_id]))
			$subscriptionData = $subscriptionData[$product_id];
		else if(isset($subscriptionData[0]))
			$subscriptionData = $subscriptionData[0];
		else
			$subscriptionData = array();

		$plans = array();
		foreach($subscriptionData as $k => $data) {
			foreach($data as $i => $v) {
				if(empty($plans[$i]))
					$plans[$i] = array();
				$plans[$i][$k] = $v;
			}
		}

		$query = 'DELETE pp.* FROM '. hikaserial::table('product_pack') . ' as pp ' .
			' LEFT JOIN ' . hikaserial::table('pack') .' as pack ON pp.pack_id = pack.pack_id ' .
			' WHERE pp.product_id = ' . $product_id.' AND (pack.pack_id IS NULL OR pack.pack_type = \'subscription\')';
		$this->db->setQuery($query);
		$this->db->execute();

		if(empty($plans))
			return;

		$planRelationType = hikaserial::get('type.plan_relation');
		$plan_relations = $planRelationType->load();
		$plan_relations = array_keys($plan_relations);

		$query = 'INSERT IGNORE INTO ' . hikaserial::table('product_pack') . ' (`product_id`, `pack_id`, `quantity`, `relation_type`, `relation_duration`, `relation_upgrade`) VALUES ';
		$plan_data = array();
		foreach($plans as $plan) {
			if(!in_array($plan['relation_type'], $plan_relations))
				continue;

			$duration_type = substr($plan['relation_duration'], -1);
			$plan['relation_duration'] = (int)$plan['relation_duration'];
			if(in_array($duration_type, array('d','w','m','y')))
				$plan['relation_duration'] .= $duration_type;

			$relation_upgrade = 0;
			if($plan['relation_type'] == 'upgrade')
				$relation_upgrade = (int)$plan['relation_upgrade'];

			$plan_data[] = '(' . $product_id . ',' . (int)$plan['pack_id'] . ',1,' . $this->db->Quote($plan['relation_type']) . ',' . $this->db->Quote($plan['relation_duration']) . ',' . (int)$relation_upgrade . ')';
			if(count($data) >= 50) {
				$this->db->setQuery($query . implode(',', $plan_data));
				$this->db->execute();
				$plan_data = array();
			}
		}
		if(count($plan_data) > 0) {
			$this->db->setQuery($query . implode(',', $plan_data));
			$this->db->execute();
			$plan_data = array();
		}
	}

	public function refreshQuantities() {
		return $this->refreshQuantity();
	}

	public function refreshQuantity($product = null, $packs = null) {
		$config = hikaserial::config();
		if($config->get('link_product_quantity', false) == false)
			return;

		$filters = array(
			'product_pack.qty' => 'pp.quantity != 0',
			'pack_type' => 'pa.pack_type IN ('.$this->db->Quote('serial').','.$this->db->Quote('').')',
		);

		if($product !== null) {
			if(is_object($product))
				$product_id = (int)$product->product_id;
			else
				$product_id = (int)$product;

			if(!empty($product_id))
				$filters['product_id'] = 'pp.product_id = ' . (int)$product_id;
		}

		if($packs !== null) {
			$pack_filter = '';
			if(is_array($packs)) {
				hikaserial::toInteger($packs);
				$pack_filter = 'pp.pack_id IN (' . implode(',', $packs) . ')';
			}
			if(is_numeric($packs)) {
				$pack_filter = 'pp.pack_id = ' . (int)$packs;
			}

			if(!empty($pack_filter)) {
				$query = 'SELECT DISTINCT pp.product_id FROM ' . hikaserial::table('product_pack') . ' AS pp WHERE ' . $pack_filter;
				$this->db->setQuery($query);
				$products = $this->db->loadColumn();

				hikaserial::toInteger($products);
				if($product !== null) {
					if(is_object($product))
						$products[] = (int)$product->product_id;
					else
						$products[] = (int)$product;
				}

				if(empty($products))
					return;

				$filters['product_id'] = 'pp.product_id IN (' . implode(',', $products) . ')';

				unset($products);
				unset($pack_filter);
			}
		}

		$query = 'SELECT pp.product_id, pa.pack_id, pp.quantity as pack_qty, pa.pack_generator, pa.pack_params ' .
			'FROM ' . hikaserial::table('product_pack') . ' AS pp ' .
			'INNER JOIN ' . hikaserial::table('pack') . ' AS pa ON pp.pack_id = pa.pack_id ' .
			'WHERE (' . implode(') AND (', $filters) . ') '.
			'ORDER BY pp.product_id ASC, pp.quantity ASC';
		$this->db->setQuery($query);
		$ret = $this->db->loadObjectList();

		$pack_ids = array();
		foreach($ret as $p) {
			if(empty($p->pack_generator))
				$pack_ids[ (int)$p->pack_id ] = (int)$p->pack_id;
		}

		$stats = array();
		if(!empty($pack_ids)) {
			$query = 'SELECT s.serial_pack_id, COUNT(s.serial_id) AS serials FROM '.hikaserial::table('serial') . ' AS s ' .
				' WHERE s.serial_pack_id IN ('.implode(',', $pack_ids).') AND s.serial_status = \'free\' ' .
				' GROUP BY s.serial_pack_id';
			$this->db->setQuery($query);
			$stats = $this->db->loadObjectList('serial_pack_id');
		}

		$products = array();
		foreach($ret as &$p) {
			$p->serials = isset($stats[(int)$p->pack_id]) ? $stats[(int)$p->pack_id]->serials : 0;

			if(!empty($p->pack_generator)) {
				$p->qty = -1;
			} else {
				$p->qty = (int)floor((int)$p->serials / (int)$p->pack_qty);
			}

			if(!empty($p->pack_params)) {
				$p->pack_params = hikaserial::unserialize($p->pack_params);
				if(!empty($p->pack_params->unlimited_quantity))
					$p->qty = -1;
			}

			if(!isset($products[(int)$p->product_id])) {
				$products[(int)$p->product_id] = $p->qty;
			} else {
				$q = (int)$products[(int)$p->product_id];
				if($p->qty >= 0 && $p->qty < $q)
					$products[(int)$p->product_id] = $p->qty;
			}
		}
		unset($p);
		unset($stats);

		JPluginHelper::importPlugin('hikashop');
		JPluginHelper::importPlugin('hikaserial');
		$dispatcher = hikaserial::getDispatcher();
		$dispatcher->trigger('onSerialRefreshQuantity', array(&$products, $ret));

		unset($ret);

		if(empty($products))
			return;

		$product_ids = array_keys($products);
		hikaserial::toInteger($product_ids);
		$query = 'SELECT p.product_id, p.product_quantity ' .
			' FROM ' . hikaserial::table('shop.product') . ' AS p ' .
			' WHERE p.product_id IN (' . implode(',', $product_ids) . ')';
		$this->db->setQuery($query);
		$products_qty = $this->db->loadObjectList('product_id');

		foreach($products as $product_id => $qty) {
			if(!isset($products_qty[ (int)$product_id ]))
				continue;
			if((int)$products_qty[ (int)$product_id]->product_quantity == (int)$qty)
				continue;

			$this->db->setQuery('UPDATE ' . hikaserial::table('shop.product') . ' SET product_quantity = ' . (int)$qty . ' WHERE product_id = ' . (int)$product_id);
			$this->db->execute();
		}
		unset($products);

		return;
	}

	public function processView(&$view) {
		$shopConfig = hikashop_config();
		$shopVersion = $shopConfig->get('version', '1.0.0');
		if(version_compare($shopVersion, '3.4.1', '<'))
			return;

		$layout = $view->getLayout();
		if($layout == 'listing')
			return $this->processViewListing($view);
		if($layout == 'show')
			return $this->processViewShow($view);
	}

	protected function getRelatedPlans($product_ids) {
		$relations = array(
			$this->db->Quote('upgrade'),
			$this->db->Quote('renew'),
			$this->db->Quote('renew_create'),
		);
		$query = 'SELECT pp.*, pack.* '.
			' FROM ' . hikaserial::table('product_pack') . ' as pp '.
			' INNER JOIN ' . hikaserial::table('pack') . ' as pack ON pp.pack_id = pack.pack_id '.
			' WHERE pack.pack_published = 1 AND pack.pack_type = '.$this->db->Quote('subscription').
				' AND pp.relation_type IN ('.implode(',', $relations).') '.
				' AND pp.product_id IN (' . implode(',', $product_ids) . ')'.
			' ORDER BY pp.pack_id ASC';
		$this->db->setQuery($query);
		$plans = $this->db->loadObjectList();

		if(empty($plans))
			return false;
		return $plans;
	}

	public function processViewShow(&$view) {

		$product_ids = array( (int)$view->element->product_id );
		$plans = $this->getRelatedPlans($product_ids);
		if(empty($plans))
			return;

		$renew = array();
		$upgrade = array();
		foreach($plans as $plan) {
			if(in_array($plan->relation_type, array('renew', 'renew_create')))
				$renew[(int)$plan->pack_id] = (int)$plan->pack_id;

			if($plan->relation_type == 'upgrade' && !empty($plan->relation_upgrade))
				$upgrade[(int)$plan->relation_upgrade] = (int)$plan->relation_upgrade;
		}
		if(empty($renew) && empty($upgrade))
			return;

		$subscriptionClass = hikaserial::get('class.subscription');

		$renewals = array();
		if(!empty($renew)) {
			$renewals = $subscriptionClass->listRenewableSubscriptions( $renew );
		}
		$upgrades = array();
		if(!empty($upgrade)) {
			$upgrades = $subscriptionClass->listUpgradableSubscriptions( $upgrade );
		}

		$config = hikaserial::config();
		$position = $config->get('subscription_selector_position', 'rightMiddle');
		if(empty($position))
			$position = 'rightMiddle';

		if(empty($view->element->extraData))
			$view->element->extraData = new stdClass();
		if(empty($view->element->extraData->$position))
			$view->element->extraData->$position = array();

		$params = new stdClass();
		$params->renewals = $renewals;
		$params->upgrades = $upgrades;
		$params->plans = $plans;
		$params->product =& $view->element;

		$content = hikaserial::getLayout('productserial', 'product_show', $params, $js);
		if(!empty($content))
			$view->element->extraData->{$position}[] = $content;
		unset($content);
	}

	public function processViewListing(&$view) {

		$product_ids = array();
		foreach($view->rows as $row) {
			$product_ids[ (int)$row->product_id ] = (int)$row->product_id;
		}
		if(empty($product_ids))
			return;

		$plans = $this->getRelatedPlans($product_ids);
		if(empty($plans))
			return;
	}
}
