<?php
/**
 * @package    HikaSerial for Joomla!
 * @version    4.0.2
 * @author     Obsidev S.A.R.L.
 * @copyright  (C) 2011-2023 OBSIDEV. All rights reserved.
 * @license    GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined('_JEXEC') or die('Restricted access');
?><?php
class hikaserialQueueClass extends hikaserialClass {

	protected $tables = array('queue');
	protected $pkeys = array('queue_id');
	protected $mailClass = null;

	public function generateQueue($last_cron = null, $now = null, $save_cfg = true) {
		$config = hikaserial::config();

		if($now === null)
			$now = time();
		if($last_cron === null)
			$last_cron = (int)$config->get('cron_last_queue', 0);

		$max_day_diff = max((int)$config->get('cron_queue_max_diff', 5), 0);
		if($max_day_diff > 0 && ($now - $last_cron > ($max_day_diff * 86400))) {
			$last_cron = $now - ($max_day_diff * 86400);
		}

		$query = 'INSERT INTO '.hikaserial::table('queue').' (queue_subscription_id, queue_email_id, queue_type, queue_created) '.
			' SELECT subscription.subscription_id, pack_email.pack_email_id, pack_email.pack_email_type, '.$now.' ' .
			' FROM '.hikaserial::table('subscription').' AS subscription ' .
			' INNER JOIN '.hikaserial::table('pack').' AS pack ON subscription.subscription_pack_id = pack.pack_id ' .
			' INNER JOIN '.hikaserial::table('pack_email').' AS pack_email ON ((pack.pack_email_inherit = 0 AND pack_email.pack_id = pack.pack_id) OR (pack.pack_email_inherit = 1 AND pack_email.pack_id = 0))'.
			' WHERE ' .
				' pack.pack_published = 1 ' .
				' AND subscription.subscription_end_date > ('.$last_cron.' + (pack_email.pack_email_days * 86400)) ' .
				' AND subscription.subscription_end_date <= ('.$now.' + (pack_email.pack_email_days * 86400))';
		$this->db->setQuery($query);
		$ret = $this->db->execute();



		if($save_cfg) {
			$cfg = array('cron_last_queue' => $now);
			$config->save($cfg);
		}

		return $ret;
	}

	public function processQueue($limit = null) {
		$config = hikaserial::config();

		if(empty($limit) || (int)$limit <= 0)
			$limit = max(5, (int)$config->get('email_batch_size', 10));

		$lock_period = max(600, (int)$config->get('queue_lock_period', 1800)); // minimum: 10min | maximum: 30min

		$now = time();
		$lock_limit = $now - $lock_period;

		$query = 'SELECT q.*, e.* FROM '.hikaserial::table('queue').' AS q '.
			' LEFT JOIN '.hikaserial::table('pack_email').' AS e ON q.queue_email_id = e.pack_email_id '.
			' WHERE q.queue_lock = 0 OR q.queue_lock < '.$lock_limit.' '.
			' ORDER BY q.queue_created';
		$this->db->setQuery($query, 0, $limit);
		$items = $this->db->loadObjectList('queue_id');

		if(empty($items))
			return 0;

		$ids = array_keys($items);

		$query = 'UPDATE '.hikaserial::table('queue').' SET queue_lock = '.$now.' WHERE queue_id IN ('.implode(',', $ids).')';
		$this->db->setQuery($query);
		$this->db->execute();

		foreach($items as $item) {
			$this->processQueueItem($item);
		}

		return count($items);
	}

	protected function processQueueItem($item) {
		if(empty($this->mailClass))
			$this->mailClass = hikaserial::get('class.mail');
		if(empty($this->subscriptionClass))
			$this->subscriptionClass = hikaserial::get('class.subscription');

		$queue_id = (int)$item->queue_id;

		$data = new stdClass();
		$data->queue = $item;

		if(!empty($data->queue->pack_email_params))
			$data->queue->pack_email_params = hikaserial::unserialize($data->queue->pack_email_params);

		$data->subscription = $this->subscriptionClass->get( (int)$item->queue_subscription_id );
		if(empty($data->subscription)) {
			$this->deleteQueueItem($queue_id);
			return;
		}

		if(empty($this->packClass))
			$this->packClass = hikaserial::get('class.pack');
		$data->plan = $this->packClass->get($data->subscription->subscription_pack_id);

		if(empty($this->userClass))
			$this->userClass = hikaserial::get('shop.class.user');
		$data->user = $this->userClass->get($data->subscription->subscription_user_id);
		unset($data->user->password);

		if($item->pack_email_type == 'renewal') {
			$item->dest_email = $data->user->user_email;
			$item->dest_name = $data->user->name;
		}

		$mail = $this->mailClass->load($data->queue->pack_email_template, $data);
		if(empty($mail)) {
			$this->deleteQueueItem($queue_id);
			return false;
		}

		$shopConfig =& hikaserial::config(false);

		$mail->subject = JText::sprintf($mail->subject, HIKASERIAL_LIVE);
		$mail->dst_email = !empty($item->dest_email) ? $item->dest_email : $shopConfig->get('from_email');
		$mail->dst_name = !empty($item->dest_name) ? $item->dest_name : $shopConfig->get('from_name');

		$this->mailClass->sendMail($mail);

		$this->deleteQueueItem($queue_id);

		return true;
	}

	protected function deleteQueueItem($queue_id) {
		$this->db->setQuery('DELETE FROM '.hikaserial::table('queue').' WHERE queue_id = '.(int)$queue_id);
		$this->db->execute();
		return true;
	}
}
