<?php
/**
 * @package    HikaSerial for Joomla!
 * @version    4.0.2
 * @author     Obsidev S.A.R.L.
 * @copyright  (C) 2011-2023 OBSIDEV. All rights reserved.
 * @license    GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined('_JEXEC') or die('Restricted access');
?><?php
include_once HIKASHOP_CLASS.'statistics.php';

class hikaserialStatisticsClass extends hikashopStatisticsClass {
	public function __construct($config = array()) {
		parent::__construct($config);

		$this->serialConfig = hikaserial::config();
	}

	public function getDashboard($place = null) {

		$now = time();
		$order_type = $this->db->Quote('sale');
		$valid_order_statuses = explode(',', $this->config->get('stats_valid_order_statuses', 'confirmed,shipped'));
		$invoice_order_statuses = $this->config->get('invoice_order_statuses', 'confirmed,shipped');
		$created_status = empty($invoice_order_statuses);
		foreach($valid_order_statuses as &$status) {
			if(trim($status) == 'created')
				$created_status = true;
			$status = $this->db->Quote(trim($status));
		}
		unset($status);

		$valid_serial_statuses = array(
			$this->db->Quote($this->serialConfig->get('used_serial_status', 'used')),
			$this->db->Quote($this->serialConfig->get('assigned_serial_status', 'assigned')),
		);

		$ret = array();

		$ret['serial_count'] = array(
			'slot' => 0,
			'order' => 1,
			'published' => 1,
			'class' => 'hkc-lg-3 hkc-sm-6',
			'label' => JText::_('HIKA_STATS_TOTAL_SERIAL'),

			'format' => 'number',
			'type' => 'tile',
			'tile' => array(
				'icon' => array('type' => 'fa', 'value' => 'barcode'),
				'view' => hikaserial::completeLink('serial&task=listing'),
			),
			'query' => array(
				'get' => 'single',
				'select' => 'COUNT(*) as value',
				'tables' => hikaserial::table('serial') . ' AS hk_serial ',
				'filters' => array(
					'serial_status' => 'hk_serial.serial_status IN ('.implode(',', $valid_serial_statuses).')'
				)
			)
		);
		$ret['pack_count'] = array(
			'slot' => 0,
			'order' => 2,
			'published' => 1,
			'class' => 'hkc-lg-3 hkc-sm-6',
			'label' => JText::_('HIKA_STATS_TOTAL_PACK'),

			'format' => 'number',
			'type' => 'tile',
			'tile' => array(
				'icon' => array('type' => 'fa', 'value' => 'folder-open'),
				'view' => hikaserial::completeLink('pack&task=listing'),
			),
			'query' => array(
				'get' => 'single',
				'select' => 'COUNT(*) as value',
				'tables' => hikaserial::table('pack') . ' AS hk_pack ',
				'filters' => array(
					'pack_published' => 'hk_pack.pack_published = 1',
					'pack_type' => 'hk_pack.pack_type IN ('.$this->db->Quote('').','.$this->db->Quote('serial').')',
				)
			)
		);
		$ret['subscription_count'] = array(
			'slot' => 0,
			'order' => 3,
			'published' => 1,
			'class' => 'hkc-lg-3 hkc-sm-6',
			'label' => JText::_('HIKA_STATS_TOTAL_SUBSCRIPTION'),

			'format' => 'number',
			'type' => 'tile',
			'tile' => array(
				'icon' => array('type' => 'fa', 'value' => 'users'),
				'view' => hikaserial::completeLink('subscription&task=listing'),
			),
			'query' => array(
				'get' => 'single',
				'select' => 'COUNT(*) as value',
				'tables' => hikaserial::table('subscription') . ' AS hk_subscription ',
				'filters' => array(
					'subscription_status' => 'hk_subscription.subscription_status = '.$this->db->Quote('active'),
					'subscription_valid' => 'hk_subscription.subscription_end_date = 0 OR hk_subscription.subscription_end_date > '.$now,
				)
			)
		);
		$ret['plan_count'] = array(
			'slot' => 0,
			'order' => 4,
			'published' => 1,
			'class' => 'hkc-lg-3 hkc-sm-6',
			'label' => JText::_('HIKA_STATS_TOTAL_PLAN'),

			'format' => 'number',
			'type' => 'tile',
			'tile' => array(
				'icon' => array('type' => 'fa', 'value' => 'folder-open'),
				'view' => hikaserial::completeLink('plan&task=listing'),
			),
			'query' => array(
				'get' => 'single',
				'select' => 'COUNT(*) as value',
				'tables' => hikaserial::table('pack') . ' AS hk_pack ',
				'filters' => array(
					'pack_published' => 'hk_pack.pack_published = 1',
					'pack_type' => 'hk_pack.pack_type = '.$this->db->Quote('subscription'),
				)
			)
		);

		if(!empty($place) && is_string($place) && preg_match('#^[_a-zA-Z0-9]+$#', $place)) {
			$blocks = $this->serialConfig->get('statistics_'.$place, null);
			if($blocks !== null) {
				$blocks = explode(',', $blocks);
				$removed = array_diff(array_keys($ret), $blocks);
				foreach($removed as $r) {
					unset($ret[$r]);
				}
			}
		}

		JPluginHelper::importPlugin('hikaserial');
		$dispatcher = hikaserial::getDispatcher();
		$extra_list = $dispatcher->trigger('onHikaserialStatisticPluginList', array(
			array(
				'created' => $created_status,
				'valid' => $valid_order_statuses,
				'offset' => $this->timeoffsetStr,
				'place' => $place
			)
		));
		if(!empty($extra_list)) {
			foreach($extra_list as $v) {
				if(!isset($v['name']))
					continue;
				$k = 'plugin.'.$v['name'];
				$ret[$k] = $v;
			}
		}

		return $ret;
	}
}
