<?php
/**
 * @package    HikaSerial for Joomla!
 * @version    4.0.2
 * @author     Obsidev S.A.R.L.
 * @copyright  (C) 2011-2023 OBSIDEV. All rights reserved.
 * @license    GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined('_JEXEC') or die('Restricted access');
?><?php
class configController extends hikaserialController {

	protected $rights = array(
		'display' => array('display','config','cancel','listing','sql','language'),
		'add' => array(),
		'edit' => array('toggle','delete'),
		'modify' => array('save','apply','share','send','savelanguage'),
		'delete' => array()
	);

	public function __construct($config = array()) {
		parent::__construct($config);
		$this->registerDefaultTask('config');
	}

	public function display($tpl = null, $params = null) {
		hikaInput::get()->set('layout', 'config');
		return parent::display();
	}

	public function sql() {
		$user = JFactory::getUser();
		$iAmSuperAdmin = false;
		$iAmSuperAdmin = $user->authorise('core.admin');
		hikaInput::get()->set('layout', 'sql');
		if(!$iAmSuperAdmin) {
			$app = JFactory::getApplication();
			$app->enqueueMessage(JText::sprintf('HIKA_SUPER_ADMIN_REQUIRE_FOR_TASK', 'sql'), 'error'); // _('HIKA_SUPER_ADMIN_REQUIRE_FOR_TASK')
			hikaInput::get()->set('layout', 'config');
		}
		return parent::display();
	}

	public function cancel(){
		$this->setRedirect(hikaserial::completeLink('dashboard', false, true));
	}

	public function save() {
		$this->store();
		return $this->cancel();
	}

	public function apply() {
		$this->store();
		return $this->display();
	}

	public function store() {
		hikaserial::checkToken() || die('Invalid Token');
		$app = JFactory::getApplication();
		$config = hikaserial::config();

		$formData = hikaInput::get()->post->get('config', array(), 'array');
		$formData['display_serial_statuses'] = implode(',', $formData['display_serial_statuses']);
		if(is_array($formData['assignable_order_statuses']))
			$formData['assignable_order_statuses'] = implode(',', $formData['assignable_order_statuses']);
		if(is_array($formData['useable_serial_statuses']))
			$formData['useable_serial_statuses'] = implode(',', $formData['useable_serial_statuses']);

		$status = $config->save($formData);

		$config->load();

		if(!$status) {
			$app->enqueueMessage(JText::_( 'ERROR_SAVING' ), 'error');
			return;
		}

		$app->enqueueMessage(JText::_( 'HIKASHOP_SUCC_SAVED' ), 'message');
		$formData = hikaInput::get()->post->get('data', array(), 'array');
		if(empty($formData['emails']))
			return;

		$configPlan = new stdClass();
		$configPlan->pack_id = 0;
		$configPlan->emails = $formData['emails'];

		$packClass = hikaserial::get('class.pack');
		$packClass->savePlanEmails( $configPlan );
	}

	public function language() {
		hikaInput::get()->set('layout', 'language');
		return parent::display();
	}

	public function savelanguage() {
		hikaserial::checkToken() || die('Invalid Token');
		$this->savelng();
		return $this->language();
	}

	public function share(){
		hikaserial::checkToken() || die('Invalid Token');
		if($this->savelng()) {
			hikaInput::get()->set('layout', 'share');
			return parent::display();
		}
		return $this->language();
	}

	public function send() {
		hikaserial::checkToken() || die('Invalid Token');

		$code = hikaInput::get()->getString('code');
		hikaInput::get()->set('code', $code);
		if(empty($code))
			return;

		$bodyEmail = hikaInput::get()->getString('mailbody');
		$true = true;

		$mailClass = hikaserial::get('shop.class.mail');
		$shopConfig = hikaserial::config(false);
		$config = hikaserial::config();
		$user = hikaserial::loadUser(true);

		$addedName = $shopConfig->get('add_names',true) ? $mailClass->cleanText(@$user->name) : '';
		$mail = $mailClass->get('language', $true);
		$mailClass->mailer->AddAddress($user->user_email, $addedName);
		$mailClass->mailer->AddAddress('translate-hikaserial@hikashop.com', 'Hikaserial Translation Team');

		$mail->subject = '[HIKASERIAL LANGUAGE FILE] ' . $code;
		$mail->altbody = 'The website '.HIKASHOP_LIVE.' using HikaSerial '.$config->get('level') . $config->get('version') . ' sent a language file : '.$code;
		$mail->altbody .= "\n\n\n" . $bodyEmail;
		$mail->html = 0;

		jimport('joomla.filesystem.file');
		$path = JPath::clean(hikashop_getLanguagePath(JPATH_ROOT) . DS . $code . DS . $code . '.' . HIKASERIAL_COMPONENT . '.ini');
		$mailClass->mailer->AddAttachment($path);
		$result = $mailClass->sendMail($mail);

		if($result) {
			hikaserial::display(JText::_('THANK_YOU_SHARING'), 'success');
		}
	}

	private function savelng(){
		jimport('joomla.filesystem.file');
		jimport('joomla.filesystem.folder');

		$code = hikaInput::get()->getString('code');
		hikaInput::get()->set('code', $code);
		$content = hikaInput::get()->getRaw('content', '');
		if(empty($code))
			return;
		$content_override = hikaInput::get()->getRaw('content_override', '');
		$folder = hikashop_getLanguagePath(JPATH_ROOT) . DS . 'overrides';
		if(!JFolder::exists($folder)) {
			JFolder::create($folder);
		}
		if(JFolder::exists($folder)) {
			$path = $folder . DS . $code . '.override.ini';
			$result = JFile::write($path, $content_override);
			if(!$result) {
				hikaserial::display(JText::sprintf('FAIL_SAVE', $path), 'error');
			}
		}

		if(empty($content))
			return;
		$path = hikashop_getLanguagePath(JPATH_ROOT) . DS . $code . DS . $code . '.' . HIKASERIAL_COMPONENT . '.ini';
		$result = JFile::write($path, $content);
		if($result) {
			hikaserial::display(JText::_('HIKASHOP_SUCC_SAVED'), 'success');
			$updateHelper = hikaserial::get('helper.update');
			$updateHelper->installMenu($code);
			$js = 'window.top.document.getElementById("image'.$code.'").src = "'.HIKASHOP_IMAGES.'icons/icon-16-edit.png"';
			$doc = JFactory::getDocument();
			$doc->addScriptDeclaration($js);
		} else {
			hikaserial::display(JText::sprintf('FAIL_SAVE', $path), 'error');
		}
		return $result;
	}
}
