<?php
/**
 * @package    HikaSerial for Joomla!
 * @version    4.0.2
 * @author     Obsidev S.A.R.L.
 * @copyright  (C) 2011-2023 OBSIDEV. All rights reserved.
 * @license    GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined('_JEXEC') or die('Restricted access');
?><?php
class planController extends hikaserialController {

	protected $type = 'pack';
	protected $toggle = array('pack_published' => 'pack_id');
	protected $rights = array(
		'display' => array('display','show','cancel','listing','generate','findList','file_entry','file_select'),
		'add' => array('add'),
		'edit' => array('edit','toggle','publish','unpublish','file_save'),
		'modify' => array('save','apply'),
		'delete' => array('remove')
	);

	public function __construct($config = array()) {
		parent::__construct($config);
		$this->registerDefaultTask('listing');
	}

	public function store() {
		return parent::adminStore();
	}

	public function generate() {
		$pack_id = hikaserial::getCID();
		$formData = hikaInput::get()->get('data', array(), 'array');

		if(!empty($formData) && !empty($formData['number_serials']) && (int)$formData['number_serials'] > 0 && !empty($pack_id)) {
			hikaserial::checkToken('request') || die('Invalid Token');

			$app = JFactory::getApplication();

			$packClass = hikaserial::get('class.pack');
			$packClass->generate($pack_id, $formData);

			if(!empty($serials)) {
				$struct = array(
					'serial_pack_id' => (int)$pack_id,
					'serial_status' => $serial_status
				);
				$serialClass = hikaserial::get('class.serial');
				$serialClass->generate($serials, $struct);

				$app->enqueueMessage( JText::sprintf('X_SERIAL_GENERATED', count($serials)) ); // _('X_SERIAL_GENERATED')
			} else {
				$app->enqueueMessage( JText::_('ERROR_GENERATING_SERIALS'), 'error' );
			}
			hikaInput::get()->set('serials', $serials);
		}

		hikaInput::get()->set('hidemainmenu', 1);
		hikaInput::get()->set('layout','generate');
		return $this->display();
	}

	public function remove() {
		hikaserial::checkToken() || die('Invalid Token');
		$cids = hikaInput::get()->get('cid', array(), 'array');
		$packClass = hikaserial::get('class.pack');
		$num = $packClass->delete($cids);
		if($num) {
			$app = JFactory::getApplication();
			$app->enqueueMessage(JText::sprintf('SUCC_DELETE_ELEMENTS', count($cids)), 'message');
		}
		return parent::listing();
	}

	public function findList() {
		$search = hikaInput::get()->getVar('search', '');
		$options = array();
		$nameboxType = hikashop_get('type.namebox');
		$elements = $nameboxType->getValues($search, 'plg.hikaserial.plans', $options);
		echo json_encode($elements);
		exit;
	}

	public function file_entry() {
		$cid = hikaserial::getCID('file_id');
		$pid = hikaInput::get()->getInt('pid', 0);

		if(empty($cid) || empty($pid))
			return false;

		hikaInput::get()->set('layout', 'file_entry');
		return parent::display();
	}

	public function file_select() {
		$cid = hikaserial::getCID('file_id');
		$pid = hikaInput::get()->getInt('pid', 0);

		if(empty($pid))
			return false;

		hikaInput::get()->set('layout', 'file_select');
		return parent::display();
	}

	public function file_save() {
		$fileClass = hikaserial::get('class.file');
		$ret = $fileClass->saveForm();
		if($ret) {
			$js = '
window.hikashop.ready(function(){
	window.parent.hikashop.submitBox({cid:'.(int)$ret.'});
});';
			$doc = JFactory::getDocument();
			$doc->addScriptDeclaration($js);
			return true;
		}
		return $this->file_select();
	}

	public function getUploadSetting($upload_key, $caller = '') {
		$pack_id = hikaInput::get()->getInt('pack_id');
		$packClass = hikaserial::get('class.pack');
		$plan = $packClass->get($pack_id);
		if(empty($upload_key) || (empty($plan) && !empty($pack_id)))
			return false;

		$upload_value = null;
		$upload_keys = array(
			'plan_file' => array(
				'type' => 'file',
				'view' => 'file_entry',
				'field' => 'data[pack_file][]',
				'delete' => true,
			)
		);

		if(empty($upload_keys[$upload_key]))
			return false;
		$upload_value = $upload_keys[$upload_key];

		$shopConfig = hikaserial::config(false);

		$uploader_id = hikaInput::get()->getString('uploader_id', '');
		$options = array(
			'max_file_size' => null,
			'upload_dir' => ($upload_value['type'] == 'image') ? $shopConfig->get('uploadfolder') : $shopConfig->get('uploadsecurefolder')
		);

		return array(
			'limit' => 1,
			'type' => $upload_value['type'],
			'layout' => 'plan',
			'view' => $upload_value['view'],
			'options' => $options,
			'extra' => array(
				'pack_id' => $pack_id,
				'delete' => !empty($upload_value['delete']),
				'uploader_id' => $uploader_id,
			)
		);
	}

	public function manageUpload($upload_key, &$ret, $uploadConfig, $caller = '') {
		if(empty($ret) || empty($ret->name) || empty($uploadConfig['extra']['pack_id']))
			return;

		$file_type = 'plan';
		$pack_id = (int)$uploadConfig['extra']['pack_id'];

		if($caller == 'upload' || $caller == 'addimage') {

			$file = new stdClass();
			$file->file_description = '';
			$file->file_name = $ret->name;
			$file->file_type = $file_type;
			$file->file_ref_id = $pack_id;
			$file->file_path = $ret->name;
			$file->file_free_download = false;
			$file->file_limit = 0;

			if(strpos($file->file_name, '.') !== false) {
				$file->file_name = substr($file->file_name, 0, strrpos($file->file_name, '.'));
			}

			$fileClass = hikaserial::get('shop.class.file');
			$status = $fileClass->save($file, $file_type);

			$ret->file_id = $status;
			$ret->params->file_id = $status;

			$ret->params->extra_fields = array(
				'data[pack_file][]' => $ret->file_id
			);


			return;
		}
	}
}
