<?php
/**
 * @package    HikaSerial for Joomla!
 * @version    4.0.2
 * @author     Obsidev S.A.R.L.
 * @copyright  (C) 2011-2023 OBSIDEV. All rights reserved.
 * @license    GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined('_JEXEC') or die('Restricted access');
?><?php
class PlgContentSerialprivatecontent extends JPlugin {
	private $juser = null;

	public function __construct(&$subject, $config) {
		parent::__construct($subject, $config);

		if(isset($this->params))
			return;

		$plugin = JPluginHelper::getPlugin('content', 'serialprivatecontent');
		if(version_compare(JVERSION, '2.5', '<')) {
			jimport('joomla.html.parameter');
			$this->params = new JParameter($plugin->params);
		} else {
			$this->params = new JRegistry($plugin->params);
		}
	}

	private function init() {
		static $init = null;
		if($init !== null)
			return $init;

		$init = defined('HIKASERIAL_COMPONENT');
		if(!$init) {
			$filename = rtrim(JPATH_ADMINISTRATOR,DS).DS.'components'.DS.'com_hikaserial'.DS.'helpers'.DS.'helper.php';
			if(file_exists($filename)) {
				include_once($filename);
				$init = defined('HIKASERIAL_COMPONENT');
			}
		}
		return $init;
	}

	protected function _isAdmin() {
		static $isAdmin = null;
		if($isAdmin !== null)
			return $isAdmin;
		$app = JFactory::getApplication();
		$isAdmin = version_compare(JVERSION,'4.0','<') ? $app->isAdmin() : $app->isClient('administrator');
		return $isAdmin;
	}

	public function onContentPrepare($context, &$article, &$params, $page = 0) {
		if($this->_isAdmin())
			return true;

		$tag_first_pos = strpos($article->text, '{privatecontent');
		if($tag_first_pos === false)
			return true;

		$regex = '#\{privatecontent\s(.*?)\}(.*)\{\/privatecontent\}#Uis';
		preg_match_all($regex, $article->text, $matches, PREG_SET_ORDER);

		$single = false;
		if(empty($matches)) {
			$regex = '#\{privatecontent\s(.*?)\/\}#Uis';
			preg_match($regex, $article->text, $matches);
			if(!empty($matches))
				$single = true;
		}

		if(empty($matches))
			return true;

		if($context == 'com_finder.indexer') {
			if(!$single) {
				foreach($matches as $match) {
					$article->text = str_replace($match[0], '', $article->text);
				}
			} else {
				$article->text = substr($article->text, 0, $tag_first_pos);
			}
			return true;
		}

		$packs = array();
		if(!$single) {
			foreach($matches as &$match) {
				$match['params'] = $this->processParams($match[1]);
				if($match['params'] === null)
					continue;
				$packs = array_merge($packs, $match['params']['pack']);
			}
			unset($match);
		} else {
			$matches['params'] = $this->processParams($matches[1]);
			if($matches['params'] !== null)
				$packs = $matches['params']['pack'];
		}

		$this->juser = JFactory::getUser();

		if($this->juser->guest || empty($packs) || !$this->init()) {
			if(!$single) {
				foreach($matches as $match) {
					$article->text = str_replace($match[0], $this->getPrivateMessage($match['params']), $article->text);
				}
			} else {
				$article->text = substr($article->text, 0, $tag_first_pos) . $this->getPrivateMessage($matches['params']);
			}

			return true;
		}

		$db = JFactory::getDBO();
		$user_id = hikaserial::loadUser(false);
		$config = hikaserial::config();

		$used_status = $config->get('used_serial_status', 'used');
		$filters = array(
			's.serial_pack_id IN ('.implode(',', $packs).')',
			's.serial_status = ' . $db->Quote($used_status),
			's.serial_user_id = ' . (int)$user_id
		);
		$query = 'SELECT s.serial_pack_id, count(s.serial_id) as count '.
				' FROM ' . hikaserial::table('serial') .' AS s '.
				' WHERE (' . implode(') AND (', $filters) . ')'.
				' GROUP BY s.serial_pack_id';
		$db->setQuery($query);
		$user_used_serials = $db->loadObjectList('serial_pack_id');

		$user_subscriptions = array();
		if(hikaserial::level(1)) {
			$filters = array(
				's.subscription_pack_id IN ('.implode(',', $packs).')',
				's.subscription_status = ' . $db->Quote('active'),
				's.subscription_end_date = 0 OR s.subscription_end_date <= '.time(),
				's.subscription_user_id = ' . (int)$user_id
			);
			$query = 'SELECT s.subscription_pack_id, count(s.subscription_id) as count '.
					' FROM ' . hikaserial::table('subscription') .' AS s '.
					' WHERE (' . implode(') AND (', $filters) . ')'.
					' GROUP BY s.subscription_pack_id';
			$db->setQuery($query);
			$user_subscriptions = $db->loadObjectList('subscription_pack_id');
		}

		if(!$single) {
			foreach($matches as $match) {
				if($this->isPackAuthorized($user_used_serials, $user_subscriptions, $match['params']['pack'])) {
					$article->text = str_replace($match[0], $match[2], $article->text);
					continue;
				}

				$article->text = str_replace($match[0], $this->getPrivateMessage($match['params']), $article->text);
			}
		} else if($this->isPackAuthorized($user_used_serials, $user_subscriptions, $matches['params']['pack'])) {
			$article->text = str_replace($matches[0], '', $article->text);
		} else {
			$article->text = substr($article->text, 0, $tag_first_pos) . $this->getPrivateMessage($matches['params']);
		}
	}

	private function processParams($paramText) {
		$regex = '#(\w+):("[^"]+"|[-.,a-zA-Z0-9]+)#i';
		preg_match_all($regex, $paramText, $matches, PREG_SET_ORDER);

		if(empty($matches))
			return null;

		$ret = array();
		foreach($matches as $match) {
			$k = $match[1];
			if(isset($ret[$k]))
				continue;
			$v = $match[2];
			if(substr($v, 0, 1) == '"' && substr($v, -1) == '"')
				$v = substr($v, 1, strlen($v) - 2);
			$ret[ $k ] = $v;
		}

		if(!empty($ret['pack'])) {
			$ret['pack'] = explode(',', $ret['pack']);
			$this->toInteger($ret['pack']);
		} else
			$ret['pack'] = array();

		return $ret;
	}

	private function toInteger(&$array) {
		if(is_array($array))
			$array = array_map('intval', $array);
		else
			$array = array();
	}

	private function isPackAuthorized($user_serials, $user_subscriptions, $packs) {
		foreach($packs as $p) {
			if(isset($user_serials[$p]) && (int)$user_serials[$p]->count > 0)
				return true;
			if(isset($user_subscriptions[$p]) && (int)$user_subscriptions[$p]->count > 0)
				return true;
		}
		return false;
	}

	private function getPrivateMessage($params) {
		$init = $this->init();
		$lang = JFactory::getLanguage();
		$lang->load('com_hikaserial', JPATH_SITE);

		$content = '<span class="hikaserial_private_content">' . JText::_('PRIVATE_CONTENT_MSG') . '</span>';
		if(!empty($params['text']))
			$content = JText::_($params['text']);
		if(isset($params['text']) && empty($params['text']))
			$content = '';

		$product_tag = strpos($content, '{product.link}');
		if($product_tag === false)
			$product_tag = strpos($content, '{link}');

		$link = '';
		if($init && (!empty($params['product']) || $product_tag !== false)) {
			$productClass = hikaserial::get('shop.class.product');
			$product = $productClass->get( (int)$params['product'] );

			$link = hikashop_contentLink('product&task=show&cid='.$product->product_id.'&name='.$product->alias, $product);
		}

		$tags = array();
		if(preg_match_all('#{[-.a-zA-Z0-9]+}#i', $content, $tags)) {
			foreach($tags[0] as $tag) {
				$data = '';
				$key = trim($tag, '{}');
				if($key == 'link' || $key == 'product.link') {
					$data = $link;
				} else if(substr($key, 0, 8) == 'product.') {
					$key = substr($key, 8);
					if(isset($product->$key))
						$data = $product->$key;
					else
						$data = @$product->{'product_'.$key};
				}
				$content = str_replace($tag, $data, $content);
			}
		}

		if($product_tag === false && !empty($params['product'])) {
			if(!empty($content))
				$content .= '<br/>';
			$content .= '<span class="hikaserial_private_content_product_link">' . JText::sprintf('PRIVATE_CONTENT_BUY_PRODUCT', $link, $product->product_name) . '</span>';
		}

		if(!empty($params['module']) && $init) {
			$js = '';
			$module_params = new JRegistry();
			$module_params->set('from_module', true);
			$module_params->set('return_url', hikaserial::currentUrl());
			$content .= trim(hikaserial::getLayout('serial', 'consume', $module_params, $js));
		}

		return $content;
	}

	public function onPrepareContent(&$article, &$params, $limitstart = 0) {
		$this->onContentPrepare(null, $article, $params, $limitstart);
	}
}
