<?php
/**
 * @package    HikaSerial for Joomla!
 * @version    4.0.2
 * @author     Obsidev S.A.R.L.
 * @copyright  (C) 2011-2023 OBSIDEV. All rights reserved.
 * @license    GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined('_JEXEC') or die('Restricted access');
?><?php
class plgHikaserialCoupongen extends hikaserialPlugin {

	protected $type = 'generator';
	protected $multiple = true;
	protected $populate = true;
	protected $name = 'coupongen';
	protected $doc_form = 'coupongen-';
	public $test = false;

	protected $serial_default_format = '[a-zA-Z0-9]{size}';
	protected $pluginConfig = array(
		'discount_id' => array('DISCOUNT_COUPON_ID', 'discount'),
		'discount_percent' => array('DISCOUNT_PRODUCT_PERCENT', 'input', '', ' %'),
		'discount_percent_tax' => array('DISCOUNT_PRODUCT_PERCENT_WITH_TAX', 'boolean'),
		'validity_value' => array('DISCOUNT_VALIDITY_PERIOD', 'period-value'),
		'validity_period' => array('', 'period-type'),
		'customer_assign' => array('DISCOUNT_CUSTOMER_ASSIGN', 'boolean', false),
		'size' => array('SERIAL_SIZE', 'input'),
		'format' => array('SERIAL_FORMAT', 'input', '[a-zA-Z0-9]{size}'),
		'test' => array('SERIAL_TEST', 'serial_test')
	);


	public function __construct(&$subject, $config) {
		parent::__construct($subject, $config);
	}

	public function generate(&$pack, &$order, $quantity, &$serials) {
		if(!isset($pack->coupongen))
			return;

		parent::pluginParams($pack->coupongen);

		if(empty($this->plugin_params->format) || !preg_match_all('#\\\[|\\\]|\[[^]]+\]\{.*\}|\[.*\]|.#iU', $this->plugin_params->format, $matches)) {
			$matches = array(array('[a-zA-Z0-9]{size}'));
		}

		$config = hikaserial::config();
		$fastRandom = (int)$config->get('use_fast_random', 0);

		if(empty($this->plugin_params->size) || $this->plugin_params->size == 0) {
			$this->plugin_params->size = 15;
		}

		for($q = 0; $q < $quantity; $q++) {
			$serial = '';
			$serialObj = new stdClass();

			$randomize = $this->initRandomize($fastRandom);

			foreach($matches[0] as $m) {
				if(strlen($m) == 1) {
					$serial .= $m;
					continue;
				}

				list($format, $repeat) = $this->generatorExtractRepeater($m);

				$serial .= $this->getRandomContent($format, $repeat, $randomize, $fastRandom);
			}

			$discount_id = (int)$this->plugin_params->discount_id;
			$result = true;
			if(!$this->test && !empty($discount_id)) {
				$discountClass = hikaserial::get('shop.class.discount');
				$data = $discountClass->get($discount_id);
			} else {
				$data = null;
			}
			if(!empty($data)) {
				unset($data->discount_id);
				$data->discount_code = $serial;
				$data->discount_published = 1;
				$data->discount_used_times = 0;

				if(!empty($this->plugin_params->validity_period) && !empty($this->plugin_params->validity_value) && (int)$this->plugin_params->validity_value > 0) {
					$date_d = date('d');
					$date_m = date('m');
					$date_y = date('Y');
					$v = (int)$this->plugin_params->validity_value;
					switch($this->plugin_params->validity_period) {
						case 'year':
							$data->discount_end = mktime(0,0,0, $date_m, $date_d, $date_y + $v);
							break;
						case 'month':
							$data->discount_end = mktime(0,0,0, $date_m + $v, $date_d, $date_y);
							break;
						case 'day':
							$data->discount_end = mktime(0,0,0, $date_m, $date_d + $v, $date_y);
							break;
					}
					if(!isset($serialObj->extradata))
						$serialObj->extradata = array();
					$serialObj->extradata['discount_end'] = $data->discount_end;
				}

				if(!empty($this->plugin_params->discount_percent)) {
					$v = hikaserial::toFloat(trim($this->plugin_params->discount_percent));
					$product_price = 0;
					$p = $this->retrieveOrderPackProduct($order, $pack);
					if(!empty($p)) {
						$product_price = hikaserial::toFloat($p->order_product_price);
						if(!empty($this->plugin_params->discount_percent_tax))
							$product_price += hikaserial::toFloat($p->order_product_tax);
					}

					if(!empty($product_price)) {
						if(!empty($order->order_currency_id))
							$data->discount_currency_id = (int)$order->order_currency_id;
						else
							$data->discount_currency_id = (int)$order->old->order_currency_id;
						$data->discount_flat_amount = $product_price * $v / 100;
						$data->discount_percent_amount = 0.0;
					}
				}

				if(!empty($this->plugin_params->customer_assign)) {
					$data->discount_user_id = !empty($order->order_user_id) ? $order->order_user_id : @$order->old->order_user_id;
					if(empty($data->discount_user_id))
						unset($data->discount_user_id);
				}

				$result = $discountClass->save($data);
				if(!empty($result) && (int)$result > 0) {
					if(!isset($serialObj->extradata))
						$serialObj->extradata = array();
					$serialObj->extradata['discount_id'] = (int)$result;
				}
			}
			if(empty($result)) {
				$app = JFactory::getApplication();
				$app->enqueueMessage(JText::_('ERR_CREATING_DISCOUNT_COUPON'));
			}

			if(!empty($serialObj) && !empty($serialObj->extradata)) {
				$serialObj->data = $serial;
				$serials[] = $serialObj;
			} else {
				$serials[] = $serial;
			}
		}
	}

	public function onAfterSerialUnassigned(&$serials) {
		if(empty($serials))
			return;

		$db = JFactory::getDBO();
		$unpublished_coupon = array();
		foreach($serials as $serial) {
			if(strpos($serial->pack_generator, '-') === false)
				continue;
			list($generator, $generator_id) = explode('-', $serial->pack_generator, 2);
			if($generator != 'plg.coupongen')
				continue;

			$unpublished_coupon[] = $db->Quote($serial->serial_data);
		}

		if(empty($unpublished_coupon))
			return;

		$query = 'UPDATE ' . hikaserial::table('shop.discount') .
				' SET discount_published = 0 '.
				' WHERE discount_type = ' . $db->Quote('coupon') . ' AND discount_code IN (' . implode(',', $unpublished_coupon) . ')';
		$db->setQuery($query);
		$db->execute();
	}

	public function configurationHead() {
		return array(
			1 => 'HIKASHOP_COUPON',
			2 => 'DISCOUNT_PRODUCT_PERCENT',
			3 => 'SERIAL_FORMAT',
			4 => 'SERIAL_SIZE'
		);
	}

	public function configurationLine($id = 0, $conf = null) {
		if(empty($this->discountClass))
			$this->discountClass = hikaserial::get('shop.class.discount');
		switch($id) {
			case 1:
				$d = null;
				if(!empty($conf->generator_params->discount_id))
					$d = $this->discountClass->get($conf->generator_params->discount_id);
				if(empty($d))
					return '<img src="'.HIKASERIAL_IMAGES.'icon-16/unpublish.png" alt="'.JText::_('ERROR').'"/>';
				return '<a href="'.hikaserial::completeLink('shop.discount&task=edit&cid='.$conf->generator_params->discount_id).'">'.$d->discount_code.'</a>';
			case 2:
				if(!empty($conf->generator_params->discount_percent))
					return $conf->generator_params->discount_percent.'%';
				return '';
			case 3:
				if(empty($conf->generator_params->format))
					return $this->serial_default_format;
				return $conf->generator_params->format;
			case 4:
				if(empty($conf->generator_params->size))
					return 12;
				return $conf->generator_params->size;
		}
		return null;
	}

	public function onDisplaySerials(&$data, $viewName) {
		if(!in_array($viewName, array('beforeMailSend', 'beforeDownloadFile', 'mailtocustomfield')))
			return;

		$db = JFactory::getDBO();
		$coupons = array(
			'id' => array(),
			'code' => array()
		);

		foreach($data as &$serial) {
			if(substr($serial->pack_generator, 0, 13) != 'plg.coupongen')
				continue;

			if(isset($serial->coupon))
				continue;

			if(!empty($serial->serial_extradata) && is_string($serial->serial_extradata)) {
				$serial->serial_extradata = hikaserial::unserialize($serial->serial_extradata);
			}
			if(!empty($serial->serial_extradata['discount_id'])) {
				$coupons['id'][] = (int)$serial->serial_extradata['discount_id'];
			} elseif(!empty($serial->serial_data)) {
				$coupons['code'][] = $db->Quote($serial->serial_data);
			}
		}
		unset($serial);

		if(empty($coupons['code']) && empty($coupons['id']))
			return;

		$filters = array();
		if(!empty($coupons['id'])) {
			$filters[] = 'discount_id IN ('.implode(',', $coupons['id']).')';
		}
		if(!empty($coupons['code'])) {
			$filters[] = 'discount_code IN ('.implode(',', $coupons['code']).')';
		}
		$query = 'SELECT * FROM ' . hikaserial::table('shop.discount').' WHERE discount_type = '.$db->Quote('coupon').' AND discount_published = 1 AND (('.implode(') OR (', $filters).'))';
		$db->setQuery($query);
		$coupons = $db->loadObjectList('discount_id');

		$currencyClass = hikaserial::get('shop.class.currency');
		foreach($coupons as &$coupon) {
			if(bccomp($coupon->discount_flat_amount, 0, 5)) {
				$coupon->display = $currencyClass->format($coupon->discount_flat_amount, $coupon->discount_currency_id);
			} else {
				$coupon->display = round($coupon->discount_percent_amount, 2).'%';
			}
		}
		unset($coupon);

		foreach($data as &$serial) {
			if(substr($serial->pack_generator, 0, 13) != 'plg.coupongen')
				continue;

			if(isset($serial->coupon))
				continue;

			if(!empty($serial->serial_extradata['discount_id'])) {
				$i = (int)$serial->serial_extradata['discount_id'];
				if(isset($coupons[$i]))
					$serial->coupon = $coupons[$i];

				continue;
			}

			if(!empty($serial->serial_data)) {
				foreach($coupons as $coupon) {
					if($coupon->discount_code != $serial->serial_data)
						continue;
					$serial->coupon = $coupon;
					break;
				}
			}
		}
		unset($serial);
	}
}
