<?php
/**
 * @package    HikaSerial for Joomla!
 * @version    4.0.2
 * @author     Obsidev S.A.R.L.
 * @copyright  (C) 2011-2023 OBSIDEV. All rights reserved.
 * @license    GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined('_JEXEC') or die('Restricted access');
?><?php
class plgHikaserialGroupsubscriber extends hikaserialPlugin {
	protected $type = 'subscriber';
	protected $multiple = false;
	protected $name = 'groupsubscriber';

	protected $pluginConfig = array();
	protected $subscriptionConfig = array(
		'groups' => array('HIKA_SERIAL_OPT_GROUPS', 'groups'),
	);

	public function __construct(&$subject, $config) {
		parent::__construct($subject, $config);
	}

	public function onSubscriptionActivation(&$subscription, $user) {
		if(empty($subscription->plan->pack_params->groupsubscriber) || empty($subscription->plan->pack_params->groupsubscriber['groups']))
			return;

		$groups = explode(',', $subscription->plan->pack_params->groupsubscriber['groups']);
		hikaserial::toInteger($groups);
		$groups = array_combine($groups, $groups);
		unset($groups[0]);
		if(empty($groups))
			return;
		$user_id = (int)$user->user_id;
		$user_cms_id = (int)$user->user_cms_id;

		if(empty($user_cms_id))
			return false;

		$check = $this->checkGroups($user_cms_id, $groups);
		if($check === true || $check === null)
			return true;

		$ret = $this->updateGroups($user_cms_id, $check);
	}

	public function onSubscriptionExpiration(&$subscription, $user, $status) {
		if(empty($subscription->plan->pack_params->groupsubscriber) || empty($subscription->plan->pack_params->groupsubscriber['groups']))
			return;

		$groups = explode(',', $subscription->plan->pack_params->groupsubscriber['groups']);
		hikaserial::toInteger($groups);
		$groups = array_combine($groups, $groups);
		unset($groups[0]);
		if(empty($groups))
			return;

		$user_id = (int)$user->user_id;
		$user_cms_id = (int)$user->user_cms_id;

		if(empty($user_cms_id))
			return false;

		$plugin_groups = array();
		if(!empty($user->subscriptions)) {
			foreach($user->subscriptions as $sub) {
				if((int)$sub->subscription_id == (int)$subscription->subscription_id)
					continue;

				if(!isset($user->plans[ (int)$sub->subscription_pack_id ]))
					continue;
				$p = $user->plans[ (int)$sub->subscription_pack_id ];

				if(empty($p->pack_params))
					continue;
				if(is_string($p->pack_params))
					$p->pack_params = hikaserial::unserialize($p->pack_params);
				if(empty($p->pack_params->groupsubscriber) || empty($p->pack_params->groupsubscriber['groups']))
					continue;

				$g = explode(',', $p->pack_params->groupsubscriber['groups']);
				hikaserial::toInteger($g);
				$g = array_combine($g, $g);
				unset($g[0]);

				$plugin_groups = array_replace($plugin_groups, $g);
			}
		}

		if(empty($plugin_groups)) {
			$remove_groups = $groups;
		} else {
			$remove_groups = array_diff($groups, $plugin_groups);
		}

		$check = $this->checkGroups($user_cms_id, $remove_groups, false);


		$ret = $this->updateGroups($user_cms_id, array(), $remove_groups);
	}

	public function onAfterSubscriptionsClose(&$closed_subscriptions, $plans, $users) {
		$this->onAfterSubscriptionsExpire($closed_subscriptions, $plans, $users);
	}

	public function onAfterSubscriptionsExpire(&$expired_subscriptions, $plans, $users) {
		$supported_subscriptions = array();
		$supported_users = array();

		foreach($expired_subscriptions as $key => &$expired_subscription) {
			if(empty($expired_subscription->subscription_user_id))
				continue;

			$user_id = (int)$expired_subscription->subscription_user_id;
			if(empty($users[$user_id]->user_cms_id))
				continue;

			$plan_id = (int)$expired_subscription->subscription_pack_id;
			$plan = $plans[$plan_id];
			if(empty($plan->pack_params->groupsubscriber) || empty($plan->pack_params->groupsubscriber['groups']))
				continue;

			$other = false;
			foreach($users[$user_id]->subscriptions as $sub) {
				if((int)$sub->subscription_pack_id == $plan_id) {
					$other = true;
					break;
				}
			}
			if($other)
				continue;

			$supported_subscriptions[$key] =& $expired_subscription;

			if(empty($supported_users[ $user_id ])) {
				$supported_users[ $user_id ] = array(
					'id' => $user_id,
					'cms' => (int)$users[$user_id]->user_cms_id,
					'groups' => array()
				);
			}

			$g = $this->_getPlanGroups($plan_id, $plans);
			$supported_users[ $user_id ]['groups'] = array_merge($supported_users[ $user_id ]['groups'], $plan->pack_params->groupsubscriber['groups']);

		}
		unset($expired_subscription);

		if(empty($supported_subscriptions))
			return;

		foreach($supported_users as $user_id => &$data) {
			$data['groups'] = array_unique($data['groups']);
			if(empty($data['groups']))
				continue;
			$user_plans = [];
			foreach($users[$user_id]->subscriptions as $sub) {
				$k = (int)$sub->subscription_pack_id;
				if(isset($user_plans[$k]))
					continue;
				$user_plans[$k] = $k;
				$g = $this->_getPlanGroups($plan_id, $plans);
				$data['actgroups'] = array_merge($data['actgroups'], $g);
			}
			$data['actgroups'] = array_unique($data['actgroups']);
			$remove_groups = array_diff($data['groups'], $data['actgroups']);
			if(empty($remove_groups))
				continue;
			$check = $this->checkGroups($user_id, $remove_groups);
			if($check !== true)
				$remove_groups = array_intersect($remove_groups, $check);
			$ret = $this->updateGroups($data['cms'], array(), $remove_groups);
		}
		unset($data);
	}

	private function _getPlanGroups($plan_id, &$plans) {
		if(empty($plans[$plan_id]))
			return [];
		$plan = $plans[$plan_id];
		if(!is_string($plan->pack_params->groupsubscriber['groups']))
			return $plan->pack_params->groupsubscriber['groups'];
		$g = explode(',', $plan->pack_params->groupsubscriber['groups']);
		hikaserial::toInteger($g);
		$g = array_combine($g, $g);
		unset($g[0]);
		$plans[$plan_id]->pack_params->groupsubscriber['groups'] = $g;
		return $g;
	}

	private function checkGroups($user_id, $groups) {
		jimport('joomla.access.access');
		JAccess::clearStatics();
		$userGroups = JAccess::getGroupsByUser($user_id, true);
		$diff = array_diff($groups, $userGroups);
		if(empty($diff))
			return true;
		return $diff;
	}

	private function updateGroups($user_id, $add_groups, $remove_groups = array()) {
		if(empty($add_groups) && empty($remove_groups))
			return null;

		$user = clone(JFactory::getUser($user_id));

		jimport('joomla.access.access');
		JAccess::clearStatics();
		$userGroups = JAccess::getGroupsByUser($user_id, true);
		if(!empty($add_groups)) {
			$userGroups = array_merge($userGroups, array_values($add_groups));
			$userGroups = array_unique($userGroups);
		}
		if(!empty($remove_groups)) {
			$userGroups = array_diff($userGroups, $remove_groups);
		}
		$user->set('groups', $userGroups);
		return $user->save();
	}

	private function checkGroup($user_id, $group_id) {
		jimport('joomla.access.access');
		JAccess::clearStatics();
		$userGroups = JAccess::getGroupsByUser($user_id, true);
		return in_array($group_id, $userGroups);
	}

	private function updateGroup($user_id, $new_group_id, $remove_group_id = 0) {
		$user = clone(JFactory::getUser($user_id));

		jimport('joomla.access.access');
		JAccess::clearStatics();
		$userGroups = JAccess::getGroupsByUser($user_id, true);
		$userGroups[] = $new_group_id;

		if(!empty($remove_group_id)) {
			$key = array_search($remove_group_id, $userGroups);
			if(is_int($key)) {
				unset($userGroups[$key]);
			}
		}

		$user->set('groups', $userGroups);
		return $user->save();
	}

	private function forceLogout($user_id) {
		$app = JFactory::getApplication();
		$conf = JFactory::getConfig();

		$handler = $conf->get('session_handler', 'none');
		if($handler == 'database') {
			$db->setQuery('DELETE FROM #__session WHERE client_id = 0 AND userid = '.(int)$user_id);
			$db->execute();
		}
		if(!hikaserial::isAdmin()) {
			$app->logout( (int)$user_id );
		}
	}
}
