<?php
/**
 * @package    HikaSerial for Joomla!
 * @version    4.0.2
 * @author     Obsidev S.A.R.L.
 * @copyright  (C) 2011-2023 OBSIDEV. All rights reserved.
 * @license    GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined('_JEXEC') or die('Restricted access');
?><?php
class plgHikaserialPdfsecure extends hikaserialPlugin {
	protected $type = 'plugin';
	protected $multiple = false;
	protected $doc_form = 'pdfsecure-';

	public function __construct(&$subject, $config) {
		parent::__construct($subject, $config);
	}


	public function onBeforeSerialDownloadFile(&$filename, &$do, &$file, &$serials) {
		$startStr = 'hikaserial:pdfsecure:';
		$l = strlen($startStr);
		if(substr($filename, 1, $l) != $startStr) {
			return;
		}
		$params = substr($filename, $l+1);
		if(substr($params, 0, 1) == ':')
			$params = substr($params, 1);
		$id = 0;

		$serial = reset($serials);
		if(!empty($file->file_pos)) {
			if(count($serials) >= $file->file_pos) {
				$serial = $serials[$file->file_pos - 1];
				$serial->position = $file->file_pos;
			} else {
				hikashop_logData('Invalid position ('.$filename.')', 'PDF Secure');
				$do = false;
				return;
			}
		}
		$serial_data = (!empty($serial) && !empty($serial->serial_data)) ? $serial->serial_data : '';
		if(empty($serial_data)) {
			hikashop_logData('Empty serial data ('.$filename.')', 'PDF Secure');
			$do = false;
			return;
		}
		$ret = $this->sendPDF($params, $serial_data);
		if(!$ret) {
			hikashop_logData('Error with PDF stamping ('.$filename.')', 'PDF Secure');
			$do = false;
			return;
		}
	}

	private function sendPDF($pdfFilename, $password) {
		$pdfLib = hikaserial::get('inc.pdf');
		if(empty($pdfLib))
			return false;

		$config = hikashop_config();
		$uploadFolder = $config->get('uploadsecurefolder');
		$uploadFolder = rtrim(JPath::clean(html_entity_decode($uploadFolder)), DS.' ').DS;
		if((!preg_match('#^([A-Z]:)?/.*#', $uploadFolder)) && ($uploadFolder[0] != '/' || !is_dir($uploadFolder))) {
			$uploadFolder = rtrim(JPath::clean(HIKASHOP_ROOT.DS.trim($uploadFolder, DS.' ').DS), DS.' ') . DS;
		}
		$realpath = realpath($uploadFolder);
		if(!empty($realpath))
			$uploadFolder = rtrim($realpath, DS.' ').DS;
		if(strpos($uploadFolder,'..') !== false)
			return false;
		if(!is_dir($uploadFolder))
			return false;

		$filename = $uploadFolder . $pdfFilename;
		if(strpos($filename, '..') !== false)
			return false;
		if(!file_exists($filename))
			return false;

		$pdf = $pdfLib->getFPDI(true, array('protect' => true));

		$templatePageCount = $pdf->setSourceFile($filename);
		for($i = 1; $i <= $templatePageCount; $i++) {
			$templateIndex = $pdf->importPage($i);
			$size = $pdf->getTemplateSize($templateIndex);
		    if($size['w'] > $size['h']) {
		        $pdf->AddPage('L', array($size['w'], $size['h']));
		    } else {
		        $pdf->AddPage('P', array($size['w'], $size['h']));
		    }
			$pdf->useTemplate($templateIndex);
		}
		$pdf->SetProtection(array(), $password);
		ob_get_clean();
		$ret = $pdf->Output('S', '', true); // last parameter: is utf-8
		if($ret !== true) {
			$extension = 'pdf';
			if(!isset($this->params))
				$this->loadPluginParams();
			$serial_data = (!empty($serial) && !empty($serial->serial_data)) ? $serial->serial_data : '';
			$this->sendHeader($extension, $serial_data);
			echo $ret;
			exit;
		}
	}

	private function sendHeader($extension, $serial = '') {
		if($this->params->get('force_download', 0)) {
			$filename = $this->params->get('force_download_filename', '');
			if(empty($filename))
				$filename = $serial;
			if(empty($filename))
				$filename = 'serial';

			jimport('joomla.filesystem.file');
			$filename = strtolower(JFile::makeSafe(trim($filename)));

			header('Content-Type: application/octet-stream');
			header('Content-Disposition: attachment; filename="' . $filename . '.' . $extension . '"');

			return true;
		}

		header('Content-Type: application/pdf');
		return true;
	}
}
