<?php
/**
 * @package    HikaSerial for Joomla!
 * @version    4.0.2
 * @author     Obsidev S.A.R.L.
 * @copyright  (C) 2011-2023 OBSIDEV. All rights reserved.
 * @license    GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined('_JEXEC') or die('Restricted access');
?><?php
class plgHikaserialPdfserial extends hikaserialPlugin {

	protected $type = 'plugin';
	protected $multiple = true;
	protected $doc_form = 'pdfserial-';

	public function __construct(&$subject, $config) {
		parent::__construct($subject, $config);
	}

	protected function loadPluginParams() {
		if(isset($this->params))
			return;

		$pluginsClass = hikaserial::get('class.plugins');
		$plugin = $pluginsClass->getByName('hikaserial', 'pdfserial');

		if(version_compare(JVERSION, '2.5', '<')) {
			jimport('joomla.html.parameter');
			$this->params = new JParameter(@$plugin->params);
		} else {
			$this->params = new JRegistry(@$plugin->params);
		}
	}

	public function configurationHead() {
		return array(
			'email' => array(
				'title' => JText::_('ATTACH_IN_EMAILS'),
				'cell' => 'align="center" width="10%"'
			),
			'download' => array(
				'title' => JText::_('ATTACH_AS_DOWNLOAD'),
				'cell' => 'align="center" width="10%"'
			),
			'packs' => array(
				'title' => JText::_('SERIAL_PACKS'),
				'cell' => 'width="30%"'
			)
		);
	}

	public function configurationLine($id = 0, $conf = null) {
		if(empty($this->toggleHelper))
			$this->toggleHelper = hikaserial::get('helper.toggle');

		switch($id) {
			case 'email':
				$v = !empty($conf->plugin_params->attach_email) ? 1 : 0;
				return $this->toggleHelper->display('', $v);
			case 'download':
				$v = !empty($conf->plugin_params->attach_download) ? 1 : 0;
				return $this->toggleHelper->display('', $v);
			case 'packs':
				if(empty($this->packs)) {
					$db = JFactory::getDBO();
					$db->setQuery('SELECT * FROM '.hikaserial::table('pack'));
					$this->packs = $db->loadObjectList('pack_id');
				}
				$ret = array();
				if(!empty($conf->plugin_params->packs)) {
					if(!is_array($conf->plugin_params->packs))
						$conf->plugin_params->packs = array($conf->plugin_params->packs);
					foreach($conf->plugin_params->packs as $p) {
						if(isset($this->packs[(int)$p]))
							$ret[] = '<a href="'.hikaserial::completeLink('pack&task=edit&cid='.(int)$p).'">'.$this->packs[(int)$p]->pack_name.'</a>';
					}
				}
				return implode(', ', $ret);
		}
		return null;
	}

	public function onPluginConfigurationSave(&$element) {
		if(empty($element->plugin_params->texts))
			return;
		foreach($element->plugin_params->texts as &$text) {
			foreach($text as $k => $v) {
				$v = trim($v);
				if($v === '')
					unset($text[$k]);
			}
		}
		unset($text);
	}

	public function onBeforeSerialMailSend(&$mail, &$mailer, &$serials, $order) {
		$query = 'SELECT plugin_id, plugin_name, plugin_params FROM '.hikaserial::table('plugin').' WHERE plugin_type = '.$this->db->Quote('pdfserial').' AND plugin_published = 1';
		$this->db->setQuery($query);
		$this->db->execute();
		$plugins = $this->db->loadObjectList();
		if(empty($plugins))
			return;

		$orderClass = hikaserial::get('shop.class.order');
		$fullOrder = $orderClass->loadFullOrder($order->order_id, true, false);

		foreach($plugins as $plugin) {
			$params = hikaserial::unserialize($plugin->plugin_params);

			if(empty($params) || empty($params->attach_email) || empty($params->packs))
				continue;

			$order_products = array();
			$products = array();

			$params->order = $order;

			foreach($serials as $k => $serial) {
				if(is_string($params->packs) && strpos(','.$params->packs.',', ','.$serial->serial_pack_id.',') === false)
					continue;
				if(is_array($params->packs) && !in_array($serial->serial_pack_id, $params->packs))
					continue;

				if(!empty($serial->serial_order_product_id)) {
					if(isset($order_products[$serial->serial_order_product_id])) {
						$serial->position = ($order_products[$serial->serial_order_product_id] + 1);
						$order_products[$serial->serial_order_product_id]++;
					} else {
						$serial->position = 1;
						$order_products[$serial->serial_order_product_id] = 1;
					}
				}

				$ret = $this->generateData($params, $serial, $fullOrder);
				if(empty($ret))
					continue;
				$this->addToMailer($mail, $mailer, $params, $plugin, $ret);
			}
		}
	}

	public function onBeforeSubscriptionMailSend(&$mail, &$mailer, &$subscriptions, $order) {
		foreach($subscriptions as &$sub) {
			$sub->serial_pack_id = $sub->subscription_pack_id;
		}
		unset($sub);
		return $this->onBeforeSerialMailSend($mail, $mailer, $subscriptions, $order);
	}

	public function onBeforeSerialDownloadFile(&$filename, &$do, &$file, &$serials) {
		$startStr = 'hikaserial:pdfserial:';
		$l = strlen($startStr);
		if(substr($filename, 1, $l) != $startStr) {
			return;
		}
		$params = substr($filename, $l+1);
		if(substr($params, 0, 1) == ':')
			$params = substr($params, 1);
		$id = 0;
		if((int)$params > 0) {
			$id = (int)$params;
			$this->pluginParams($id, 'pdfserial');
		} else {
			$query = 'SELECT plugin_params FROM '.hikaserial::table('plugin').
				' WHERE plugin_type = '.$this->db->Quote('pdfserial').' AND plugin_published = 1 AND plugin_name = ' . $this->db->Quote($params);
			$this->db->setQuery($query);
			$this->db->execute();
			$data = $this->db->loadResult();
			$this->plugin_params = null;
			if(!empty($data)) {
				$this->plugin_params = hikaserial::unserialize($data);
			}
		}

		if(empty($this->plugin_params) || empty($this->plugin_params->attach_download)) {
			$do = false;
			return;
		}

		$serial = reset($serials);
		if(!empty($file->file_pos)) {
			if(count($serials) >= $file->file_pos) {
				$serial = $serials[$file->file_pos - 1];
				$serial->position = $file->file_pos;
			} else {
				$do = false;
				return;
			}
		}

		$orderClass = hikaserial::get('shop.class.order');
		$order = $orderClass->loadFullOrder($file->order_id, true, false);

		$this->plugin_params->order = $order;
		$do = false;

		$ret = $this->generateData($this->plugin_params, $serial, $order);
		if(empty($ret))
			return;
		$this->sendFileData($this->plugin_params, $serial, $ret);
	}

	public function onBeforeSubscriptionDownloadFile(&$filename, &$do, &$file, &$subscriptions) {
		foreach($subscriptions as &$sub) {
			$sub->serial_pack_id = $sub->subscription_pack_id;
		}
		unset($sub);
		return $this->onBeforeSerialDownloadFile($filename, $do, $file, $subscriptions);
	}

	private function generateData(&$params, &$serial, &$fullOrder) {
		if(empty($params->texts))
			return false;

		$data = $this->loadData($params->texts, $serial, $fullOrder);
		if($data === false || empty($data))
			return false;
		$ret = $this->generatePDF($data, $params, null);
		if(empty($ret))
			return false;

		return $ret;
	}

	private function loadData($texts, $serial, $order) {
		if(empty($texts))
			return false;

		$data = array();
		$orderClass = hikaserial::get('class.order');

		foreach($texts as $text) {
			$content = null;
			$t = $text['type'];
			if(substr($t, -1) == '.')
				$t .= @$text['type_ex'];
			list($table, $field) = explode('.', $t, 2);

			if(in_array($table, array('dyntext', 'translation'))) {
				if($table == 'translation')
					$field = JText::_($field);

				$content = $field;
				if(preg_match_all('#\{([_a-z0-9]+\.[._a-z0-9]+)\}#i', $field, $matches)) {
					foreach($matches[1] as $match) {
						list($t,$f) = explode('.', $match, 2);
						$c = $orderClass->retrieveConvertedData($t, $f, $serial, $order);
						if($c === null && empty($serial))
							$content = '';
						$content = str_replace('{'.$match.'}', $c, $content);
					}
				}
			} elseif($table == 'pdf') {
				$data[] = array(
					'pdf' => $field,
					'format' => @$text['format'],
					'format_ex' => @$text['format_ex'],
					'x' => @$text['x'],
					'y' => @$text['y'],
					'w' => @$text['w'],
					'h' => @$text['h'],
					'size' => @$text['size'],
					'color' => @$text['color'],
					'font' => @$text['font']
				);
			} else {
				$content = $orderClass->retrieveConvertedData($table, $field, $serial, $order);
			}

			if(empty($content))
				continue;

			$data[] = array(
				'text' => $content,
				'format' => @$text['format'],
				'format_ex' => @$text['format_ex'],
				'x' => @$text['x'],
				'y' => @$text['y'],
				'w' => @$text['w'],
				'h' => @$text['h'],
				'size' => @$text['size'],
				'color' => @$text['color'],
				'font' => @$text['font']
			);
		}

		return $data;
	}

	private function addToMailer(&$mail, &$mailer, &$params, &$plugin, $ret) {
		if(!isset($this->params))
			$this->loadPluginParams();
		$format = $this->params->get('pdf_name_format', '');
		$virtual_file = (int)$this->params->get('virtual_file', 1);
		$use_mime = (int)$this->params->get('use_mime', 1);

		$extension = 'pdf';
		if(empty($format)) {
			$filename = 'pdf'.uniqid().'.'.$extension;
		} else if(strpos($format, '{' === false) && strpos($format, '}' === false)) {
			$filename = $format.uniqid().'.'.$extension;
		} else {
			$filename = str_replace(
					array('{data}', '{id}', '{order.id}', '{order.number}', '{invoice.id}', '{invoice.number}'),
					array($serial->serial_data, uniqid(), $params->order->order_id, $params->order->order_number, $params->order->order_invoice_id, $params->order->order_invoice_number),
					$format
				) . '.' . $extension;
		}

		if($virtual_file) {
			if($use_mime)
				$mailer->AddStringAttachment($ret, $filename, 'base64', 'application/pdf');
			else
				$mailer->AddStringAttachment($ret, $filename, 'base64');
		} else {
			static $nb = 0;
			$tmpFilename = HIKASERIAL_MEDIA.'images'.DS.'safe'.DS.'tmp_pdfserial_'.$plugin->plugin_id.'_'.($nb++).'.'.$extension;
			$writeRet = JFile::write($tmpFilename, $ret);
			$mailer->addAttachment($tmpFilename, $filename);
		}
	}

	private function sendFileData(&$params, &$serial, $ret) {
		$extension = 'pdf';
		if(!isset($this->params))
			$this->loadPluginParams();
		$serial_data = (!empty($serial) && !empty($serial->serial_data)) ? $serial->serial_data : '';
		$this->sendHeader($extension, $serial_data);

		if($ret !== true)
			echo $ret;
		exit;
	}

	private function generatePdf($data, $params, $dest_path = null) {
		if(empty($data))
			return false;

		$pdfLib = hikaserial::get('inc.pdf');
		if(empty($pdfLib))
			return false;

		$pdf_template = $this->getFilePath($params->pdf_path);

		$templateIndex = null;
		if(empty($pdf_template)) {
			$pdf = $pdfLib->getDefaultPDF();
		} else {
			$pdf = $pdfLib->getFPDI();

			$templatePageCount = $pdf->setSourceFile($pdf_template);
			$templateIndex = $pdf->importPage(1);
		}

		$isFPDF = (bool)(class_exists('FPDF') && $pdf instanceof FPDF);

		$pdf->SetCreator('HikaSerial (www.hikashop.com)');
		$pdf->SetAutoPageBreak(1);

		if(!$isFPDF) {
			$pdf->setPrintHeader(false);
			$pdf->setPrintFooter(false);
			$pdf->setImageScale(PDF_IMAGE_SCALE_RATIO);
		}

		if(!empty($pdf_template)) {
			$size = $pdf->getTemplateSize($templateIndex);
		    if($size['w'] > $size['h']) {
		        $pdf->AddPage('L', array($size['w'], $size['h']));
		    } else {
		        $pdf->AddPage('P', array($size['w'], $size['h']));
		    }
			$pdf->useTemplate($templateIndex);
		} else {
			$pdf->AddPage();
		}

		foreach($data as $d) {
			if(!is_array($d))
				$d = array('text' => $d);

			if(!empty($d['pdf'])) {
				$this->handlePDFaction($pdf, $d, array('tplId' => $templateIndex));
			}

			if(empty($d['text']))
				continue;

			if(!empty($d['format']) && $d['format'] != 'raw') {
				$format = $d['format'];
				if(substr($format, -1) == '.')
					$format .= @$d['format_ex'];
				if(strpos($format, '.')===false) $format.='.';
				list($format, $format_ex) = explode('.', $format, 2);

				switch($format) {
					case 'date':
						$d['text'] = hikaserial::getDate( (int)$d['text'], $format_ex );
						break;
					case 'price':
						if(empty($currencyClass))
							$currencyClass = hikaserial::get('shop.class.currency');
						$d['text'] = $currencyClass->format(hikaserial::toFloat($d['text']), $params->order->order_currency_id);
						break;
					case 'qrcode':
						$d['text'] = $this->generateImageQrCode($pdf, $d, $format_ex);
						break;
					case 'barcode':
						$d['text'] = $this->generateImageBarCode($pdf, $d, $format_ex);
						break;
					case 'image':
						$d['text'] = $this->includeImage($pdf, $d);
						break;
					case 'strip':
						$d['text'] = strip_tags($d['text']);
						break;
					case 'html':
						if($isFPDF) {
							$d['text'] = strip_tags($d['text']);
						}
						break;
					default:
						$content = $d['text'];
						$d['text'] = null;
						$dispatcher = hikaserial::getDispatcher();
						$dispatcher->trigger('onCustomPdfSerialFormat', array($format, $format_ex, $content, &$pdf, &$d));
						break;
				}
			}
			if(!empty($d['text'])) {
				if(!empty($d['color']) && ((substr($d['color'],0,1) == '#' && strlen(trim($d['color'])) == 7) || strlen(trim($d['color'])) == 6)) {
					$rgb = str_split(ltrim($d['color'], '#'), 2);
					$pdf->SetTextColor(hexdec($rgb[0]), hexdec($rgb[1]), hexdec($rgb[2]));
				} else {
					$pdf->SetTextColor(0,0,0);
				}

				$font = (!empty($d['font'])) ? $d['font'] : $params->default_font;
				if(empty($d['size'])) $d['size'] = 12;
				if(empty($d['w'])) $d['w'] = 0;
				if(empty($d['h'])) $d['h'] = 5;
				if(empty($d['a']) || !in_array($d['a'], array('L','C','R','J'))) $d['a'] = 'L';

				$pdf->SetFont($font, '', (int)$d['size']); // font name, style: B+I+U, font size

				if(!empty($d['x']) && !empty($d['y']))
					$pdf->SetXY((float)$d['x'], (float)$d['y']);
				elseif(!empty($d['x']))
					$pdf->setX((float)$d['x']);
				elseif(!empty($d['y']))
					$pdf->setY((float)$d['x']);

				if(!$isFPDF && !empty($d['format']) && $d['format'] == 'html') {
					$pdf->writeHTMLCell((int)$d['w'], (int)$d['h'], '', '', $d['text'], 0, 1, false, true, $d['a']);
				} else {
					$pdf->MultiCell((int)$d['w'], (int)$d['h'], $isFPDF ? utf8_decode($d['text']) : $d['text'], 0, $d['a'], false);
				}
			}
		}

		ob_get_clean();

		if(!$isFPDF) {
			if(empty($dest_path)) {
				return $pdf->Output('', 'S');
			}
			return $pdf->Output($dest_path, 'F');
		}
		if(empty($dest_path)) {
			return $pdf->Output('S', '', true); // last parameter: is utf-8
		}
		return $pdf->Output('F', $dest_path, true); // last parameter: is utf-8
	}

	private function handlePDFaction(&$pdf, &$d, $options = array()) {
		if(empty($d['pdf']))
			return;

		switch($d['pdf']) {
			case 'newpage':
				if(isset($options['tplId']) && $options['tplId'] !== null) {
					$size = $pdf->getTemplateSize($options['tplId']);
					if($size['w'] > $size['h']) {
						$pdf->AddPage('L', array($size['w'], $size['h']));
					} else {
						$pdf->AddPage('P', array($size['w'], $size['h']));
					}
					$pdf->useTemplate($options['tplId']);
				} else {
					$pdf->AddPage(); // Orientation (P,L), Size (A3,A4,A5,Letter,Legal)
				}
				break;
			case 'title':
				$pdf->SetTitle($d['text']); // isUTF8
				$d['text'] = null;
				break;
			case 'author':
				$pdf->SetAuthor($d['text']); // isUTF8
				$d['text'] = null;
				break;
			case 'subject':
				$pdf->SetSubject($d['text']); // isUTF8
				$d['text'] = null;
				break;
			case 'keywords':
				$pdf->SetKeywords($d['text']); // isUTF8
				$d['text'] = null;
				break;

			case 'margins':
				$pdf->SetMargins($d['x'], $d['y']);
				break;

			case 'pagebreak':
				$y = in_array(strtolower($d['text']), array('y','1','yes','o'));
				$pdf->SetAutoPageBreak($y);
				$d['text'] = null;
				break;

			case 'line':
				$lw = $pdf->GetLineWidth();
				if(!empty($d['size']))
					$pdf->SetLineWidth($d['size']);
				if(!empty($d['color']) && strlen($d['color']) >= 6) {
					$rgb = str_split(ltrim($d['color'], '#'), 2);
					$pdf->SetDrawColor(hexdec($rgb[0]), hexdec($rgb[1]), hexdec($rgb[2]));
				}
				$pdf->Line($d['x'],$d['y'],$d['w'],$d['h']);
				$pdf->SetLineWidth($lw);
				break;

			case 'rect':
				$lw = $pdf->GetLineWidth();
				if(!empty($d['size']))
					$pdf->SetLineWidth($d['size']);

				if(!empty($d['color']) && strlen($d['color']) >= 6) {
					$rgb = str_split(ltrim($d['color'], '#'), 2);
					$pdf->SetDrawColor(hexdec($rgb[0]), hexdec($rgb[1]), hexdec($rgb[2]));
				}

				$style = 'D'; // D : draw / F : fill / DF : draw & fill
				$pdf->Rect($d['x'],$d['y'],$d['w'],$d['h'],$style);

				$pdf->SetLineWidth($lw);
				break;
		}
	}

	private function generateImageQrCode($pdf, $d, $format_ex = null) {
		$level = 'l';
		$pixelPerPoint = 3;
		$outerFrame = 0;
		if(!empty($d['size']))
			$pixelPerPoint = (int)$d['size'];

		if(!empty($format_ex))
			$format_ex = strtolower($format_ex);
		if(!empty($format_ex) && in_array($format_ex, array('l','m','q','h')))
			$level = $format_ex;

		if(method_exists($pdf, 'write2DBarcode')) {
			$rgb = (!empty($d['color']) && strlen($d['color']) >= 6) ? str_split(ltrim($d['color'], '#'), 2) : array(0,0,0);
			$style = array(
				'border' => $outerFrame,
				'vpadding' => 'auto',
				'hpadding' => 'auto',
				'fgcolor' => array(hexdec($rgb[0]), hexdec($rgb[1]), hexdec($rgb[2])),
				'bgcolor' => false,
				'module_width' => $pixelPerPoint,
				'module_height' => $pixelPerPoint
			);
			$pdf->write2DBarcode($d['text'], 'QRCODE,'.strtoupper($level), $d['x'], $d['y'], 0, 0, $style, 'N');
		} else {
			$pixelPerPoint *= 72;

			$QRCode = hikaserial::get('inc.qrcode', $level, $pixelPerPoint, $outerFrame, $d['color']);
			$qrcode_image = $QRCode->getImage($d['text']);
			if(empty($qrcode_image))
				return null;

			$imgH = imagesy($qrcode_image)/8;
			$imgW = imagesx($qrcode_image)/8;

			if(method_exists($pdf, 'GDImage')) {
				$pdf->GDImage($qrcode_image, $d['x'], $d['y'], $imgW, $imgH);
			} else {
				hikaserial::get('inc.stream');
				hikaserialStreamInc::$cache['image.png'] = '';
				imagepng($qrcode_image, 'serialMem://image.png');
				$ret = $pdf->Image('serialMem://image.png', $d['x'], $d['y'], $imgW, $imgH, 'png');
				unset(hikaserialStreamInc::$cache['image.png']);
			}
			imagedestroy($qrcode_image);
		}
		return null;
	}

	private function generateImageBarCode($pdf, $d, $format_ex = null) {
		$format = 'ean13';
		$width = 1;
		$showText = true;
		if(empty($d['h']))
			$d['h'] = 40;
		if(empty($d['size']))
			$d['size'] = 1;

		if(!empty($format_ex))
			$format_ex = strtolower($format_ex);
		if(!empty($format_ex) && in_array($format_ex, array('code39','int25','ean13','upca','upce','code128','ean8','postnet')))
			$format = $format_ex;

		if(method_exists($pdf, 'write1DBarcode')) {
			$convert = array(
				'code39' => 'C39',
				'int25' => 'I25',
				'ean13' => 'EAN13',
				'ean8' => 'EAN8',
				'upca' => 'UPCA',
				'upce' => 'UPCE',
				'code128' => 'C128',
				'postnet' => 'POSTNET'
			);
			$rgb = (!empty($d['color']) && strlen($d['color']) >= 6) ? str_split(ltrim($d['color'], '#'), 2) : array(0,0,0);
			$style = array(
				'position' => '',
				'align' => 'C',
				'stretch' => false,
				'fitwidth' => true,
				'cellfitalign' => '',
				'border' => true,
				'hpadding' => 'auto',
				'vpadding' => 'auto',
				'fgcolor' => array(hexdec($rgb[0]), hexdec($rgb[1]), hexdec($rgb[2])),
				'bgcolor' => false,
				'text' => $showText,
				'font' => 'helvetica',
				'fontsize' => 8,
				'stretchtext' => 4
			);
			$pdf->write1DBarcode($d['text'], $convert[$format], $d['x'], $d['y'], '', $d['h'], hikaserial::toFloat($d['size']), $style, 'N');
		} else {
			$BarCode = hikaserial::get('inc.barcode', $format, $d['h'], $d['size'], $showText);
			$barcode_image = $BarCode->getImage($d['text']);
			if(empty($barcode_image))
				return null;

			$imgH = imagesy($barcode_image);
			$imgW = imagesx($barcode_image);

			if(method_exists($pdf, 'GDImage')) {
				$pdf->GDImage($barcode_image, $d['x'], $d['y'], $imgW, $imgH);
			} else {
				hikaserial::get('inc.stream');
				hikaserialStreamInc::$cache['image.png'] = '';
				imagepng($qrcode_image, 'serialMem://image.png');
				$ret = $pdf->Image('serialMem://image.png', $d['x'], $d['y'], $imgW * $pixelPerPoint, $imgH * $pixelPerPoint, 'png');
				unset(hikaserialStreamInc::$cache['image.png']);
			}
			imagedestroy($barcode_image);
		}

		return null;
	}

	public function getFilePath($filename) {
		$filename = trim($filename);
		if(empty($filename))
			return $filename;

		jimport('joomla.filesystem.file');
		$shopConfig = hikaserial::config(false);
		$ret = '';

		$uploadFolder = ltrim(JPath::clean(html_entity_decode($shopConfig->get('uploadfolder'))),DS);
		$uploadFolder = rtrim($uploadFolder,DS).DS;
		if((!preg_match('#^([A-Z]:)?/.*#', $uploadFolder)) && ($uploadFolder[0] != '/' || !is_dir($uploadFolder)))
			$uploadFolder = rtrim(JPath::clean(HIKASHOP_ROOT.DS.trim($uploadFolder, DS.' ').DS), DS.' ') . DS;
		$clean_filename = JPath::clean(realpath($uploadFolder.$filename));

		$uploadSecureFolder = ltrim(JPath::clean(html_entity_decode($shopConfig->get('uploadsecurefolder'))),DS);
		$uploadSecureFolder = rtrim($uploadSecureFolder,DS).DS;
		if((!preg_match('#^([A-Z]:)?/.*#', $uploadSecureFolder)) && ($uploadSecureFolder[0] != '/' || !is_dir($uploadSecureFolder)))
			$uploadSecureFolder = rtrim(JPath::clean(HIKASHOP_ROOT.DS.trim($uploadSecureFolder, DS.' ').DS), DS.' ') . DS;
		$clean_secure_filename = JPath::clean(realpath($uploadSecureFolder.$filename));

		if(JFile::exists($clean_filename)) {
			$ret = $clean_filename;
		} elseif(JFile::exists($clean_secure_filename)) {
			$ret = $clean_secure_filename;
		} elseif(JFile::exists(HIKASHOP_MEDIA.'images'.DS.$filename)) {
			$ret = HIKASHOP_MEDIA.'images'.DS.$filename;
		} elseif(JFile::exists(HIKASHOP_ROOT.'images'.DS.$filename)) {
			$ret = HIKASHOP_ROOT.'images'.DS.$filename;
		}

		return $ret;
	}

	private function includeImage($pdf, $d) {
		$file_path = $this->getFilePath($d['text']);
		if(empty($file_path))
			return null;

		$extension = strtolower(substr($file_path, strrpos($file_path, '.') + 1));
		$file_path = JPath::clean($file_path);

		$finalW = 0;
		$finalH = 0;
		if(!empty($d['w']) || !empty($d['h'])) {
			if(empty($d['w'])) $d['w'] = 0;
			if(empty($d['h'])) $d['h'] = 0;
			list($imgW, $imgH, $imgType, $imgAttr) = getimagesize($file_path);
			$scale = $this->scaleImage($imgW, $imgH, (int)$d['w'], (int)$d['h']);
			if(!empty($scale)) {
				$finalW = $scale[0];
				$finalH = $scale[1];
			}
		}

		$pdf->Image($file_path, $d['x'], $d['y'], $finalW, $finalH);
		return null;
	}

	private function scaleImage($x, $y, $cx, $cy, $scaleMode = 'inside') {
		if(empty($cx)) $cx = 999999;
		if(empty($cy)) $cy = 999999;
		if ($x >= $cx || $y >= $cy) {
			$rx = 0; $ry = 0;
			if ($x>0) $rx = $cx / $x;
			if ($y>0) $ry = $cy / $y;

			if($scaleMode == 'outside') {
				$r = ($rx > $ry) ? $rx : $ry;
			} else {
				$r = ($rx > $ry) ? $ry : $rx;
			}
			$x = intval($x * $r);
			$y = intval($y * $r);
			return array($x,$y);
		}
		return false;
	}

	private function sendHeader($extension, $serial = '') {
		if($this->params->get('force_download', 0)) {
			$filename = $this->params->get('force_download_filename', '');
			if(empty($filename))
				$filename = $serial;
			if(empty($filename))
				$filename = 'serial';

			jimport('joomla.filesystem.file');
			$filename = strtolower(JFile::makeSafe(trim($filename)));

			header('Content-Type: application/octet-stream');
			header('Content-Disposition: attachment; filename="' . $filename . '.' . $extension . '"');

			return true;
		}

		header('Content-Type: application/pdf');
		return true;
	}
}
