<?php
/**
 * @package    HikaSerial for Joomla!
 * @version    4.0.2
 * @author     Obsidev S.A.R.L.
 * @copyright  (C) 2011-2023 OBSIDEV. All rights reserved.
 * @license    GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined('_JEXEC') or die('Restricted access');
?><?php
class plgHikaserialPointsgen extends hikaserialPlugin {

	protected $type = 'generator';
	protected $multiple = true;
	protected $populate = true;
	protected $name = 'pointsgen';
	protected $doc_form = 'pointsgen-';
	public $test = false;

	protected $serial_default_format = '[a-zA-Z0-9]{size}';
	protected $pluginConfig = array(
		'value' => array('POINTS_VALUE', 'input'),
		'currency_rate' => array('POINTS_CURRENCY_RATE', 'input', '', ' ¤ = 1 point'),
		'size' => array('SERIAL_SIZE', 'input'),
		'format' => array('SERIAL_FORMAT', 'input', '[a-zA-Z0-9]{size}'),
		'test' => array('SERIAL_TEST', 'serial_test')
	);

	public function __construct(&$subject, $config) {
		parent::__construct($subject, $config);
	}

	public function onPluginConfiguration(&$elements) {
		$ret = parent::onPluginConfiguration($elements);

		$shopConfig = hikaserial::config(false);
		$main_currency = $shopConfig->get('main_currency', 1);
		$currencyClass = hikaserial::get('shop.class.currency');
		$currency = $currencyClass->get($main_currency);

		$this->pluginConfig['currency_rate'][3] = JText::sprintf('POINTS_CONVERSION', $currency->currency_code . ' ' . $currency->currency_symbol);

		return $ret;
	}

	public function generate(&$pack, &$order, $quantity, &$serials) {
		if(!isset($pack->pointsgen))
			return;

		parent::pluginParams($pack->pointsgen);

		if(empty($this->plugin_params->format) || !preg_match_all('#\\\[|\\\]|\[[^]]+\]\{.*\}|\[.*\]|.#iU', $this->plugin_params->format, $matches)) {
			$matches = array(array('[a-zA-Z0-9]{size}'));
		}

		$config = hikaserial::config();
		$fastRandom = (int)$config->get('use_fast_random', 0);

		if(empty($this->plugin_params->size) || $this->plugin_params->size == 0) {
			$this->plugin_params->size = 15;
		}

		for($q = 0; $q < $quantity; $q++) {
			$serial = '';
			$serialObj = new stdClass();

			$randomize = $this->initRandomize($fastRandom);

			foreach($matches[0] as $m) {
				if(strlen($m) == 1) {
					$serial .= $m;
					continue;
				}

				list($format, $repeat) = $this->generatorExtractRepeater($m);

				$serial .= $this->getRandomContent($format, $repeat, $randomize, $fastRandom);
			}

			if(!$this->test) {
				if(!isset($serialObj->extradata))
					$serialObj->extradata = array();

				$serialObj->extradata['points_value'] = (int)$this->plugin_params->value;

				if(!empty($this->plugin_params->currency_rate)) {
					$v = hikaserial::toFloat(trim($this->plugin_params->currency_rate));
					$product_price = 0;
					$p = $this->retrieveOrderPackProduct($order, $pack);
					if(!empty($p)) {
						$product_price = hikaserial::toFloat($p->order_product_price);
						if(!empty($this->plugin_params->discount_percent_tax))
							$product_price += hikaserial::toFloat($p->order_product_tax);
					}

					if(!empty($product_price)) {
						$shopConfig = hikaserial::config(false);
						$main_currency = $shopConfig->get('main_currency', 1);
						$currencyClass = hikaserial::get('shop.class.currency');
						$currency = $currencyClass->get($main_currency);

						if(isset($order->order_currency_id))
							$order_currency_id = $order->order_currency_id;
						else
							$order_currency_id = hikashop_getCurrency();

						if($main_currency != $order_currency_id)
							$product_price = $currencyClass->convertUniquePrice($product_price, $main_currency, $order_currency_id);

						$serialObj->extradata['points_value'] += (int)($product_price / $v);
					}
				}

				if(!empty($order->hikaserial->formData['points_value'])) {
					$serialObj->extradata['points_value'] += (int)$order->hikaserial->formData['points_value'];
				}
			}

			if(!empty($serialObj) && !empty($serialObj->extradata)) {
				$serialObj->data = $serial;
				$serials[] = $serialObj;
			} else {
				$serials[] = $serial;
			}
		}
	}


	public function configurationHead() {
		return array();
	}

	public function configurationLine($id = 0, $conf = null) {
		return null;
	}

	public function populateForm(&$pack) {
		$ret = '';

		if(isset($pack->pointsgen))
			parent::pluginParams($pack->pointsgen);

		if(isset($this->plugin_params)) {
			$ret .= '<tr>
				<td class="key"><label>'.JText::_('POINTS_VALUE').'</label></td>
				<td><strong>'.$this->plugin_params->value.'</strong></td>
			</tr>';

			if(!empty($this->plugin_params->currency_rate)) {
				$shopConfig = hikaserial::config(false);
				$main_currency = $shopConfig->get('main_currency', 1);
				$currencyClass = hikaserial::get('shop.class.currency');
				$currency = $currencyClass->get($main_currency);

				$ret .= '<tr>
					<td class="key"><label>'.JText::_('POINTS_CURRENCY_RATE').'</label></td>
					<td><strong>'.$this->plugin_params->currency_rate.' '.JText::sprintf('POINTS_CONVERSION', $currency->currency_code . ' ' . $currency->currency_symbol).'</strong></td>
				</tr>';
			}

			$ret .= '<tr>
				<td class="key"><label>'.JText::_('ADD_POINTS').'</label></td>
				<td><input type="text" name="data[points_value]" value="0"/></td>
			</tr>';
		} else {
			$ret .= '<tr>
				<td class="key"><label>'.JText::_('POINTS_VALUE').'</label></td>
				<td><input type="text" name="data[points_value]" value=""/></td>
			</tr>';
		}

		return $ret;
	}

	private function getAUP($warning = false) {
		static $aup = null;
		if(!isset($aup)) {
			$aup = false;
			$api_AUP = JPATH_SITE.DS.'components'.DS.'com_alphauserpoints'.DS.'helper.php';
			if(file_exists($api_AUP)) {
				require_once ($api_AUP);
				if(class_exists('AlphaUserPointsHelper'))
					$aup = true;
			}
			if(!$aup && $warning) {
				$app = JFactory::getApplication();
				if(hikaserial::isAdmin())
					$app->enqueueMessage('The HikaShop UserPoints plugin requires the component AlphaUserPoints to be installed. If you want to use it, please install the component or use another mode.');
			}
		}
		return $aup;
	}
}
