<?php
/**
 * @package    HikaSerial for Joomla!
 * @version    4.0.2
 * @author     Obsidev S.A.R.L.
 * @copyright  (C) 2011-2023 OBSIDEV. All rights reserved.
 * @license    GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined('_JEXEC') or die('Restricted access');
?><?php
class plgHikaserialTimelimitedgen extends hikaserialPlugin {

	protected $type = 'generator';
	protected $multiple = true;
	protected $populate = true;
	protected $name = 'timelimitedgen';
	protected $doc_form = 'timelimitedgen-';

	protected $serial_default_format = '[a-zA-Z0-9]{size}';

	protected $pluginConfig = array(
		'validity_value' => array('DISCOUNT_VALIDITY_PERIOD', 'period-value'),
		'validity_period' => array('', 'period-type'),
		'size' => array('SERIAL_SIZE', 'input'),
		'format' => array('SERIAL_FORMAT', 'input', '[a-zA-Z0-9]{size}'),
		'test' => array('SERIAL_TEST', 'serial_test')
	);


	public function __construct(&$subject, $config) {
		parent::__construct($subject, $config);
	}

	public function generate(&$pack, &$order, $quantity, &$serials) {
		if(!isset($pack->timelimitedgen))
			return;

		parent::pluginParams($pack->timelimitedgen);

		if(empty($this->plugin_params->format) || !preg_match_all('#\\\[|\\\]|\[[^]]+\]\{.*\}|\[.*\]|.#iU', $this->plugin_params->format, $matches)) {
			$matches = array(array('[a-zA-Z0-9]{size}'));
		}

		$config = hikaserial::config();
		$fastRandom = (int)$config->get('use_fast_random', 0);

		if(empty($this->plugin_params->size) || $this->plugin_params->size == 0) {
			$this->plugin_params->size = 15;
		}

		for($q = 0; $q < $quantity; $q++) {
			$serial = '';
			$serialObj = new stdClass();

			$randomize = $this->initRandomize($fastRandom);

			foreach($matches[0] as $m) {
				if(strlen($m) == 1) {
					$serial .= $m;
					continue;
				}

				list($format, $repeat) = $this->generatorExtractRepeater($m);

				$serial .= $this->getRandomContent($format, $repeat, $randomize, $fastRandom);
			}

			if(!empty($this->plugin_params->validity_period) && !empty($this->plugin_params->validity_value) && (int)$this->plugin_params->validity_value > 0) {
				$date_d = date('d');
				$date_m = date('m');
				$date_y = date('Y');
				$v = (int)$this->plugin_params->validity_value;
				$discount_end = -1;
				switch($this->plugin_params->validity_period) {
					case 'year':
						$discount_end = mktime(0,0,0, $date_m, $date_d, $date_y + $v);
						break;
					case 'month':
						$discount_end = mktime(0,0,0, $date_m + $v, $date_d, $date_y);
						break;
					case 'day':
						$discount_end = mktime(0,0,0, $date_m, $date_d + $v, $date_y);
						break;
				}
				if($discount_end > 0) {
					if(!isset($serialObj->extradata))
						$serialObj->extradata = array();
					$serialObj->extradata['validity_end'] = $discount_end;
				}
			}

			if(empty($this->test) && !empty($serialObj) && !empty($serialObj->extradata)) {
				$serialObj->data = $serial;
				$serials[] = $serialObj;
			} else {
				$serials[] = $serial;
			}
		}
		return true;
	}

	public function onBeforeSerialConsume(&$serial, $user_id, &$do, &$extra_data) {
		if(substr($serial->pack->pack_generator, 0, 19) != 'plg.timelimitedgen-')
			return;

		if(!empty($extra_data) && !empty($extra_data['validity_end'])) {
			$do = false;
			return false;
		}

		if(empty($serial->serial_extradata['validity_end']))
			return;

		$date = (int)$serial->serial_extradata['validity_end'];
		if($date < time())
			$do = false;
	}

	public function configurationHead() {
		return array(
			1 => 'format',
			2 => 'size',
			2 => 'validity'
		);
	}

	public function configurationLine($id = 0, $conf = null) {
		switch($id) {
			case 1:
				if(empty($conf->generator_params->format))
					return $this->serial_default_format;
				return $conf->generator_params->format;
			case 2:
				if(empty($conf->generator_params->size))
					return 12;
				return $conf->generator_params->size;
			case 3:
				$v = (int)$conf->generator_params->validity_value;
				if(empty($v))
					return JText::_('UNLIMITED');
				if($v == 1)
					return JText::sprintf('HIKASERIAL_PERIOD_' . strtoupper($this->generator_params->validity_period), $v);
				return JText::sprintf('HIKASERIAL_PERIOD_' . strtoupper($this->generator_params->validity_period) . 'S', $v);
		}
		return null;
	}
}
