<?php
/**
 * @package    HikaSerial for Joomla!
 * @version    4.0.2
 * @author     Obsidev S.A.R.L.
 * @copyright  (C) 2011-2023 OBSIDEV. All rights reserved.
 * @license    GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined('_JEXEC') or die('Restricted access');
?><?php
class plgHikashopSerials extends JPlugin {
	public function __construct(&$subject, $config) {
		parent::__construct($subject, $config);
	}

	private function init() {
		static $init = null;
		if($init !== null)
			return $init;

		$init = defined('HIKASERIAL_COMPONENT');
		if(!$init) {
			$filename = rtrim(JPATH_ADMINISTRATOR,DS).DS.'components'.DS.'com_hikaserial'.DS.'helpers'.DS.'helper.php';
			if(file_exists($filename)) {
				include_once($filename);
				$init = defined('HIKASERIAL_COMPONENT');
			}
		}
		return $init;
	}

	protected function _isAdmin() {
		static $isAdmin = null;
		if($isAdmin !== null)
			return $isAdmin;
		if(function_exists('hikashop_isClient'))
			$isAdmin = hikashop_isClient('administrator');
		else
			$isAdmin = JFactory::getApplication()->isAdmin();
		return $isAdmin;
	}

	public function onProductFormDisplay(&$product, &$html) {
	}

	public function onProductBlocksDisplay(&$product, &$html) {
		if(!$this->init())
			return;

		$params = new HikaParameter('');
		if(!empty($product->product_id)) {
			$params->set('product_id', $product->product_id);
		} else {
			$params->set('product_id', 0);
		}
		$js = '';
		$ret = hikaserial::getLayout('productserial', 'shop_block', $params, $js);
		if(!empty($ret))
			$html[] = $ret;

		if(!hikaserial::level(1))
			return;
		$ret = hikaserial::getLayout('productserial', 'shop_subscription', $params, $js);
		if(!empty($ret))
			$html[] = $ret;
	}

	public function onMarketProductBlocksDisplay(&$product, &$html) {
		if(!defined('HIKAMARKET_COMPONENT') || !$this->init())
			return;
		$marketConfig = hikamarket::config();
		if(!$marketConfig->get('frontend_edition',0))
			return;
		if(!hikamarket::acl('product/edit/plugin/hikaserial') && !hikamarket::acl('product/edit/plugin/hikasubscription'))
			return;

		$params = new HikaParameter('');
		$params->set('product_id', !empty($product->product_id) ? $product->product_id : 0);
		if(hikamarket::acl('product/edit/plugin/hikaserial')) {
			$js = '';
			$ret = hikaserial::getLayout('productserial', 'market_block', $params, $js);
			if(!empty($ret))
				$html[] = $ret;
		}
		if(hikaserial::level(1) && hikamarket::acl('product/edit/plugin/hikasubscription')) {
			$js = '';
			$ret = hikaserial::getLayout('productserial', 'market_subscription_block', $params, $js);
			if(!empty($ret))
				$html[] = $ret;
		}
	}

	public function onMarketAclPluginListing(&$categories) {
		if(!$this->init())
			return;

		$categories['product'][] = 'hikaserial';
		if(hikaserial::level(1))
			$categories['product'][] = 'hikasubscription';

		if(empty($categories['root']['plugins']))
			$categories['root']['plugins'] = array();
		$categories['root']['plugins']['hikaserial'] = array(
			'pack' => array(
				'listing',
				'add',
				'edit' => array(
					'name',
					'data',
					'generator',
					'published',
					'description',
					'vendor'
				)
			),
			'serial' => array(
				'listing',
				'export',
				'import'
			)
		);

		if(!hikaserial::level(1))
			return;
	}

	public function onVendorPanelDisplay(&$buttons, &$statistics) {
		if(!$this->init())
			return;

		$marketConfig = hikamarket::config();
		$marketVersion = $marketConfig->get('version', '1.0.0');
		if(version_compare($marketVersion, '1.6.6', '>=')) {
			$buttons['hikaserial'] = array(
				'url' => hikamarket::completeLink('serials'),
				'level' => 1,
				'icon' => 'iconM-48-serial',
				'fa' => array('<i class="fas fa-stack-2x fa-barcode"></i>','<i class="fas fa-stack-1x fa-key hk-icon-dark"></i>'),
				'name' => JText::_('HIKA_SERIALS'),
				'description' => '',
				'display' => hikamarket::acl('plugins/hikaserial')
			);
		}

		if(hikamarket::acl('vendor/statistics')) {
			$vendor_id = hikamarket::loadVendor(false);
			$db = JFactory::getDBO();

			if(!$this->init())
				return;

			$statistics['serial_count'] = array(
				'slot' => 1,
				'order' => 3,
				'published' => 0,
				'type' => 'plugin',
				'plugin' => 'hikaserial',
				'label' => JText::_('HIKA_SERIALS'),
				'query' => array(
					'get' => 'list',
					'select' => 'pack.pack_name, pack.pack_id, serial.serial_status, COUNT(serial.serial_id) as count',
					'tables' => array(
						'serial' => hikaserial::table('serial').' AS serial ',
						'pack' => 'INNER JOIN '.hikaserial::table('pack').' AS pack ON serial.serial_pack_id = pack.pack_id ',
						'hk_order' => 'INNER JOIN '.hikaserial::table('shop.order').' AS hk_order ON serial.serial_order_id = hk_order.order_parent_id',
					),
					'filters' => array(
						'order_vendor' => ('hk_order.order_vendor_id = ' . (int)$vendor_id),
						'order_type' => 'hk_order.order_type = ' . $db->Quote('subsale'),
					),
					'group' => 'pack.pack_name, serial.serial_status',
					'order' => 'pack.pack_id, serial.serial_status'
				)
			);

			if($vendor_id <= 1) {
				$statistics['serial_count']['query']['tables']['hk_order'] = 'INNER JOIN '.hikaserial::table('shop.order').' AS hk_order ON serial.serial_order_id = hk_order.order_id';
				$statistics['serial_count']['query']['filters']['order_type'] = 'hk_order.order_type = ' . $db->Quote('sale');
				unset($statistics['serial_count']['query']['filters']['order_vendor']);
			}
		}
	}

	public function onHikamarketPluginController($ctrl) {
		if($ctrl != 'serials')
			return;

		if($this->_isAdmin() || !$this->init())
			return;

		return array(
			'type' => '',
			'component' => 'com_hikaserial',
			'name' => 'serials',
			'file' => 'market'
		);
	}

	public function onHikashopPluginController($ctrl) {
		if(!in_array($ctrl, array('serial','subscription')))
			return;

		if($this->_isAdmin() || !$this->init())
			return;

		if($ctrl == 'serial') {
			return array(
				'type' => '',
				'component' => 'com_hikaserial',
				'name' => 'serial',
				'file' => 'shop_serial'
			);
		} else if($ctrl == 'subscription') {
			return array(
				'type' => '',
				'component' => 'com_hikaserial',
				'name' => 'subscription',
				'file' => 'shop_subscription'
			);
		}
		return;
	}

	public function onHikamarketStatisticPluginDisplay($data) {
		if(!isset($data['type']) || $data['type'] != 'plugin')
			return;
		if(!isset($data['plugin']) || $data['plugin'] != 'hikaserial')
			return;

		if(empty($data['value']))
			return JText::_('HIKAM_EMPTY_CHART');

		$ret = '<table style="width:100%" class="hikamarket_stat_table"><thead><tr><th>'.JText::_('PACK_NAME').'</th><th>'.JText::_('SERIAL_STATUS').'</th><th>'.JText::_('TOTAL_SERIALS').'</th></tr></thead><tbody>';
		foreach($data['value'] as $v) {
			$ret .= '<tr><td>'.htmlentities($v->pack_name).'</td><td>'.htmlentities($v->serial_status).'</td><td>'. (int)$v->count.'</td></tr>';
		}
		$ret .= '</tbody></table>';
		return $ret;
	}

	public function onUserAccountDisplay(&$buttons) {
		if(!$this->init())
			return;

		$serialConfig = hikaserial::config();
		$show_serials = ((int)$serialConfig->get('front_serial_listing', 1) == 1);
		$show_subscriptions = ((int)$serialConfig->get('front_subscription_listing', 1) == 1) && hikaserial::level(1);
		$show_downloads = ((int)$serialConfig->get('front_download_listing', 1) == 1) && hikaserial::level(1);
		if(!$show_serials && !$show_subscriptions && !$show_downloads)
			return;

		$cancel_id = '';
		if($show_serials) {
			$buttons['serial_listing'] = array(
				'link' => hikaserial::completeLink('shop.serial&task=listing'.$cancel_id),
				'image' => 'serial',
				'fontawesome' => '<i class="fas fa-stack-2x fa-barcode"></i><i class="fas fa-stack-1x fa-key hk-icon-dark"></i>',
				'level' => 0,
				'text' => JText::_('HIKA_SERIALS'),
				'description' => JText::_('HIKA_SERIAL_LISTING_DESCRIPTION'),
			);
		}
		if($show_subscriptions) {
			$buttons['subscription_listing'] = array(
				'link' => hikaserial::completeLink('shop.subscription&task=listing'.$cancel_id),
				'image' => 'subscription',
				'fontawesome' => '<i class="far fa-calendar fa-stack-2x"></i><i class="fa fa-sync fa-stack-1x hk-icon-dark" style="top:12%"></i>',
				'level' => 0,
				'text' => JText::_('HIKA_SUBSCRIPTIONS'),
				'description' => JText::_('HIKA_SUBSCRIPTION_LISTING_DESCRIPTION'),
			);
		}
		if($show_downloads) {
			$buttons['subscription_downloads'] = array(
				'link' => hikaserial::completeLink('shop.subscription&task=downloads'.$cancel_id),
				'image' => 'downloads',
				'fontawesome' => '<i class="far fa-calendar fa-stack-2x"></i><i class="fas fa-arrow-down fa-stack-1x hk-icon-dark" style="top:10%"></i>',
				'level' => 0,
				'text' => JText::_('HIKA_SUBSCRIPTION_DOWNLOADS'),
				'description' => JText::_('HIKA_SUBSCRIPTION_DOWNLOADS_DESCRIPTION'),
			);
		}
		return true;
	}

	public function onAfterProductCreate(&$product) {
		if(!$this->init())
			return;

		if(!hikaserial::isAdmin() & !hikaserial::initMarket())
			return;

		$class = hikaserial::get('class.product');
		$class->saveForm($product);
	}

	public function onAfterProductUpdate(&$product) {
		if(!$this->init())
			return;

		if(!hikaserial::isAdmin() & !hikaserial::initMarket())
			return;

		$class = hikaserial::get('class.product');
		$class->saveForm($product);
	}

	public function onBeforeOrderCreate(&$order, &$do) {
		if(!$this->init())
			return;

		$class = hikaserial::get('class.order');
		$class->preUpdate($order);
	}

	public function onAfterOrderCreate(&$order, &$send_email) {
		if(!$this->init())
			return;

		$class = hikaserial::get('class.order');
		$class->postUpdate($order);
	}

	public function onBeforeOrderUpdate(&$order, &$do) {
		if(!$this->init())
			return;

		$class = hikaserial::get('class.order');
		$class->preUpdate($order);
	}

	public function onAfterOrderUpdate(&$order, &$send_email) {
		if(!$this->init())
			return;

		$class = hikaserial::get('class.order');
		$class->postUpdate($order);
	}

	public function onAfterOrderProductsListingDisplay(&$order, $type) {
		if(!$this->init())
			return;

		$types = array(
			'order_back_show', 'order_back_invoice', 'order_front_show', 'email_notification_html',
			'order_frontvendor_show', 'order_frontvendor_invoice', 'order_frontmarket_show'
		);

		if(in_array($type, $types)) {
			$params = new HikaParameter('');
			if(isset($order->order_id)) {
				$params->set('order_id', (int)$order->order_id);
			} else {
				$params->set('order_id', (int)$order->products[0]->order_id);
			}
			$params->set('order_obj', $order);
			$js = '';
			echo hikaserial::getLayout('orderserial', 'show_'.$type, $params, $js);
		}
	}

	public function onMarketOrderEditionLoading(&$order) {
		if(!$this->init())
			return;

		$block = hikaInput::get()->getString('block', null);
		if(!empty($block) && !in_array($block, array('product', 'edit_product')))
			return;

		$marketHelper = hikaserial::get('helper.market');
		$marketHelper->processOrderEditionLoading($order, $block);
	}

	public function onBeforeCreateRecurringSuborder($order_id, &$order) {
		if(!$this->init() || !hikaserial::level(1))
			return;
		$orderClass = hikaserial::get('class.order');
		$orderClass->beforeCreateRecurringSuborder($order_id, $order);
	}

	public function onBeforeMailSend(&$mail, &$mailer) {
		if(!$this->init())
			return;

		$mailClass = hikaserial::get('class.mail');
		$mailClass->beforeMailSend($mail, $mailer);
	}

	public function onMailTemplateListing(&$files, $mail_name) {
		if(!in_array($mail_name, array('mailtocustomfield', 'pack_quantity_low', 'subscription_expiration_notice')))
			return;
		jimport('joomla.filesystem.folder');
		$emailFiles = JFolder::files(HIKASHOP_MEDIA.'mail'.DS.'template'.DS, '^([-_A-Za-z]*)(\.html)?\.php$');
		if(empty($emailFiles))
			return;
		foreach($emailFiles as $emailFile) {
			$file = str_replace(array('.html.php', '.php'), '', $emailFile);
			if(substr($file, -9) == '.modified')
				continue;
			$key = strtoupper($file);
			$files[$file] = array(
				'folder' => HIKASHOP_MEDIA.'mail'.DS.'template'.DS,
				'filename' => $file,
				'file' => $file
			);
		}
	}

	public function onBeforeDownloadFile(&$filename, &$do, &$file) {
		if(!$this->init())
			return;
		$downloadClass = hikaserial::get('class.download');
		if(!empty($downloadClass))
			$downloadClass->beforeDownloadFile($filename, $do, $file);
	}

	public function onBeforeOrderExport(&$rows, &$view) {
		if(!$this->init())
			return;
		$orderClass = hikaserial::get('class.order');
		$orderClass->beforeOrderExport($rows, $view);
	}

	public function onViewsListingFilter(&$views, $client) {
		if(!$this->init())
			return;

		switch($client){
			case 0:
				$views[] = array(
					'client_id' => 0,
					'name' => HIKASERIAL_NAME,
					'component' => HIKASERIAL_COMPONENT,
					'view' => HIKASERIAL_FRONT.'views'.DS
				);
				break;
			case 1:
				$views[] = array(
					'client_id' => 1,
					'name' => HIKASERIAL_NAME,
					'component' => HIKASERIAL_COMPONENT,
					'view' => HIKASERIAL_BACK.'views'.DS
				);
				break;
			default:
				$views[] = array(
					'client_id' => 0,
					'name' => HIKASERIAL_NAME,
					'component' => HIKASERIAL_COMPONENT,
					'view' => HIKASERIAL_FRONT.'views'.DS
				);
				$views[] = array(
					'client_id' => 1,
					'name' => HIKASERIAL_NAME,
					'component' => HIKASERIAL_COMPONENT,
					'view' => HIKASERIAL_BACK.'views'.DS
				);
				break;
		}
	}

	public function onHikashopBeforeDisplayView(&$viewObj) {
		$viewName = $viewObj->getName();
		$layout = $viewObj->getLayout();

		if($viewName == 'menu') {
			if(!$this->_isAdmin() || !$this->init())
				return;
			$class = hikaserial::get('class.menu');
			$class->processView($viewObj);
			return;
		}

		if($viewName == 'order' && $layout == 'listing') {
			if($this->_isAdmin() || !$this->init())
				return;
			$class = hikaserial::get('class.order');
			$class->processView($viewObj);
			return;
		}
	}

	public function onHikashopLanguageChange($locale) {
		$path = hikashop_getLanguagePath(JPATH_ROOT).DS.$locale.DS.$locale.'.com_hikaserial.ini';
		if(file_exists($path))
			hikashop_loadTranslationFile($path);
	}

	public function onMailListing(&$files) {
		if(!$this->init())
			return;

		jimport('joomla.filesystem.folder');
		$emailFiles = JFolder::files(HIKASERIAL_MEDIA.'mail'.DS, '^([-_A-Za-z]*)(\.html)?\.php$');
		if(empty($emailFiles))
			return;
		foreach($emailFiles as $emailFile) {
			$file = str_replace(array('.html.php', '.php'), '', $emailFile);
			if(substr($file, -9) == '.modified')
				continue;
			$key = strtoupper($file);
			$files[] = array(
				'folder' => HIKASERIAL_MEDIA.'mail'.DS,
				'name' => JText::_('SERIAL_' . $key),
				'filename' => $file,
				'file' => 'serial.'.$file
			);
		}
	}

	public function onHikashopCronTrigger(&$messages) {
		if(!$this->init() || !hikaserial::level(1))
			return;
		$cronHelper = hikaserial::get('helper.cron');
		$cronHelper->process($messages);
	}

	public function onOrderStatusListingLoad(&$orderstatus_columns, &$rows) {
		if(!$this->init())
			return;

		$config = hikaserial::config();
		$orderstatus_columns['serial'] = array(
			'text' => JText::_('HIKA_SERIALS'),
			'title' => JText::_('SERIAL_ASSIGNABLE_ORDER_STATUSES'),
			'description' => JText::_('SERIAL_ASSIGNABLE_ORDER_STATUSES_DESC'),
			'key' => '',
			'default' => $config->get('assignable_order_statuses', 'confirmed,shipped'),
			'type' => 'toggle',
			'trigger' => 'plg.hikashop.serials.serialAssignableStatusUpdate'
		);
	}

	public function serialAssignableStatusUpdate(&$controller, $elementPkey, $value, $extra) {
		if(!$this->init())
			return;
		$config = hikaserial::config();
		$currentValue = explode(',', $config->get('assignable_order_statuses', 'confirmed,shipped'));
		if($value) {
			if(!in_array($elementPkey, $currentValue)) {
				$currentValue[] = $elementPkey;
			}
		} else if(in_array($elementPkey, $currentValue)) {
			$key = array_search($elementPkey,$currentValue);
			unset($currentValue[$key]);
		}
		$data = array('assignable_order_statuses' => trim(implode(',', $currentValue),','));
		$config->save($data);
	}

	public function onCheckoutWorkflowLoad(&$workflow, &$shop_closed, $cart_id) {
		if(empty($cart_id))
			return;
		$cartClass = hikashop_get('class.cart');
		$cart = $cartClass->get($cart_id);
		unset($cartClass);
		if($cart->cart_type != 'subscription')
			return;
		if(!$this->init() || !hikaserial::level(1))
			return;
		$checkoutClass = hikaserial::get('class.checkout');
		$checkoutClass->onCheckoutWorkflowLoad($workflow, $shop_closed, $cart_id, $cart);
	}

	public function onUnknownCheckCartQuantities(&$cart, $parent_products, &$ret) {
		if($cart->cart_type != 'subscription')
			return;
		if(!$this->init() || !hikaserial::level(1))
			return;
		$cartClass = hikaserial::get('class.cart');
		$cartClass->onCheckCartQuantities($cart, $parent_products, $ret);
	}

	public function onAfterProductCheckQuantities(&$products, &$cart, $options) {
		if(!$this->init() || !hikaserial::level(1))
			return;
		$cartClass = hikaserial::get('class.cart');
		$cartClass->onProductCheckQuantities($products, $cart, $options);
	}

	public function onAfterCheckCartQuantities(&$cart, $parent_products, &$ret) {
		if(!$this->init() || !hikaserial::level(1))
			return;
		$cartClass = hikaserial::get('class.cart');
		$cartClass->onCheckCartQuantities($cart, $parent_products, $ret);
	}

	public function onCompareCartProducts($product, $cart_product, &$do) {
		if(!$do)
			return;
		if(!$this->init() || !hikaserial::level(1))
			return;
		$cartClass = hikaserial::get('class.cart');
		$cartClass->onCompareCartProducts($product, $cart_product, $do);
	}

	public function onCheckPaymentOptions(&$paymentOptions, &$order) {
		if(!$this->init() || !hikaserial::level(1))
			return;
		$cartClass = hikaserial::get('class.cart');
		$cartClass->checkPaymentOptions($paymentOptions, $order);
	}

	public function onCheckoutStepList(&$list) {
		$list['plg.serial.coupon'] = 'HikaSerial ' . JText::_('HIKASHOP_COUPON');
	}

	public function onAfterCartProductsLoad(&$cart) {
		if(empty($cart->cart_params) || empty($cart->cart_params->subscription))
			return;
		if(!$this->init() || !hikaserial::level(1))
			return;
		$cartClass = hikaserial::get('class.cart');
		$cartClass->onAfterCartProductsLoad($cart);
	}

	public function onAfterCartSave(&$cart) {
		if(empty($cart->cart_products))
			return;
		$f = false;
		foreach($cart->cart_products as $cp) {
			if(!empty($cp->extra) && !empty($cp->extra['sub'])) {
				$f = true;
				break;
			}
		}
		if(!$f)
			return false;
		if(!$this->init() || !hikaserial::level(1))
			return;
		$cartClass = hikaserial::get('class.cart');
		$cartClass->onAfterCartSaveWithSub($cart);
	}

	public function onCheckoutStepDisplay($layoutName, &$html, &$view, $pos = null, $options = null) {
		if($layoutName == 'plg.serial.coupon' || $layoutName == 'serialcoupon') {
			if(!$this->init())
				return;

			$params = new stdClass();
			$params->view = $view;
			$params->pos = $pos;
			$params->options = $options;
			$js = null;

			$shopConfig = hikaserial::config(false);
			if($shopConfig->get('checkout_legacy', 0)) {
				$html .= hikaserial::getLayout('checkoutserial', 'coupon', $params, $js);
			} else {
				$html .= hikaserial::getLayout('checkoutserial', 'show_block_coupon', $params, $js);
			}
		}
	}

	public function onBeforeCheckoutStep($controllerName, &$go_back, $original_go_back, &$controller) {
	}

	public function onAfterCheckoutStep($controllerName, &$go_back, $original_go_back, &$controller) {
		if($controllerName == 'plg.serial.coupon' || $controllerName == 'serialcoupon') {
			if(!$this->init())
				return;

			$checkoutClass = hikaserial::get('class.checkout');
			return $checkoutClass->afterCheckoutStep($controllerName, $go_back, $original_go_back, $controller);
		}
	}

	public function onNameboxTypesLoad(&$types) {
		if(!$this->init())
			return;
		$types['plg.hikaserial.pack'] = array(
			'class' => array(
				'file' => HIKASERIAL_CLASS . 'pack.php',
				'name' => 'hikaserialPackClass'
			),
			'name' => 'pack_id',
			'mode' => 'list',
			'displayFormat' => '{pack_name}',
			'params' => array(
				'pack_type' => 'serial'
			),
			'url' => 'index.php?option='.HIKASERIAL_COMPONENT.'&ctrl=pack&task=findList&pack_type=serial',
			'url_params' => array(),
			'options' => array(
				'olist' => array(
					'table' => array(
						'pack_name' => 'HIKA_NAME',
						'pack_data' => 'PACK_DATA',
						'pack_generator' => 'PACK_GENERATOR',
						'pack_id' => 'ID'
					),
					'displayFormat' => '{pack_name}',
				)
			)
		);
		$types['plg.hikaserial.packs'] = array(
			'class' => array(
				'file' => HIKASERIAL_CLASS . 'pack.php',
				'name' => 'hikaserialPackClass'
			),
			'name' => 'pack_id',
			'mode' => 'list',
			'displayFormat' => '{pack_name}',
			'params' => array(
				'pack_type' => ''
			),
			'url' => 'index.php?option='.HIKASERIAL_COMPONENT.'&ctrl=pack&task=findList',
			'url_params' => array(),
			'options' => array(
				'olist' => array(
					'table' => array(
						'pack_name' => 'HIKA_NAME',
						'pack_type' => 'SERIAL_PACK',
						'pack_data' => 'PACK_DATA',
						'pack_id' => 'ID'
					),
					'displayFormat' => '{pack_name}',
				)
			)
		);
		$types['plg.hikaserial.plan'] = array(
			'class' => array(
				'file' => HIKASERIAL_CLASS . 'pack.php',
				'name' => 'hikaserialPackClass'
			),
			'name' => 'pack_id',
			'mode' => 'list',
			'displayFormat' => '{pack_name}',
			'params' => array(
				'pack_type' => 'subscription'
			),
			'url' => 'index.php?option='.HIKASERIAL_COMPONENT.'&ctrl=pack&task=findList&pack_type=subscription',
			'url_params' => array(),
			'options' => array()
		);
		$types['plg.hikaserial.serial_status'] = array(
			'class' => array(
				'file' => HIKASERIAL_TYPE . 'serial_status.php',
				'name' => 'hikaserialSerial_statusType'
			),
			'name' => 'serial_status',
			'mode' => 'list',
			'displayFormat' => '{serial_status}',
			'params' => array(),
			'url' => 'index.php?option='.HIKASERIAL_COMPONENT.'&ctrl=serial_status&task=findList',
			'url_params' => array(),
			'options' => array(),
		);
		$types['plg.hikaserial.pack_group'] = array(
			'class' => array(
				'file' => HIKASERIAL_CLASS . 'pack_group.php',
				'name' => 'hikaserialPack_groupClass'
			),
			'name' => 'pack_group_id',
			'mode' => 'list',
			'displayFormat' => '{pack_group_name}',
			'params' => array(),
			'url' => 'index.php?option='.HIKASERIAL_COMPONENT.'&ctrl=pack_group&task=findList',
			'url_params' => array(),
			'options' => array(),
		);
	}

	public function onHikashopBeforeCheckDB(&$createTable, &$custom_fields, &$structure, &$helper) {
		if(!$this->init())
			return;
		$helper->parseTableFile(HIKASERIAL_BACK.'_database'.DS.'install.sql', $createTable, $structure);
		if(isset($structure['#__hikashop_order'])) {
			$structure['#__hikashop_order']['order_serial_params'] = '`order_serial_params` TEXT NOT NULL DEFAULT \'\'';
			$structure['#__hikashop_order']['order_parent_id'] = '`order_parent_id` INT(10) NOT NULL DEFAULT 0';
		}
	}
}
