<?php
/**
 * @package    HikaSerial for Joomla!
 * @version    4.0.2
 * @author     Obsidev S.A.R.L.
 * @copyright  (C) 2011-2023 OBSIDEV. All rights reserved.
 * @license    GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined('_JEXEC') or die('Restricted access');
?><?php
class hikaserialProductHelper {
	protected $db = null;

	public function __construct() {
		$this->db = JFactory::getDBO();
	}

	public function loadProducts($options = array()) {
		if(empty($options))
			return false;

		if(empty($options['filter']))
			return false;

		$default_filters = array(
			'published' => true,
			'available' => true
		);
		$options['filter'] = array_merge($default_filters, $options['filter']);

		$sql = array(
			'select' => array(
				'product' => 'p.*',
			),
			'tables' => array(),
			'filters' => array()
		);

		$this->applyFilters($sql, $options['filter']);
		if(empty($sql['filters']))
			return false;

		$query = 'SELECT '.implode(', ', $sql['select']).' FROM '.hikashop_table('product').' AS p '.implode(' ', $sql['tables']).' WHERE ('.implode(') AND (', $sql['filters']).')';
		$this->db->setQuery($query);
		$ret = $this->db->loadObjectList('product_id');

		if(!empty($options['load']))
			$this->applyLoad($ret, $options);

		return $ret;
	}

	private function applyFilters(&$sql, $filters) {
		if(empty($filters['ids']) && empty($filters['category_ids']))
			return false;

		$now = time();

		if(!empty($filters['ids'])) {
			hikaserial::toInteger($filters['ids']);
			$sql['filters']['product_ids'] = 'p.product_id IN ('.implode(',', $filters['ids']).')';
		}

		if(!empty($filters['published']))
			$sql['filters']['product_published'] = 'p.product_published = 1';

		if(!empty($filters['available']) || !empty($filters['stock']))
			$sql['filters']['product_quantity'] = 'p.product_quantity = -1 OR p.product_quantity > 0';

		if(!empty($filters['available']) || !empty($filters['sale_dates'])) {
			$sql['filters']['product_sale_start'] = 'p.product_sale_start = 0 OR p.product_sale_start < '.$now;
			$sql['filters']['product_sale_end'] = 'p.product_sale_end = 0 OR p.product_sale_end > '.$now;
		}

		if(isset($filters['user'])) {
			$acl_filters = array();
			hikashop_addACLFilters($acl_filters, 'product_access', 'p', 2, false, $filters['user']);
			foreach($acl_filters as $k => $acl_filter) {
				$sql['filters']['product_acl_'.$k] = $acl_filter;
			}
		}
	}

	private function applyLoad(&$product, $options) {
		$load = $options['load'];

		if(!empty($load['price'])) {
			$price_display_type = 'all';
			$user_id = 0;
			if(isset($options['filter']['user']))
				$user_id = (int)$options['filter']['user'];

			$currencyClass = hikaserial::get('shop.class.currency');
			$currencyClass->getProductsPrices($product, array(
				'price_display_type' => $price_display_type,
				'user_id' => $user_id
			));
		}
	}
}
