<?php
/**
 * @package    HikaSerial for Joomla!
 * @version    4.0.2
 * @author     Obsidev S.A.R.L.
 * @copyright  (C) 2011-2023 OBSIDEV. All rights reserved.
 * @license    GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
?><?php
defined('_JEXEC') or die('Restricted access');

class hikaserialPdfInc {
	private $pdf = null;
	private $options = array(
		'orientation' => 'P',
		'unit' => 'mm',
		'size' => 'A4',
	);

	public function __construct($options = array()) {
		$this->setOptions($options);
	}

	public function setOptions($options) {
		if(empty($options) || !is_array($options))
			return;

		$this->options = array_merge($this->options, $options);
		$this->reset();
	}

	public function reset() {
		if($this->pdf === null)
			return;
		unset($this->pdf);
		$this->pdf = null;
	}

	private function initLibs($elements = null) {
		if(empty($elements))
			$elements = array();
		if(!is_array($elements))
			$elements = array($elements);

		$main_libs = array('fpdf', 'tcpdf');
		$check = array_intersect($main_libs, $elements);
		if((in_array('*', $elements) || empty($check)) && !class_exists('FPDF') && !class_exists('TCPDF')) {
			$config = hikaserial::config();
			$main_lib = $config->get('main_pdf_lib', 'tcpdf');
			if(!in_array($main_lib, $main_libs))
				$main_lib = 'fpdf';
			$elements[] = $main_lib;
		}

		if(!defined('FPDF_FONTPATH'))
			define('FPDF_FONTPATH', HIKASERIAL_MEDIA.'fonts'.DS.'fpdf'.DS);
		if(!defined('K_PATH_FONTS'))
			define('K_PATH_FONTS', HIKASERIAL_MEDIA.'fonts'.DS.'tcpdf'.DS);

		if(in_array('fpdf', $elements) && !class_exists('FPDF'))
			include_once HIKASERIAL_INC.'fpdf'.DS.'fpdf.php';

		if(in_array('tcpdf', $elements) && !class_exists('TCPDF'))
			include_once HIKASERIAL_INC.'tcpdf'.DS.'tcpdf.php';

		if(in_array('fpdi', $elements) && !class_exists('FPDI'))
			include_once HIKASERIAL_INC.'fpdi'.DS.'fpdi.php';
		if(in_array('fpdi.protect', $elements) && !class_exists('FPDI_Protection'))
			include_once HIKASERIAL_INC.'fpdi'.DS.'fpdi_protection.php';
	}

	public function getDefaultPDF() {
		if($this->pdf !== null)
			return $this->pdf;

		$config = hikaserial::config();
		$main_lib = $config->get('main_pdf_lib', 'tcpdf');
		switch($main_lib) {
			case 'tcpdf':
				return $this->getTCPDF();
			case 'fpdf':
				return $this->getFPDF();
		}
		return $this->getFPDF();
	}

	public function getFPDF() {
		if($this->pdf !== null)
			return $this->pdf;

		$this->initLibs('fpdf');
		$this->pdf = new FPDF($this->options['orientation'], $this->options['unit'], $this->options['size']);
		return $this->pdf;
	}

	public function getTCPDF() {
		if($this->pdf !== null)
			return $this->pdf;

		$this->initLibs('tcpdf');
		$this->pdf = new TCPDF($this->options['orientation'], $this->options['unit'], $this->options['size']);
		return $this->pdf;
	}

	public function getFPDI($fpdf = true, $options = null) {
		if($this->pdf !== null)
			return $this->pdf;
		if(!empty($options) && is_array($options) && !empty($options['protect'])) {
			$this->initLibs(array('fpdf', 'fpdi', 'fpdi.protect'));
			$this->pdf = new FPDI_Protection($this->options['orientation'], $this->options['unit'], $this->options['size']);
			return $this->pdf;
		}
		$this->initLibs('fpdi');
		$this->pdf = new FPDI($this->options['orientation'], $this->options['unit'], $this->options['size']);
		return $this->pdf;
	}
}
