<?php
/**
 * @package    HikaSerial for Joomla!
 * @version    4.0.2
 * @author     Obsidev S.A.R.L.
 * @copyright  (C) 2011-2023 OBSIDEV. All rights reserved.
 * @license    GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined('_JEXEC') or die('Restricted access');
?><?php
class SubscriptionController extends hikaserialController {
	protected $rights = array(
		'display' => array(
			'show', 'listing', 'renew', 'upgrade', 'cancelrenew',
			'downloads', 'download',
		),
		'add' => array(),
		'edit' => array(),
		'modify' => array(),
		'delete' => array()
	);

	public function __construct($config = array(), $skip = false) {
		$config['base_path'] = rtrim(JPATH_SITE,DS).DS.'components'.DS.'com_hikaserial'.DS;
		parent::__construct($config, $skip);

		$this->display = $this->rights['display'];
		$this->modify_views = $this->rights['edit'];
		$this->add = $this->rights['add'];
		$this->modify = $this->rights['modify'];
		$this->delete = $this->rights['delete'];
	}

	private function requireJoomlaAccount() {
		$user = JFactory::getUser();
		if(!$user->guest)
			return true;

		$app = JFactory::getApplication();
		$app->enqueueMessage(JText::_('PLEASE_LOGIN_FIRST'));

		global $Itemid;
		$url = ('index.php?option=com_users&view=login') . (!empty($Itemid) ? '&Itemid=' . $Itemid : '');

		$app->redirect(JRoute::_($url . '&return='.urlencode(base64_encode(hikaserial::currentUrl())), false));
	}

	public function show() {
		$this->requireJoomlaAccount();

		$app = JFactory::getApplication();
		$cid = hikaserial::getCID('subscription_id');
		if(empty($cid))
			$app->redirect( hikaserial::completeLink('shop.subscription&task=listing', false, true));

		$hkUserId = hikaserial::loadUser(false);

		$subscriptionClass = hikaserial::get('class.subscription');
		$subscription = $subscriptionClass->get($cid);

		if(empty($subscription) || $subscription->subscription_user_id != $hkUserId) {
			$app->enqueueMessage(JText::_('HIKASERIAL_INVALID_DATA'), 'error');
			$app->redirect(hikaserial::completeLink('shop.subscription&task=listing', false, true));
		}

		hikaInput::get()->set('layout', 'show');
		return $this->display();
	}

	public function listing() {
		$this->requireJoomlaAccount();

		hikaInput::get()->set('layout', 'listing');
		return $this->display();
	}

	public function downloads() {
		$this->requireJoomlaAccount();

		hikaInput::get()->set('layout', 'downloads');
		return $this->display();
	}

	public function download() {
		$this->requireJoomlaAccount();

		$app = JFactory::getApplication();
		$sub_id = hikaInput::get()->getInt('subscription_id', 0);
		$file_id = hikaserial::getCID('file_id');
		if(empty($sub_id) || empty($file_id))
			$app->redirect( hikaserial::completeLink('shop.subscription&task=downloads', false, true));

		$hkUserId = hikaserial::loadUser(false);

		$subscriptionClass = hikaserial::get('class.subscription');
		$subscription = $subscriptionClass->get($sub_id);

		if(empty($subscription) || $subscription->subscription_user_id != $hkUserId) {
			$app->enqueueMessage(JText::_('HIKASERIAL_INVALID_DATA'), 'error');
			$app->redirect(hikaserial::completeLink('shop.subscription&task=downloads', false, true));
		}

		$downloadClass = hikaserial::get('class.download');
		$ret = $downloadClass->downloadSubscriptionFile($file_id, $subscription);

		if($ret !== true) {
			$app->redirect(hikaserial::completeLink('shop.subscription&task=downloads', false, true));
		}
		exit;
	}

	public function renew() {
		$app = JFactory::getApplication();
		$tmpl = hikaInput::get()->getCmd('tmpl', '');

		$subscription_id = hikaInput::get()->getInt('subscription', 0);
		$product_id = hikaInput::get()->getInt('product', 0);

		if(empty($subscription_id) || empty($product_id)) {
			if($tmpl == 'ajax') {
				echo '{ret:0}';
				exit;
			}
			return false;
		}

		$cartClass = hikaserial::get('class.cart');
		$status = $cartClass->addRenewSubscription($subscription_id, $product_id);

		if($tmpl == 'ajax') {
			$cart = false;
			if($status)
				$cart = $cartClass->getFullCart(0);

			$used_data = array('product' => $product_id, 'quantity' => 1);
			$ret = $cartClass->getAjaxCartData($used_data, $cart, $status);
			echo json_encode($ret);
			exit;
		}

		if($status) {
			$this->redirectToCheckout($status);
			return true;
		}
		$app->redirect(hikaserial::completeLink('shop.subscription&task=listing', false, true));
	}

	public function upgrade() {
		$app = JFactory::getApplication();
		$tmpl = hikaInput::get()->getCmd('tmpl', '');

		$subscription_id = hikaInput::get()->getInt('subscription', 0);
		$product_id = hikaInput::get()->getInt('product', 0);

		if(empty($subscription_id) || empty($product_id)) {
			if($tmpl == 'ajax') {
				echo '{ret:0}';
				exit;
			}
			return false;
		}

		$cartClass = hikaserial::get('class.cart');
		$status = $cartClass->addUpgradeSubscription($subscription_id, $product_id);

		if($tmpl == 'ajax') {
			$cart = false;
			if($status)
				$cart = $cartClass->getFullCart(0);

			$used_data = array('product' => $product_id, 'quantity' => 1);
			$ret = $cartClass->getAjaxCartData($used_data, $cart, $status);
			echo json_encode($ret);
			exit;
		}

		if($status) {
			$this->redirectToCheckout($status);
			return true;
		}
		$app->redirect(hikaserial::completeLink('shop.subscription&task=listing', false, true));
	}

	public function cancelrenew() {
		$this->requireJoomlaAccount();

		$app = JFactory::getApplication();
		$tmpl = hikaInput::get()->getCmd('tmpl', '');

		$subscription_id = hikaInput::get()->getInt('subscription', 0);
		if(empty($subscription_id))
			$app->redirect(hikaserial::completeLink('shop.subscription&task=listing', false, true));

		$user_id = hikaserial::loadUser(false);
		$subscriptionClass = hikaserial::get('class.subscription');

		$subscription = $subscriptionClass->get($subscription_id, true);
		if(empty($subscription) || $subscription->subscription_user_id != $user_id) {
			$app->enqueueMessage(JText::_('INVALID_DATA'), 'error');
			$app->redirect(hikaserial::completeLink('shop.subscription&task=listing', false, true));
		}

		if(empty($subscription->subscription_auto_renew)) {
			$app->enqueueMessage(JText::_('SUBSCRIPTION_NO_AUTO_RENEW'), 'error');
			$app->redirect(hikaserial::completeLink('shop.subscription&task=listing', false, true));
		}

		if(!hikaserial::checkToken()) {
			hikaInput::get()->set('layout', 'cancelrenew');
			return $this->display();
		}

		$ret = $subscriptionClass->cancelRenew($subscription);
		if(!empty($ret)) {
			$app->enqueueMessage(JText::_('SUBSCRIPTION_RENEW_CANCELED'));
		} else if($ret === false) {
			$app->enqueueMessage(JText::_('SUBSCRIPTION_RENEW_NOT_CANCELED'), 'error');
		}
		$app->redirect(hikaserial::completeLink('shop.subscription&task=listing', false, true));
	}

	protected function redirectToCheckout($cart_id) {
		$url = 'checkout&cart_id='.(int)$cart_id;

		global $Itemid;
		if(!empty($Itemid))
			$url .= '&Itemid=' . $Itemid;

		$url = hikashop_completeLink($url, false, true);
		$this->setRedirect($url);
	}
}
